  CREATE TABLE "HIERARCHIES"
   (	"HVER" NUMBER,
	"HTYPE" VARCHAR2(8) 
	    CONSTRAINT "HIERARCHIES_NN_HTYPE" NOT NULL ENABLE,
	"DESCR" VARCHAR2(128),
	"VALID_FROM" DATE,
	"VALID_TO" DATE,
	 CONSTRAINT "HIERARCHIES_PK" PRIMARY KEY ("HVER") ENABLE
   )
 ;

  CREATE TABLE "DEVICE_TYPES"
   (	"DPT_ID" NUMBER,
	"DPTYPE" VARCHAR2(64) 
	    CONSTRAINT "DEVICE_TYPES_NN_DPTYPE" NOT NULL ENABLE,
	"VALID_TO" DATE,
	 CONSTRAINT "DEVICE_TYPES_PK" PRIMARY KEY ("DPT_ID") ENABLE
   )
 ;

  CREATE TABLE "ITEMS"
   (	"HVER" NUMBER,
	"ID" NUMBER,
	"PARENT" NUMBER,
	"NAME" VARCHAR2(512) 
	    CONSTRAINT "ITEMS_NN_NAME" NOT NULL ENABLE,
	"TYPE" VARCHAR2(64) 
	    CONSTRAINT "ITEMS_NN_TYPE" NOT NULL ENABLE,
	"DETAIL" VARCHAR2(64),
	"DESCRIPTION" VARCHAR2(128),
	"DPID" NUMBER,
	"DPNAME" VARCHAR2(512),
	"DPT_ID" NUMBER,
	 CONSTRAINT "ITEMS_PK" PRIMARY KEY ("ID") ENABLE,
	 CONSTRAINT "ITEMS_FK_HVER" FOREIGN KEY ("HVER")
	  REFERENCES "HIERARCHIES" ("HVER") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "ITEMS_FK_PARRENT" FOREIGN KEY ("PARENT")
	  REFERENCES "ITEMS" ("ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "ITEMS_FK_DPTID" FOREIGN KEY ("DPT_ID")
	  REFERENCES "DEVICE_TYPES" ("DPT_ID") ENABLE
   )
 ;

  CREATE UNIQUE INDEX "ITEMS_DPNAME_IDX" ON "ITEMS" ("DPNAME");
  CREATE INDEX "ITEMS_PARENT_AND_ID" ON "ITEMS" ("PARENT", "ID");
  CREATE INDEX "ITEMS_PARENT_IDX" ON "ITEMS" ("PARENT");

  CREATE TABLE "RECIPES"
   (	"TAGID" NUMBER,
	"NAME" VARCHAR2(128) 
	    CONSTRAINT "RECIPES_NN_NAME" NOT NULL ENABLE,
	"HVER" NUMBER,
	"DESCRIPTION" VARCHAR2(128),
	"RECIPE_TYPE" VARCHAR2(128),
	 CONSTRAINT "RECIPES_PK" PRIMARY KEY ("TAGID") ENABLE,
	 CONSTRAINT "RECIPES_FK_HVER" FOREIGN KEY ("HVER")
	  REFERENCES "HIERARCHIES" ("HVER") ON DELETE CASCADE ENABLE
   )
 ;


  CREATE TABLE "RECIPE_VERSIONS"
   (	"RVER" NUMBER
	 CONSTRAINT "RECIPE_VERSIONS_NN_RVER" NOT NULL ENABLE,
	"DESCRIPTION" VARCHAR2(255),
	"USER_CREATED" VARCHAR2(16),
	"DATE_CREATED" DATE,
	 CONSTRAINT "RECIPE_VERSIONS_PK" PRIMARY KEY ("RVER") ENABLE
   )
 ;


  CREATE TABLE "RECIPE_DATA"
   (	"PROPID" NUMBER,
	"RVER" NUMBER 
	    CONSTRAINT "RECIPE_DATA_NN_RVER" NOT NULL ENABLE,
	"ID" NUMBER 
	    CONSTRAINT "RECIPE_DATA_NN_ID" NOT NULL ENABLE,
	"HAS_VALUE" NUMBER,
	"HAS_ALERT" NUMBER,
	"PROPVALUE" CLOB,
	"ALERT_TYPE" NUMBER,
	"ALERT_ACTIVE" NUMBER,
	"ALERT_CLASSES" VARCHAR2(256),
	"ALERT_TEXTS" VARCHAR2(1024),
	"ALERT_LIMITS" VARCHAR2(128),
	"DEVELEM_ID" NUMBER,
	 CONSTRAINT "RECIPE_DATA_PK" PRIMARY KEY ("PROPID") ENABLE,
	 CONSTRAINT "RECIPE_DATA_FK_ID" FOREIGN KEY ("ID")
	  REFERENCES "ITEMS" ("ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "RECIPE_DATA_FK_VERSION" FOREIGN KEY ("RVER")
	  REFERENCES "RECIPE_VERSIONS" ("RVER") ON DELETE CASCADE ENABLE
   )
 ;

  CREATE INDEX "RECIPE_DATA_DEV_IDX" ON "RECIPE_DATA" ("ID","DEVELEM_ID");

  CREATE TABLE "RECIPE_TAGS"
   (	"TAGID" NUMBER 
	    CONSTRAINT "RECIPE_TAGS_NN_TAGID" NOT NULL ENABLE,
	"PROPID" NUMBER 
	    CONSTRAINT "RECIPE_TAGS_NN_PROPID" NOT NULL ENABLE,
	"VALID_FROM" DATE 
	    CONSTRAINT "RECIPE_TAGS_NN_VALID_FROM" NOT NULL ENABLE,
	"VALID_TO" DATE,
	"RVER" NUMBER,
	 CONSTRAINT "RECIPE_TAGS_FK_PROPID" FOREIGN KEY ("PROPID")
	  REFERENCES "RECIPE_DATA" ("PROPID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "RECIPE_TAGS_UK" UNIQUE ("PROPID","TAGID","VALID_FROM") ENABLE,
	 CONSTRAINT "RECIPE_TAGS_RK_RVER" FOREIGN KEY ("RVER") REFERENCES "RECIPE_VERSIONS"("RVER") ON DELETE CASCADE ENABLE
   )
 ;

  CREATE INDEX "RECIPE_TAGS_PROPID_IDX" ON "RECIPE_TAGS"  ("PROPID");
  CREATE INDEX "RECIPE_TAGS_VALIDTO_IDX" ON "RECIPE_TAGS"  ("VALID_TO");



  CREATE TABLE "DEVICE_ELEMENTS"
   (	"DEVELEM_ID" NUMBER,
	"DPT_ID" NUMBER CONSTRAINT 
	    "DEVICE_ELEMENTS_NN_DPT_ID" NOT NULL ENABLE,
	"ELEMENTNAME" VARCHAR2(128) 
	    CONSTRAINT "DEVICE_ELEMENTS_NN_ELEMENTNAME" NOT NULL ENABLE,
	"ELEMENTTYPE" NUMBER 
	    CONSTRAINT "DEVICE_ELEMENTS_NN_ELEMENTTYPE" NOT NULL ENABLE,
	 CONSTRAINT "DEVICE_ELEMENTS_PK" PRIMARY KEY ("DEVELEM_ID") ENABLE,
	 CONSTRAINT "DEVICE_ELEMENTS_FK_DPT_ID" FOREIGN KEY ("DPT_ID")
	  REFERENCES "DEVICE_TYPES" ("DPT_ID") ON DELETE CASCADE ENABLE
   )
 ;


CREATE TABLE config_items_new
( "TAG_ID" NUMBER,
  "CITEM_ID" NUMBER,
  "ITEM_ID" NUMBER
    CONSTRAINT CONFIG_ITEMS_NEW_NN_ITEM_ID NOT NULL ENABLE,
  "DPT_ID" NUMBER
    CONSTRAINT CONFIG_ITEMS_NEW_NN_DPT_ID NOT NULL ENABLE,
  "VALID_FROM" DATE
    CONSTRAINT CONFIG_ITEMS_NEW_NN_VALID_FROM NOT NULL ENABLE,
  "VALID_TO" DATE,
  CONSTRAINT "CONFIG_ITEMS_NEW_PK" PRIMARY KEY ("CITEM_ID") ENABLE
) ;

CREATE UNIQUE INDEX "CONFIG_ITEMS_NEW_IDX1" ON "CONFIG_ITEMS_NEW"
(
  "TAG_ID",
  "VALID_TO",
  "ITEM_ID"
);




  CREATE TABLE "CONFIG_ITEMPROPS"
   (	"IPROP_ID" NUMBER,
	"CITEM_ID" NUMBER
	    CONSTRAINT "CONFIG_ITEMPROPS_NN_CITEM_ID" NOT NULL,
	"DEVELEM_ID" NUMBER
	    CONSTRAINT "CONFIG_ITEMPROPS_NN_DEVELEM_ID" NOT NULL,
	"DPEALIAS" VARCHAR2(256),
	 CONSTRAINT "CONFIG_ITEMPROPS_PK" PRIMARY KEY ("IPROP_ID") ENABLE,
	 CONSTRAINT "CONFIG_ITEMPROPS_FK_CITEM_ID" FOREIGN KEY ("CITEM_ID")
	  REFERENCES "CONFIG_ITEMS_NEW" ("CITEM_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_ITEMPROPS_FK_DEVELEM_ID" FOREIGN KEY ("DEVELEM_ID")
	  REFERENCES "DEVICE_ELEMENTS" ("DEVELEM_ID") ON DELETE CASCADE ENABLE
   );

  CREATE INDEX "CONFIG_ITEMPROPS_CITEM_ID" ON "CONFIG_ITEMPROPS"(citem_id);


   CREATE TABLE "CONFIG_VALUES"
     (	"IPROP_ID" NUMBER,
	"PVALUE" VARCHAR2(32),
	CONSTRAINT "CONFIG_VALUES_FK_IPROPID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE
     );

   create unique index "CONFIG_VALUES_IPROP" on CONFIG_VALUES (iprop_id);

   CREATE TABLE "CONFIG_VALUES_BIG"
     (	"IPROP_ID" NUMBER,
	"CVALUE" CLOB,
	CONSTRAINT "CONFIG_VALUESBIG_FK_IPROPID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE
     );

   create unique index "CONFIG_VALUESBIG_IPROP" on config_values_big(iprop_id);


  CREATE TABLE "CONFIG_DESC"
   (	"CONF_ID" NUMBER,
	"CONF_TAG" VARCHAR2(32)
	    CONSTRAINT "CONFIG_DESC_NN_CONF_TAG" NOT NULL ENABLE,
	"DESCRIPTION" VARCHAR2(255),
	"HVER" NUMBER,
	 CONSTRAINT "CONFIG_DESC_PK" PRIMARY KEY ("CONF_ID") ENABLE,
	 CONSTRAINT "CONFIG_DESC_FK_HVER" FOREIGN KEY ("HVER")
	  REFERENCES "HIERARCHIES" ("HVER") ON DELETE CASCADE ENABLE
   )
 ;

  CREATE UNIQUE INDEX "CONFIG_DESC_CONF_TAG_IDX" ON "CONFIG_DESC" ("CONF_TAG");

  CREATE TABLE "REFERENCES"
   (	"REFVER" NUMBER
	    CONSTRAINT "REFERENCES_NN_REFVER" NOT NULL ENABLE,
	"ID" NUMBER
	    CONSTRAINT "REFERENCES_NN_ID" NOT NULL ENABLE,
	"REF_ID" NUMBER
	    CONSTRAINT "REFERENCES_NN_REF_ID" NOT NULL ENABLE,
	"VALID_FROM" DATE,
	"VALID_TO" DATE,
	 CONSTRAINT "REFERENCES_FK_REFVER" FOREIGN KEY ("REFVER")
	  REFERENCES "CONFIG_DESC" ("CONF_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "REFERENCES_FK_ID" FOREIGN KEY ("ID")
	  REFERENCES "ITEMS" ("ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "REFERENCES_FK_REF_ID" FOREIGN KEY ("REF_ID")
	  REFERENCES "ITEMS" ("ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "REFERENCES_UK" UNIQUE ("REFVER","ID","VALID_FROM") ENABLE
   )
 ;


  CREATE TABLE "CONFIG_ADDRESSES"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_ADDRESSES_NN_IPROP_ID" NOT NULL ENABLE,
	"TYPE" VARCHAR2(16),
	"DRIVER" NUMBER,
	"DIRECTION" NUMBER,
	"DATATYPE" NUMBER,
	"ACTIVE" NUMBER,
	"OPTIONS" VARCHAR2(255),
	 CONSTRAINT "CONFIG_ADDRESSES_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_ADDRESSES_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;


  CREATE TABLE "CONFIG_ALERTS"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_ALERTS_NN_IPROP_ID" NOT NULL ENABLE,
	"TYPE" NUMBER,
	"ACTIVE" NUMBER,
	"DISCRETE" NUMBER, 
	"IMPULSE" NUMBER,
	 CONSTRAINT "CONFIG_ALERTS_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_ALERTS_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;




  CREATE TABLE "CONFIG_DPEALERTS"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_DPEALERTS_NN_IPROP_ID" NOT NULL ENABLE,
	"TEXTS" VARCHAR2(1024),
	"CLASSES" VARCHAR2(256),
	"LIMITS" VARCHAR2(128),
	"INCLUSIVE" VARCHAR2(40), 
	"NEGATED" VARCHAR2(40),
	 CONSTRAINT "CONFIG_DPEALERTS_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_DPEALERTS_UK" UNIQUE ("IPROP_ID") ENABLE	  
   )
 ;


  CREATE TABLE "CONFIG_SUMALERTS"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_SUMALERTS_NN_IPROP_ID" NOT NULL ENABLE,
	"TEXTS" VARCHAR2(128),
	"CLASS" VARCHAR2(64),
	"SUMDPELIST" VARCHAR2(4000),
	"THRESHOLD" NUMBER,
	 CONSTRAINT "CONFIG_SUMALERTS_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_SUMALERTS_UK" UNIQUE ("IPROP_ID") ENABLE	  
   )
 ;


  CREATE TABLE "CONFIG_ALERTS_MISC"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_ALERTS_MISC_NN_IPROP_ID" NOT NULL ENABLE,
	"PANEL" VARCHAR2(128),
	"PANELPARAMS" VARCHAR2(128),
	"HELP" VARCHAR2(128),
	"STATEBITS" VARCHAR2(64),
	"LIMITSSTATEBITS" VARCHAR2(1024),
	"TEXTSWENT" VARCHAR2(1024),
	 CONSTRAINT "CONFIG_ALERTS_MISC_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_ALERTS_MISC_UK" UNIQUE ("IPROP_ID") ENABLE	  
   )
 ;


  CREATE TABLE "CONFIG_ARCHIVING"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_ARCHIVING_NN_IPROP_ID" NOT NULL ENABLE,
	"CLASSNAME" VARCHAR2(32),
	"TYPE" NUMBER,
	"SMOOTHPROC" NUMBER,
	"DEADBAND" NUMBER,
	"TIMEINTVAL" NUMBER,
	"ACTIVE" NUMBER,
	 CONSTRAINT "CONFIG_ARCHIVING_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_ARCHIVING_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;


  CREATE TABLE "CONFIG_DPFUNCTIONS"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_DPFUNCTIONS_NN_IPROP_ID" NOT NULL ENABLE,
	"PARAMS" VARCHAR2(4000),
	"GLOBALS" VARCHAR2(1024),
	"DEFINITION" VARCHAR2(1024),
	"FUNCTYPE" NUMBER, 
	"STATFUNCTYPES" VARCHAR2(128), 
	"STATOPTS" VARCHAR2(64),
	 CONSTRAINT "CONFIG_DPFUNCTIONS_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_DPFUNCTIONS_UK" UNIQUE ("IPROP_ID") ENABLE	  
   )
 ;


  CREATE TABLE "CONFIG_CONVERSIONS"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_CONVERSIONS_NN_IPROP_ID" NOT NULL ENABLE,
	"TYPE" NUMBER,
	"CONVTYPE" NUMBER,
	"CONVORDER" NUMBER,
	"ARGUMENTS" VARCHAR2(64),
	 CONSTRAINT "CONFIG_CONVERSIONS_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_CONVERSIONS_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;


  CREATE TABLE "CONFIG_PVRANGES"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_PVRANGES_NN_IPROP_ID" NOT NULL ENABLE,
	"MINVALUE" NUMBER,
	"MAXVALUE" NUMBER,
	"NEGRANGE" NUMBER,
	"IGNOUTSIDE" NUMBER,
	"INCLMIN" NUMBER,
	"INCLMAX" NUMBER,
	"PVRVALS" VARCHAR2(512),
	"PVRTYPE" NUMBER,
	 CONSTRAINT "CONFIG_PVRANGES_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_PVRANGES_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;


  CREATE TABLE "CONFIG_SMOOTHING"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_SMOOTHING_NN_IPROP_ID" NOT NULL ENABLE,
	"PROC" NUMBER,
	"DEADBAND" NUMBER,
	"TIMEINTVAL" NUMBER,
	 CONSTRAINT "CONFIG_SMOOTHING_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_SMOOTHING_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;


  CREATE TABLE "CONFIG_UF"
   (	"IPROP_ID" NUMBER
	     CONSTRAINT "CONFIG_UF_NN_IPROP_ID" NOT NULL ENABLE,
	"UNIT" VARCHAR2(16),
	"FMT" VARCHAR2(16),
	 CONSTRAINT "CONFIG_UF_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_UF_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;

  CREATE TABLE "CONFIG_GENERAL"
   (	"IPROP_ID" NUMBER
	    CONSTRAINT "CONFIG_GENERAL_NN_IPROP_ID" NOT NULL ENABLE,
        "NUM" NUMBER(1)
    	    CONSTRAINT "CONFIG_GENERAL_NN_NUM" NOT NULL ENABLE,
	"WHAT" VARCHAR2(20)
	    CONSTRAINT "CONFIG_GENERAL_NN_WHAT" NOT NULL ENABLE,
	"DATA" VARCHAR2(4000),
	 CONSTRAINT "CONFIG_GENERAL_FK_IPROP_ID" FOREIGN KEY ("IPROP_ID")
	  REFERENCES "CONFIG_ITEMPROPS" ("IPROP_ID") ON DELETE CASCADE ENABLE,
	 CONSTRAINT "CONFIG_GENERAL_UK" UNIQUE ("IPROP_ID") ENABLE
   )
 ;





CREATE OR REPLACE VIEW v_recipesall
(
    propid
  , proptype
  , item_id
  , develem_id
  , recipe_ver
  , valid_from
  , valid_to
  , tag
  , recipe_type
  , recipe_comment
  , "HIERARCHY"
  , dpname
  , property
  , dptype
  , detail
  , elementtype
  , has_value
  , has_alert
  , propvalue
  , alert_type
  , alert_active
  , alert_classes
  , alert_texts
  , alert_limits
)
AS
SELECT
	rt.propid,
	de.elementtype,
	i.id,
	rd.develem_id,
	rd.rver,
	rt.valid_from,
	rt.valid_to,
	r.name,
	r.recipe_type,
	r.description,
	h.htype,
	i.dpname,
	de.elementname,
	i.type,
	i.detail,
	de.elementtype,
	rd.has_value,
	rd.has_alert,
	rd.propvalue,
	rd.alert_type,
	rd.alert_active,
	rd.alert_classes,
	rd.alert_texts,
	rd.alert_limits
    FROM recipe_tags rt, recipes r, recipe_data rd ,items i,hierarchies h, device_elements de
    WHERE 	rt.propid=rd.propid
	AND 	r.tagid=rt.tagid
	AND 	h.hver=i.hver
	AND 	i.id=rd.id
	AND 	de.develem_id=rd.develem_id(+)
 ;

CREATE OR REPLACE VIEW v_references
(
    item_id
  , ref_id
  , config_id
  , dpname
  , refdpname
  , config_tag
)
AS
select 
	r.id, 
	r.ref_id, 
	r.refver,
	i.dpname, 
	j.dpname, 
	c.conf_tag
    from items i, items j, references r, config_desc c
    where 	r.id=i.id 
	and 	r.ref_id=j.id 
	and	r.refver=c.conf_id 
	and 	r.valid_to is null
 ;


CREATE OR REPLACE VIEW v_config_items
(
    item_id
  , citem_id
  , dpt_id
  , name
  , dpname
  , dptype
  , tag
  , hver
  , description
  , "TYPE"
  , model
  , parent_id
  , dpid
)
AS
select 
	i.id, 
	c.citem_id, 
	t.dpt_id,
	i.name,
	i.dpname,
	t.dptype,
	d.conf_tag,
	i.hver,
	i.description,
	i.type,
	i.detail,
	i.parent,
	i.dpid
    from   items i, config_items_new c, device_types t, config_desc d
    where  	i.id=c.item_id 
	and 	t.dpt_id=c.dpt_id 
	and 	d.conf_id=c.tag_id
	and 	c.valid_to is null
 ;


CREATE OR REPLACE FORCE VIEW "V_DEVICE_CONFIGS" 
    (	"DPT_ID", 
	"DEVELEM_ID", 
	"DPTYPE", 
	"ELEMENTNAME", 
	"ELEMENTTYPE"
    ) AS select 
	d.dpt_id, 
	e.develem_id, 
	d.dpType, 
	e.elementName, 
	e.elementType
    from device_types d, device_elements e
    where 	d.dpt_id=e.dpt_id(+) 
	and 	d.valid_to is null
 ;

CREATE OR REPLACE FORCE VIEW "V_DEVICE_CONFIGS_ALL" 
    (	"DPT_ID", 
	"DEVELEM_ID", 
	"DPTYPE", 
	"ELEMENTNAME", 
	"ELEMENTTYPE"
    ) AS select 
	d.dpt_id, 
	e.develem_id, 
	d.dpType, 
	e.elementName, 
	e.elementType
    from device_types d, device_elements e
    where 	d.dpt_id=e.dpt_id(+) 
 ;

CREATE OR REPLACE FORCE VIEW v_item_properties
(
    citem_id
  , iprop_id
  , develem_id
  , item_dpname
  , dpttype
  , prop_name
  , dpe_name
  , dpe_type
  , value
  , tag
)
AS
select 
	c.citem_id, 
	c.iprop_id, 
	c.develem_id,
	i.dpname, 
	i.dptype, 
	p.elementname,
	i.dpname||p.elementname,
	p.elementtype,
	V.VALUE VAL,
	i.tag
    from config_itemprops c, v_config_items i, v_device_configs_all p,
    (
	SELECT IPROP_ID IPROP_ID, PVALUE VALUE FROM CONFIG_VALUES
	UNION ALL
	SELECT IPROP_ID IPROP_ID, CAST( cvalue AS VARCHAR2(4000)) VALUE FROM CONFIG_VALUES_BIG
 ) V
    where 	p.develem_id=c.develem_id 
	and 	i.citem_id=c.citem_id
	and c.iprop_id=v.iprop_id(+);

CREATE OR REPLACE VIEW v_config_alerts
(
    iprop_id
  , "TYPE"
  , active
  , discrete
  , impulse
  , texts
  , textswent
  , classes
  , limits
  , inclusive
  , negated
  , sumdpelist
  , threshold
  , statebits
  , limitsstatebits
  , panel
  , panelparams
  , help
)
AS
select
	al.iprop_id,
	al.type,
	al.active,
	al.discrete,
	al.impulse,
	CASE al.type WHEN 59 THEN s.texts ELSE a.texts END,
	m.textswent,
	CASE al.type WHEN 59 THEN s.class ELSE a.classes END,
	a.limits,
	a.inclusive,
	a.negated,
	s.sumdpelist,
	s.threshold,
	m.statebits,
	m.limitsstatebits,
	m.panel,
	m.panelParams,
	m.help
    from config_alerts al, config_dpealerts a, config_sumalerts s, config_alerts_misc m
    where 	a.iprop_id(+)=al.iprop_id
	and	s.iprop_id(+)=al.iprop_id
	and  m.iprop_id(+)=al.iprop_id;

   CREATE SEQUENCE  "CITEM_ID_SEQ"   INCREMENT BY 1 START WITH 1 CACHE 10 NOORDER  NOCYCLE;
   CREATE SEQUENCE  "CONF_ID_SEQ"  INCREMENT BY 1 START WITH 1 NOCACHE  NOORDER  NOCYCLE;
   CREATE SEQUENCE  "DEVELEM_ID_SEQ"  INCREMENT BY 1 START WITH 1 CACHE 10 NOORDER  NOCYCLE;
   CREATE SEQUENCE  "DPT_ID_SEQ"  INCREMENT BY 1 START WITH 1 NOCACHE  NOORDER  NOCYCLE;
   CREATE SEQUENCE  "IPROP_ID_SEQ"  INCREMENT BY 1 START WITH 1 CACHE 100 NOORDER  NOCYCLE;
   CREATE SEQUENCE  "ITEMS_ID_SEQ"  INCREMENT BY 1 START WITH 1 CACHE 10 NOORDER  NOCYCLE;
   CREATE SEQUENCE  "RECIPES_RVER_SEQ"  INCREMENT BY 1 START WITH 1 NOCACHE  NOORDER  NOCYCLE;
   CREATE SEQUENCE  "RECIPES_TAGID_SEQ"  INCREMENT BY 1 START WITH 1 NOCACHE  NOORDER  NOCYCLE;
   CREATE SEQUENCE  "RECIPE_DATA_PROPID_SEQ"  INCREMENT BY 1 START WITH 1 CACHE 10 NOORDER  NOCYCLE;


  CREATE GLOBAL TEMPORARY TABLE "CDB_API_PARAMS"
   (	"I1" NUMBER,
	"I2" NUMBER,
	"I3" NUMBER,
	"I4" NUMBER,
	"I5" NUMBER,
	"I6" NUMBER,
	"I7" NUMBER,
	"I8" NUMBER,
	"S1" VARCHAR2(512),
	"S2" VARCHAR2(512),
	"S3" VARCHAR2(256),
	"S4" VARCHAR2(256),
	"S5" VARCHAR2(256),
	"S6" VARCHAR2(256),
	"S7" VARCHAR2(4000),
	"S8" VARCHAR2(4000),
	"S9" VARCHAR2(16),
	"S10" VARCHAR2(16),
	"D1" DATE,
	"D2" DATE,
	"C1" CLOB
   ) ON COMMIT PRESERVE ROWS
 ;

CREATE GLOBAL TEMPORARY TABLE "TMP_ITEM_PROPS"
   (    "IPROP_ID" NUMBER,
        "DPENAME" VARCHAR2(512)
   ) ON COMMIT PRESERVE ROWS;

create unique index tip_dpe_idx    on TMP_ITEM_PROPS(DPEName);
CREATE UNIQUE INDEX tip_ipropd_idx ON tmp_item_props(iprop_id);

