/**@name LIBRARY: unCoreLHCServices.ctl

@author: Frederic BERNARD (AB-CO)

Creation Date: 23/04/2004

Modification History: 
	07/10/2005 Celine
	. add CMWServer functions
	03/03/2005 Cline
	. modify g_diLoggingCritical_Alarm
	27/01/2005 Cline
	. add systemName to allow working in distributed mode 
	
version 1.0

External Functions: 
	
Internal Functions: 
	. unCoreLHCServices_ClosePanel
	. unCoreLHCServices_initialize
	. unCoreLHCServices_checkManagerNumber
	. unCoreLHCServices_getLHCLoggingManager
	. unCoreLHCServices_getLaserManager
	. unCoreLHCServices_getCMWServerManager
	. unCoreLHCServices_getCMWClientDriver
	. unCoreLHCServices_PVSS
	. unCoreLHCServices_checkUnicosDevice	
	. unCoreLHCServices_set_SystemName
		
Purpose: 
	Utility library for the LHCLogging and Laser componenst.
	 	
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. const variable: the following constantes are global to the componemts
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

//--------------------------------------------------------------------------------------------------------------------------------


// constant declaration
//------------------
const int c_Ok=10000;
const int c_Error=200;
const int c_Warning=100;
const int c_Stopped=1000;
const int c_Blocked=1200;
const int c_Timeout=2000;
const int c_ARCHIVE=0;
const int c_MANAGER=1;
const int c_Min_Rate=30; 	 		//minimun time (sec) before checking man_Nun
const int c_Timeout_DpeStats=60;	//Timeout in secondes to update _ValueArchive_*.statistics.dpElements

const string c_Logging_Pattern="LHCLogging_Manager_";
const string c_Laser_Pattern="Laser_Manager_";
const string c_CMWServer_Pattern="CMWServer_Manager_";

const string c_Activity="CRYO";

// global declaration
//------------------
dyn_string g_diLoggingCritical_Alarm;
dyn_string g_diLaserCritical_Alarm;
dyn_string g_diCMWServerCritical_Alarm;

//--------------------------------------------------------------------------------------------------------------------------------
// unCoreLHCServices_initialize
/**
Purpose:
Initialize Globale Variables for Logging and Laser (They are used by Panel and System Integrity)

	@param sComponent: string, input, the component,

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCoreLHCServices_initialize(string sComponent)
{
	switch(sComponent)
		{
		case c_Logging_Pattern:
			//these Values = Errors 
			g_diLoggingCritical_Alarm=makeDynInt(10200,10210,10211,10212,10213,10214,10215,10250,10251,10252,10253,10260,10360,10361,10362);
			break;
		
		case c_Laser_Pattern:
			g_diLaserCritical_Alarm=makeDynInt(1000,10003,10210,10211,10213,10214,10255,10256,10254);
			break;
		case c_CMWServer_Pattern:
			g_diCMWServerCritical_Alarm=makeDynInt(10210,10211,10212,10213,10214,10215,10255,10360,10361,10362);
			break;
		}
				
}

// unCoreLHCServices_checkManagerNumber
/**
Purpose: check if Manager Number is free (cf. Logging and Laser = API)

Parameter: 	iNum, int, input, Manager Number to check
						sSystemName, string, input, Name of the system
				
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
bool unCoreLHCServices_checkManagerNumber(int iNum, string sSystemName)
{
dyn_int dsNum;
dyn_string dsManagers;
int len, i, iMan_Num;
string sMan_Num;


//Logging
unCoreLHCServices_getLHCLoggingManager(dsManagers, sSystemName);
len=dynlen(dsManagers);
for (i=1;i<=len;i++)
	{
	sMan_Num=substr(dsManagers[i],strpos(dsManagers[i],c_Logging_Pattern) + strlen(c_Logging_Pattern));
	sscanf(sMan_Num,"%d",iMan_Num);
	if (iMan_Num>0)
		dynAppend(dsNum, iMan_Num);
	else
		return false;
	}

//Laser
dynClear(dsManagers);
unCoreLHCServices_getLaserManager(dsManagers, sSystemName);
len=dynlen(dsManagers);
for (i=1;i<=len;i++)
	{	
	sMan_Num=substr(dsManagers[i],strpos(dsManagers[i],c_Laser_Pattern) + strlen(c_Laser_Pattern));
	sscanf(sMan_Num,"%d",iMan_Num);
	if (iMan_Num>0 && dynContains(dsNum,iMan_Num)==0)
		dynAppend(dsNum, iMan_Num);
	else
		return false;
	}

//CMWServer
dynClear(dsManagers);
unCoreLHCServices_getCMWServerManager(dsManagers, sSystemName);
len=dynlen(dsManagers);
for (i=1;i<=len;i++)
	{	
	sMan_Num=substr(dsManagers[i],strpos(dsManagers[i],c_CMWServer_Pattern) + strlen(c_CMWServer_Pattern));
	sscanf(sMan_Num,"%d",iMan_Num);
	if (iMan_Num>0 && dynContains(dsNum,iMan_Num)==0)
		dynAppend(dsNum, iMan_Num);
	else
		return false;
	}

//Check
if (dynContains(dsNum,iNum)==0)
	return true;
else
	return false;
}

// unCoreLHCServices_getLHCLoggingManager
/**
Purpose: return the LHCLogging manager name.

Parameter: dsLHCLoggingManager, dyn_string, output, result of the function
						sSystemName, string, input, Name of the system
	
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCoreLHCServices_getLHCLoggingManager(dyn_string &dsLHCLoggingManager, string sSystemName)
{
dyn_string dsTemp;
int i, length;

	dsTemp=dpTypes("_Un_LHCLogging_Manager*",getSystemId(sSystemName));
	if (dynlen(dsTemp)>0)
		{
		dynClear(dsTemp);
		dsTemp = dpNames(sSystemName+"*","_Un_LHCLogging_Manager");	// Get DP
		length = dynlen(dsTemp);
		for(i=1;i<=length;i++)					// For each DP, delete system name
			{
			dsTemp[i] = substr(dsTemp[i], strlen(sSystemName));
			}
		dynUnique(dsTemp);
		dynSortAsc(dsTemp);
		}

	dsLHCLoggingManager = dsTemp;
}

// unCoreLHCServices_getLaserManager
/**
Purpose: return the Laser manager name.

Parameter: dsLaserManager, dyn_string, output, result of the function
						sSystemName, string, input, Name of the system
							
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCoreLHCServices_getLaserManager(dyn_string &dsLaserManager, string sSystemName)
{
	dyn_string dsTemp;
	int i, length;
	
	dsTemp=dpTypes("_Un_Laser_Manager*",getSystemId(sSystemName));

	if (dynlen(dsTemp)>0)
		{
		dynClear(dsTemp);
		dsTemp = dpNames(sSystemName+"*","_Un_Laser_Manager");	// Get DP
		length = dynlen(dsTemp);
		for(i=1;i<=length;i++)					// For each DP, delete system name
			{
			dsTemp[i] = substr(dsTemp[i], strlen(sSystemName));
			}
		dynUnique(dsTemp);
		dynSortAsc(dsTemp);
		}
		
	dsLaserManager = dsTemp;
}

// unCoreLHCServices_getCMWServerManager
/**
Purpose: return the CMWServer manager name.

Parameter: dsCMWServerManager, dyn_string, output, result of the function
						sSystemName, string, input, Name of the system
							
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCoreLHCServices_getCMWServerManager(dyn_string &dsCMWServerManager, string sSystemName)
{
	dyn_string dsTemp;
	int i, length;
	
	dsTemp=dpTypes("_Un_CMWServer_Manager*",getSystemId(sSystemName));

	if (dynlen(dsTemp)>0)
		{
		dynClear(dsTemp);
		dsTemp = dpNames(sSystemName+"*","_Un_CMWServer_Manager");	// Get DP
		length = dynlen(dsTemp);
		for(i=1;i<=length;i++)					// For each DP, delete system name
			{
			dsTemp[i] = substr(dsTemp[i], strlen(sSystemName));
			}
		dynUnique(dsTemp);
		dynSortAsc(dsTemp);
		}
		
	dsCMWServerManager = dsTemp;
}
// unCoreLHCServices_getCMWClientDriver
/**
Purpose: return the CMWClient driver name.

Parameter: dsCMWClientDriver, dyn_string, output, result of the function
						sSystemName, string, input, Name of the system
							
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCoreLHCServices_getCMWClientDriver(dyn_string &dsCMWClientDriver, string sSystemName)
{
	dyn_string dsTemp;
	int i, length;
	
	dsTemp=dpTypes("_Un_CMWClient_Driver*",getSystemId(sSystemName));

	if (dynlen(dsTemp)>0)
		{
		dynClear(dsTemp);
		dsTemp = dpNames(sSystemName+"*","_Un_CMWClient_Driver");	// Get DP
		length = dynlen(dsTemp);
		for(i=1;i<=length;i++)					// For each DP, delete system name
			{
			dsTemp[i] = substr(dsTemp[i], strlen(sSystemName));
			}
		dynUnique(dsTemp);
		dynSortAsc(dsTemp);
		}
		
	dsCMWClientDriver = dsTemp;
}
// unCoreLHCServices_PVSS
/**
Purpose: PVSS status in Console

Parameter: 	iState, int, input, State with int
			sState, string, output, Message text for state
						
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCoreLHCServices_PVSS(int iState, string &sState)
{
switch(iState)
	{
	case 0:
	sState="Stopped";
	break;
	
	case 1:
	sState="Started";
	break;
	
	case 2:
	sState="Blocked";
	break;
	
	default:
	sState="Error";
	}
}


// unCoreLHCServices_checkUnicosDevice
/**
Purpose: check if the component UnicosObject is installed

Parameter: 	sPatern, string, input, Patern
			bResult, bool, input, result of the search
						
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCoreLHCServices_checkUnicosDevice(string sPatern, bool &bResult)
{
dyn_string dsDp;
 
dsDp=dpTypes();
if (dynContains(dsDp,"_UnObjects"))
	{
	dsDp =dpNames(sPatern, "_UnObjects");
	if (dynlen(dsDp)>0)
		{
		if (isFunctionDefined("unGenericButtonFunctionsHMI_closePanel"))
			bResult=true;
		else
			bResult=false;
		}
	else
		bResult=false;
	}
else
	bResult=false;
}

// unCoreLHCServices_set_SystemName
/**
Purpose: Manage System Name in Dp

Parameter: 	sDataPoint, string, output, Dpe name
			bState, bool, output, Change done
			bSystenName, bool, input, Need or not SystemName 
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCoreLHCServices_set_SystemName(string &sDataPoint, bool &bState, bool bSystenName)
{
string sSystemName, sSystem_defined;
int iPos;

sSystemName=getSystemName();
iPos=strpos(sDataPoint, ":");

bState=false;
if ((bSystenName) && (iPos<0))		//Need SystemName
	{
	sDataPoint=sSystemName+sDataPoint;
	bState=true;
	}
	
else
	{
	if ((!bSystenName) && (iPos>=0))	//Remove SystemName
		{
		sSystem_defined = unGenericDpFunctions_getSystemName(sDataPoint);
		sDataPoint=substr(sDataPoint, strlen(sSystem_defined), strlen(sDataPoint));
		bState=true;
		}
	}
}


// unCoreLHCServices_ClosePanel
/**
Purpose: check if the system can close the panel in the case of Unicos component

Parameter: 	sName, string, input, myModuleName()+"/"+myPanelName()
						
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCoreLHCServices_ClosePanel(string sName)
{
int pos;
	
	if(globalExists("g_unGraphicalFrame_childOpenedPanel")) {
		pos = dynContains(g_unGraphicalFrame_childOpenedPanel, sName);
		if(pos >0)
			dynRemove(g_unGraphicalFrame_childOpenedPanel, pos);
	}
	unGenericButtonFunctionsHMI_closePanel(true);
}

//@}

