/**@name LIBRARY: unLHCLogging.ctl

@author: Frdric BERNARD (AB-CO-IS)

Creation Date: 25/09/2003 

Modification History: 
	18/01/2008: Frederic
		- new function to know number of items per archives in the file: unLHCLogging_getArchiveItems()
		
	29/08/2007: Herve
		- dpe man_ShowStatistics string instead of int
		
	02/04/2007: Herve
		- in unLHCLogging_Configuration: set the limits
		
	02/03/2007: Herve
		- in unLHCLogging_Configuration: set the limits
		
	07/02/2006: Herve
		- in unLHCLogging_Integrity check if <= instead of ==
		
	29/09/2005 Celine
		- remove systemName in unLHCLogging_Integrity
	
	27/09/2005 Celine
		- use ctrlConfRead to check the config file
	
	18/05/2005 Celine
		- modify unLHCLogging_CheckLine and unLHCLogging_getFromFile : add alias 

	18/04/2005 Cline
	 	- modify settings: 
	 		. rename man_FileSize to tcp_FileSize

	03/03/2005 Cline
	 - modify settings: 
	 		. add	tcp_Timeout, sql_AlwaysUsePeriod, arch_TotalDpe, man_FileSize
			. remove : tcp_RecvTimeout, tcp_SendTimeout, man_instance, tcp_QueryLocalPort, sql_Window, Copy.arch_OperationMode

	27/01/2005 Cline
   - add systemName in unLHCLogging_Configuration to allow working in distributed mode

	23/04/2004 Frederic:
	 - move functions to unCoreLHCServices.ctl (Common to Laser and LHCLogging components)

version 1.1

External Functions: 
	
Internal Functions: 
	
Purpose: 
This library contains generic functions for LHCLogging component.

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL), UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.

	Functions:
		unLHCLogging_DefaultSettingsManager
		unLHCLogging_LoadSettingsManager
		unLHCLogging_DefaultSettingsArchive
		unLHCLogging_LoadSettingsArchive
		unLHCLogging_getStatus
		unLHCLogging_StateArchives
		unLHCLogging_Configuration
		unLHCLogging_Check_Delay_Archive
		unLHCLogging_Calculate_Delay_Archive
		unLHCLogging_Save_Delay_Archive
		unLHCLogging_Integrity
		unLHCLogging_Check_Integrity
		unLHCLogging_Update_PVSS_Status
		unLHCLogging_SaveState
		unLHCLogging_GetPosition
		unLHCLogging_getFromFile
		unLHCLogging_CheckLine
		unLHCLogging_setStatusTable
		_unSystemIntegrity_LHCLogging_CB	
*/

//@{

//LHCLoggingManager
const int LOGGING_MAN_SQL_DATATTRIBUTES=1;
const int LOGGING_MAN_CONFIGPATH=2;
const int LOGGING_MAN_TCP_LOCALHOST=3;
const int LOGGING_MAN_TCP_REMOTEHOST=4;
const int LOGGING_MAN_TCP_DATAPATH=5;
const int LOGGING_MAN_TCP_METADATAPATH=6;
const int LOGGING_MAN_SQL_TIMEOUT=7;
const int LOGGING_MAN_SHOWSTATISTICS=8;
const int LOGGING_MAN_TCP_REMOTEPORT=9;
const int LOGGING_MAN_TCP_TIMEOUT=10;
const int LOGGING_MAN_PERIOD=11;
const int LOGGING_MAN_TCP_FILESIZE=12;
const int LOGGING_MAN_TCP_DATASCHEMA=13;
const int LOGGING_MAN_TCP_METADATASCHEMA=14;
const int LOGGING_MAN_BEHIND_BACKUP=15;
const int LOGGING_MAN_BEHIND_QUERY=16;
const int LOGGING_MAN_WILDCHARS = 17;
const int LOGGING_MAN_MSEC_RESOLUTION = 18;

//LHCloggingArchive
const int LOGGING_ARCH_SQL_PERIOD=1;
const int LOGGING_ARCH_SQL_MAXDPENUMBER=2;
const int LOGGING_ARCH_MAXCPUUSAGE=3;
const int LOGGING_ARCH_SQL_ALWAYSUSEPERIOD=4;
const int LOGGING_ARCH_DATAPATH=5;
const int LOGGING_ARCH_TOTALDPE=6;

//LHCLogging State
const int State_Command=1;
const int State_StateOut=2;
const int State_Message=3;
const int State_PVSS=4;
const int State_Timeout=5;
const int State_LastRefEnd=6;

//Symbol positions
const int g_iArchiveSettings_X=610;
const int g_iArchiveSettings_Y=300;
const int g_iManagerSettings_X=10;
const int g_iManagerSettings_Y=320;
const int g_iArchiveStatus_X=0;
const int g_iArchiveStatus_Y=180;

//Catalogue name
const string g_sCat_LHCLogging="unLHCLogging";

//user allowed to modify the LHCLogging if Unicos Package not installed
const string c_LHCLogging_userAdmin="root"; 

// unLHCLogging_DefaultSettingsManager

/**
Purpose: set the default settings for Manager DP

Parameter: 	sDP, string, input, DataPoint Name
			exceptionInfo, dyn string, output, Message

Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_DefaultSettingsManager(string sDP, dyn_string &exceptionInfo)
{
int iRes, iTimeout, iDelay;
dyn_string dsSettings;
string sFunction, sName, sSystem;

sSystem = unGenericDpFunctions_getSystemName(sDP);
unLHCLogging_Configuration(c_MANAGER, sSystem, dsSettings, sFunction, sName, iTimeout, iDelay, exceptionInfo);
if (dynlen(exceptionInfo)==0)
	{
	if (dpExists(sDP) && dynlen(dsSettings)>=LOGGING_MAN_TCP_TIMEOUT)
			{
			if(dsSettings[LOGGING_MAN_SHOWSTATISTICS] == "")
				dsSettings[LOGGING_MAN_SHOWSTATISTICS] = "0x0";
			iRes=dpSet(sDP + ".Settings.sql_DataAttributes", dsSettings[LOGGING_MAN_SQL_DATATTRIBUTES],
						sDP + ".Settings.man_ConfigPath", dsSettings[LOGGING_MAN_CONFIGPATH],
						sDP + ".Settings.tcp_LocalHost", dsSettings[LOGGING_MAN_TCP_LOCALHOST],
						sDP + ".Settings.tcp_RemoteHost", dsSettings[LOGGING_MAN_TCP_REMOTEHOST],
						sDP + ".Settings.tcp_DataPath", dsSettings[LOGGING_MAN_TCP_DATAPATH],
						sDP + ".Settings.tcp_MetadataPath", dsSettings[LOGGING_MAN_TCP_METADATAPATH],
						sDP + ".Settings.sql_Timeout", dsSettings[LOGGING_MAN_SQL_TIMEOUT],		
						sDP + ".Settings.man_ShowStatistics", dsSettings[LOGGING_MAN_SHOWSTATISTICS],
						sDP + ".Settings.tcp_RemotePort", dsSettings[LOGGING_MAN_TCP_REMOTEPORT],
						sDP + ".Settings.tcp_Timeout", dsSettings[LOGGING_MAN_TCP_TIMEOUT],
						sDP + ".Settings.man_NumPeriod",dsSettings[LOGGING_MAN_PERIOD],
						sDP + ".Settings.tcp_FileSize",dsSettings[LOGGING_MAN_TCP_FILESIZE],
						sDP + ".Settings.tcp_DataSchema",strsplit(dsSettings[LOGGING_MAN_TCP_DATASCHEMA],","),
						sDP + ".Settings.tcp_MetadataSchema",strsplit(dsSettings[LOGGING_MAN_TCP_METADATASCHEMA],","),
						sDP + ".Settings.sql_BehindBackup", dsSettings[LOGGING_MAN_BEHIND_BACKUP],
						sDP + ".Settings.sql_BehindQuery", dsSettings[LOGGING_MAN_BEHIND_QUERY],
						sDP + ".Settings.man_WildChars", dsSettings[LOGGING_MAN_WILDCHARS],
						sDP + ".Settings.man_MilliResol", dsSettings[LOGGING_MAN_MSEC_RESOLUTION],
						sDP + ".State.st_CommandIn", 100,				//Stop
						sDP + ".State.st_StateOut", 1000,			//Unknown state
						sDP + ".State.ev_MessageOut", c_Ok);		//OK
						
			}
	else
		fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_DefaultSettingsManager(): " + getCatStr(g_sCat_LHCLogging,"DPDOESNOTEXIST"),"");
	}
}

// unLHCLogging_LoadSettingsManager
/**
Purpose: Get  the settings for Manager

Parameter: 	sDP, string, input, DataPoint Name
			dsParam, dyn string, output, String Parameters
			ddiParam, dyn dyn int, output, Instance Parameter
			exceptionInfo, dyn string, output, Message
			
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unLHCLogging_LoadSettingsManager(string sDP, dyn_string &dsParam, dyn_dyn_int &ddiParam, dyn_string &exceptionInfo)
{
int iRes, i, length;

if (dpExists(sDP))
	{
	iRes=dpGet(sDP + ".Settings.sql_DataAttributes", dsParam[LOGGING_MAN_SQL_DATATTRIBUTES],
				sDP + ".Settings.man_ConfigPath", dsParam[LOGGING_MAN_CONFIGPATH],
				sDP + ".Settings.tcp_LocalHost", dsParam[LOGGING_MAN_TCP_LOCALHOST],
				sDP + ".Settings.tcp_RemoteHost", dsParam[LOGGING_MAN_TCP_REMOTEHOST],
				sDP + ".Settings.tcp_DataPath", dsParam[LOGGING_MAN_TCP_DATAPATH],
				sDP + ".Settings.tcp_MetadataPath", dsParam[LOGGING_MAN_TCP_METADATAPATH],
				sDP + ".Settings.sql_Timeout", dsParam[LOGGING_MAN_SQL_TIMEOUT],
				sDP + ".Settings.man_ShowStatistics", dsParam[LOGGING_MAN_SHOWSTATISTICS],
				sDP + ".Settings.tcp_RemotePort", dsParam[LOGGING_MAN_TCP_REMOTEPORT],
				sDP + ".Settings.tcp_Timeout", dsParam[LOGGING_MAN_TCP_TIMEOUT],				
				sDP + ".Settings.man_NumPeriod", dsParam[LOGGING_MAN_PERIOD],
				sDP + ".Settings.tcp_FileSize",dsParam[LOGGING_MAN_TCP_FILESIZE],
				sDP + ".Settings.tcp_DataSchema",dsParam[LOGGING_MAN_TCP_DATASCHEMA],
				sDP + ".Settings.tcp_MetadataSchema",dsParam[LOGGING_MAN_TCP_METADATASCHEMA],
				sDP + ".Settings.sql_BehindBackup", dsParam[LOGGING_MAN_BEHIND_BACKUP],
				sDP + ".Settings.sql_BehindQuery", dsParam[LOGGING_MAN_BEHIND_QUERY],
				sDP + ".Settings.man_WildChars", dsParam[LOGGING_MAN_WILDCHARS],
				sDP + ".Settings.man_MilliResol", dsParam[LOGGING_MAN_MSEC_RESOLUTION]);
		if(dsParam[LOGGING_MAN_SHOWSTATISTICS] == "")
			dsParam[LOGGING_MAN_SHOWSTATISTICS] = "0x0";
	//check data
	length=dynlen(dsParam);
	for (i=1;i<=length;i++)
		{
		if (dsParam[i]=="")
			{
			dsParam[i]="-1";
			}
		}
	}
else
	fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_LoadSettingsManager(): " + getCatStr(g_sCat_LHCLogging,"DPDOESNOTEXIST"),"");
	
}


// unLHCLogging_DefaultSettingsArchive
/**
Purpose: set the default settings for Manager DP

Parameter: 	sDP, string, input, DataPoint Name
						exceptionInfo, dyn string, output, Message
			
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_DefaultSettingsArchive(string sDP,dyn_string &exceptionInfo)
{
int iRes, iTimeout, iDelay;
dyn_string dsSettings;
string sFunction, sName, sSystem;

sSystem = unGenericDpFunctions_getSystemName(sDP);
unLHCLogging_Configuration(c_ARCHIVE, sSystem, dsSettings, sFunction, sName, iTimeout, iDelay, exceptionInfo);
if (dynlen(exceptionInfo)==0)
	{
	if (dpExists(sDP) && dynlen(dsSettings)>=5)
		{
		iRes=dpSet(	sDP + ".Settings.sql_Period", dsSettings[LOGGING_ARCH_SQL_PERIOD],
					sDP + ".Settings.sql_MaxDpeNumber", dsSettings[LOGGING_ARCH_SQL_MAXDPENUMBER],
					sDP + ".Settings.arch_MaxCpuUsage", dsSettings[LOGGING_ARCH_MAXCPUUSAGE],
					sDP + ".Settings.sql_AlwaysUsePeriod", dsSettings[LOGGING_ARCH_SQL_ALWAYSUSEPERIOD],
					sDP + ".Settings.arch_DataPath", dsSettings[LOGGING_ARCH_DATAPATH],
					sDP + ".Settings.arch_LastDpe", -1,
					sDP + ".State.st_CommandIn", 100,
					sDP + ".State.st_StateOut", 1000,
					sDP + ".State.ev_MessageOut", c_Ok);

		}
	else
		fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_DefaultSettingsArchive(): " + getCatStr(g_sCat_LHCLogging,"DPDOESNOTEXIST"),"");
	}
}

// unLHCLogging_LoadSettingsArchive
/**
Purpose: Get  the settings for Manager

Parameter: 	sDP, string, input, DataPoint Name
			dsParam, dyn string, output, String Parameters
			exceptionInfo, dyn string, output, Message
			
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unLHCLogging_LoadSettingsArchive(string sDP, dyn_string &dsParam, dyn_string &exceptionInfo)
{
int iRes, i, length;

if (dpExists(sDP + ".Settings"))
	{
	iRes=dpGet(sDP + ".Settings.sql_Period", dsParam[LOGGING_ARCH_SQL_PERIOD],
		sDP + ".Settings.sql_MaxDpeNumber", dsParam[LOGGING_ARCH_SQL_MAXDPENUMBER],
		sDP + ".Settings.arch_MaxCpuUsage", dsParam[LOGGING_ARCH_MAXCPUUSAGE],
		sDP + ".Settings.sql_AlwaysUsePeriod", dsParam[LOGGING_ARCH_SQL_ALWAYSUSEPERIOD],
		sDP + ".Settings.arch_DataPath",dsParam[LOGGING_ARCH_DATAPATH],
		sDP + ".Settings.arch_TotalDpe",dsParam[LOGGING_ARCH_TOTALDPE]);
			
	//check data
	length=dynlen(dsParam);
	for (i=1;i<=length;i++)
		{
		if (dsParam[i]=="")
			{
			dsParam[i]="-1";
			}
		}
	}
else
	fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_LoadSettingsArchive:" + getCatStr(g_sCat_LHCLogging,"DPDOESNOTEXIST"),"");
}


// unLHCLogging_getStatus
/**
Purpose: get state of LHCLogging system

Parameter: 	iIndex, int, input, Position in Array  
			iState, int, output, State of Manager or Archive  
						
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unLHCLogging_getStatus(int iIndex, int &iState)
{
int iState_1=0, iState_2=0, iState_3=0;

iState=0;
//DebugN("getStatus",iIndex );
	if (dynlen(g_diLogging_PVSS)>=iIndex && dynlen(g_diLogging_Message)>=iIndex && dynlen(g_dbLogging_Alarm_Timeout)>=iIndex && iIndex>0) //check of dynlen
		{
//		DebugN("g_diLogging_PVSS,g_diLogging_Message,iIndex,g_dbLogging_Alarm_Timeout",g_diLogging_PVSS,g_diLogging_Message,iIndex,g_dbLogging_Alarm_Timeout);
		//PVSS
		if ((g_diLogging_PVSS[iIndex]!=1)) 														//Archive or Manager Stopped or Blocked...
			iState_1=1;
	
		//Message
		if ((g_diLogging_Message[iIndex]>10001) && (g_diLogging_Message[iIndex]!=10150))		//10150 -> Manager Internal Purpose
			{
			iState_2=g_diLogging_Message[iIndex];		
			}
		
		//Timeout
		if (g_dbLogging_Alarm_Timeout[iIndex])		
			iState_3=c_Timeout;																		//Warning																									
		}
	else
		{
		iState_1=1;
		}
	
	if (dynContains(g_diLoggingCritical_Alarm,iState_2)>0 || (iState_1==1))	//Error(s)
		iState=c_Error;
	else
		{
		if (iState_3==c_Timeout || iState_2>0)	
			iState=c_Warning;
		}
//		DebugN("iState",iState);
}

// unLHCLogging_StateArchives
/**
Purpose: set Dpe to know the Archive status for one Manager

Parameter: 	sArchive, string, input, Archive Name
			iState, int, input, state for this archive
						
Usage: Public

PVSS manager usage: Ctrl, UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
**/

unLHCLogging_StateArchives(string sArchive, int iIndex, int iState)
{
dyn_string dsLHCLoggingManager, exceptionInfo;
int iRes, i, iArchiveState, len, iIndex_man;
string sDatapoint, sManager, sSystem;
bool bWarning;

	if (iIndex>0)
		{
		//1. Status for one archive
		if (iState!=0) 				//=100 or 200
			{
			if (g_dbLogging_Alarm_Timeout[iIndex] && iState!=c_Error)	//=Alarm Timeout
				iArchiveState=c_Timeout;
			else													//=Error	
				{
				if (g_diLogging_Message[iIndex]!=c_Ok)
					iArchiveState=g_diLogging_Message[iIndex];
				else
					iArchiveState=c_Stopped;						//=Archive stopped			
				}
			}
		else
			iArchiveState=c_Ok;										//=OK
			
		g_diLogging_GlobalState[iIndex]=iArchiveState;
	
		//2. Global Status for the archives linked with one Manager	
		sSystem = unGenericDpFunctions_getSystemName(sArchive);
		sManager=substr(sArchive,strpos(sArchive, c_Logging_Pattern),strlen( c_Logging_Pattern) + 2);
		sDatapoint=sSystem + sManager + ".State.st_Archives";

		if (dpExists(sDatapoint))
			{
			len=dynlen(g_dsLogging_Name);
			
			for(i=1;i<=len;i++)										//For each Manager and linked Archive
				{
				if (strpos(g_dsLogging_Name[i],sManager)>=0)	
					{
					if (g_diLogging_GlobalState[i]!=0)
						{
						if(dynContains(g_diLoggingCritical_Alarm,g_diLogging_GlobalState[i])>0)	//Error
							{
							if (g_diLogging_Message[i]!=c_Ok)		
								iArchiveState=g_diLogging_Message[i];				
							else
								iArchiveState=c_Stopped;			//=Archive stopped	
								
							i=len+1; 								//Exit 
							}
						else										//Warning		
							{
							if (g_diLogging_GlobalState[i]==c_Timeout)
								iArchiveState=g_diLogging_GlobalState[i];				
							else
								{
								if (!bWarning)
									iArchiveState=g_diLogging_Message[i];
								}
													
							bWarning=true;
							}
						}
					else
						{
						if (!bWarning)
							iArchiveState=c_Ok;					//OK
						}
					}		
				}
			
			//Need to update Archive State Dp?
			unLHCLogging_GetPosition(sManager, iIndex_man);
			
			if (g_diOldArchives_State[iIndex_man]!=iArchiveState)
				{		
				iRes=dpSet(sDatapoint,iArchiveState);			//Writ Global Result in Dpe
				g_diOldArchives_State[iIndex_man]=iArchiveState;
				}
			}
		else
			{
			fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_StateArchives:" + getCatStr(g_sCat_LHCLogging,"DPDOESNOTEXIST"),"");
			if(isFunctionDefined("unMessageText_sendException"))
				unMessageText_sendException("*", "*", "LHCLogging", "user", "*", exceptionInfo);
			}
		}
}


// unLHCLogging_Configuration
/**
Purpose: Initialize LHCLogging configuration

Parameter: 	iType, int, input, 1=Manager and 0=Archive
			sSystemName, string, input, System Name
			dsDefault_Settings, dyn_string, output, Return all the settings			
			sFunction_Name, string, output, Function Name to be used to update Table for configFile
			sComponent_Name, string, output, Name of the Component used for this Manager
			iTimeOut, int, input, Time in minutes before rising warning for archive inactivity
			iDelay, int, input, Delay in secondes to check archive inactivity
			exceptionInfo, dyn string, output, Message
									
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_Configuration(int iType, string sSystemName, dyn_string &dsDefault_Settings, string  &sFunction_Name, string &sComponent_Name, int &iTimeOut, int &iDelay, dyn_string &exceptionInfo_Temp)
{
	int iRes;
	string sDpSettings;
	
	if (!dpExists(sSystemName + "LHCLogging_Manager_Info.defaultSettings"))
		{
		iRes=dpCreate("LHCLogging_Manager_Info","_Un_LHCLogging_Config");
		if (iRes<0)
			fwException_raise(exceptionInfo_Temp, "ERROR", "unLHCLogging_Configuration(): " + getCatStr(g_sCat_LHCLogging,"DPCREATEFAILURE"),"");	
		else
			{
			if (dpExists("LHCLogging_Manager_Info"))
				{
				//ARCHIVE
				dsDefault_Settings[LOGGING_ARCH_SQL_PERIOD]="300";
				dsDefault_Settings[LOGGING_ARCH_SQL_MAXDPENUMBER]="200";
				dsDefault_Settings[LOGGING_ARCH_MAXCPUUSAGE]="0.50";
				dsDefault_Settings[LOGGING_ARCH_DATAPATH]="data/xml_sender/";			
				dsDefault_Settings[LOGGING_ARCH_SQL_ALWAYSUSEPERIOD]="FALSE";				
				
				sDpSettings="Archive_Settings";
				iRes= dpSetWait(sSystemName + "LHCLogging_Manager_Info.defaultSettings." + sDpSettings, dsDefault_Settings);
					
				//MANAGER
				dsDefault_Settings[LOGGING_MAN_SQL_DATATTRIBUTES]="_offline.._stime,_offline.._value";
				dsDefault_Settings[LOGGING_MAN_CONFIGPATH]="data/LHCLoggingData";
				dsDefault_Settings[LOGGING_MAN_TCP_LOCALHOST]="localhost";
				dsDefault_Settings[LOGGING_MAN_TCP_REMOTEHOST]="remotehost";
				dsDefault_Settings[LOGGING_MAN_TCP_DATAPATH]="/Loaders/LoggingDataLoader";
				dsDefault_Settings[LOGGING_MAN_TCP_METADATAPATH]="/Loaders/LoggingMetadataLoader";
				dsDefault_Settings[LOGGING_MAN_SQL_TIMEOUT]="180";
				dsDefault_Settings[LOGGING_MAN_SHOWSTATISTICS]="0x0";
				dsDefault_Settings[LOGGING_MAN_TCP_REMOTEPORT]="80";
				dsDefault_Settings[LOGGING_MAN_TCP_TIMEOUT]="360";
				dsDefault_Settings[LOGGING_MAN_PERIOD]="10";
				dsDefault_Settings[LOGGING_MAN_TCP_FILESIZE]="500000";					
				dsDefault_Settings[LOGGING_MAN_TCP_DATASCHEMA]="<?xml version='1.0' standalone='no'?>,<archiver_data xmlns = \\\"http://cern.ch/lhclogging/Schema/ArchiverDataElem\\\">,";
				dsDefault_Settings[LOGGING_MAN_TCP_METADATASCHEMA]="<?xml version='1.0' standalone='no'?>,<variable_registration xmlns=\\\"http://cern.ch/lhclogging/Schema/VariableRegistration\\\">,";				
				dsDefault_Settings[LOGGING_MAN_BEHIND_BACKUP] = 180;
				dsDefault_Settings[LOGGING_MAN_BEHIND_QUERY] = 60;
				dsDefault_Settings[LOGGING_MAN_WILDCHARS] = "#$n";
				dsDefault_Settings[LOGGING_MAN_MSEC_RESOLUTION] = "3";
				sDpSettings="Manager_Settings";
				sFunction_Name="get_UnicosInfos";
				sComponent_Name="Cryo Unicos";
				iTimeOut=120;
				iDelay=50;
				
				iRes= dpSet(sSystemName + "LHCLogging_Manager_Info.configFile.Function", sFunction_Name, sSystemName + "LHCLogging_Manager_Info.configFile.ComponentName", sComponent_Name, 
							sSystemName + "LHCLogging_Manager_Info.defaultSettings.Timeout", iTimeOut, sSystemName + "LHCLogging_Manager_Info.defaultSettings.Delay", iDelay,
							sSystemName + "LHCLogging_Manager_Info.defaultSettings." + sDpSettings, dsDefault_Settings);
				}
			}
		}
	else
		{
		switch	(iType)
			{
			case c_ARCHIVE:
				sDpSettings="Archive_Settings";
				dsDefault_Settings=makeDynString();										
				
				iRes= dpGet(sSystemName + "LHCLogging_Manager_Info.defaultSettings." + sDpSettings, dsDefault_Settings);	
				break;
				
			case c_MANAGER:
				sDpSettings="Manager_Settings";
				dsDefault_Settings=makeDynString();										
				sFunction_Name="";
				sComponent_Name="";
				iTimeOut=0;
				iDelay=50;
				
				iRes= dpGet(sSystemName + "LHCLogging_Manager_Info.defaultSettings." + sDpSettings, dsDefault_Settings, sSystemName + "LHCLogging_Manager_Info.configFile.Function", sFunction_Name, 
							sSystemName + "LHCLogging_Manager_Info.configFile.ComponentName", sComponent_Name, 
							sSystemName + "LHCLogging_Manager_Info.defaultSettings.Timeout", iTimeOut, sSystemName + "LHCLogging_Manager_Info.defaultSettings.Delay", iDelay);
				break;
			
			default:
				dsDefault_Settings=makeDynString("");
			}
		}
}

// unLHCLogging_Check_Delay_Archive
/**
Purpose: Function call by a new Thread

Parameter: 	sDp, string, input, Dp for the archive
						sSystem, string, input, System Name
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_Check_Delay_Archive(string sDp, int iIndex)
{

if (g_iLogging_ArchiveDelay==0)
	g_iLogging_ArchiveDelay=10;

while (true) 	{		
    		 	unLHCLogging_Calculate_Delay_Archive(sDp, iIndex); 
    		 	delay(g_iLogging_ArchiveDelay);
    		 	} 
}

// unLHCLogging_SaveState
/**
Purpose: Save in Global Variables the State for Manager(s) and Archive(s)

Parameter: 	sName, string, input, Archive's name or Manager's name
			diState, dyn_int, values for the Status of this name
			index, int, output, position in Array of this name
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_SaveState(string sName, dyn_int diState, int &index)
{
	unLHCLogging_GetPosition(sName, index);
	
	if (dynlen(diState)>0 && index>0)
		{
		g_diLogging_Message[index]=diState[State_Message];
		g_diLogging_Command[index]=diState[State_Command];
		g_diLogging_PVSS[index]=diState[State_PVSS];
		g_diLogging_State[index]=diState[State_StateOut];
		g_dtLogging_LastTime[index]=diState[State_Timeout];
		g_dtLogging_LastRef[index]=diState[State_LastRefEnd];
		}		
}


// unLHCLogging_GetPosition
/**
Purpose: Get Position in Global Variables from the name (Manager or Archive)

Parameter: 	sName, string, input, Archive's name or Manager's name
			iIndex, int, output, position in Array of this name
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_GetPosition(string sName, int &iIndex)
{
int i, length;

	iIndex=1;
	
	sName=substr(sName,strpos(sName,":") + 1 , strlen(sName));
	length=dynlen(g_dsLogging_Name);
	for (i=1;i<=length;i++)
		{
		if (g_dsLogging_Name[i]==sName)
			{
			iIndex=i;
			i=length+1;
			}
		}
}

// unLHCLogging_Calculate_Delay_Archive
/**
Purpose: Check Time range between Last sending of data and now (before loss of data during switch)

Parameter: 	sDp_Archive, string, input, Dp name for the archive
						iPosition_Archive, string, input, Position of the archive in the array
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_Calculate_Delay_Archive(string sDp_Archive, int iPosition_Archive)
{
int iRes, iStatus; 
bool bChange=false;
string sArchive;

	unLHCLogging_Save_Delay_Archive(iPosition_Archive, bChange);
		
	if (bChange)	//Need update
		{
		sArchive = sDp_Archive;
		//Update Manager Status
		unLHCLogging_getStatus(iPosition_Archive, iStatus); 	
		
		unLHCLogging_StateArchives(sArchive, iPosition_Archive, iStatus);	
		
		//Update Archive display if Thread started from Panel
		if(g_bPanel)
			unLHCLogging_setStatusTable(0, sArchive, iPosition_Archive, iStatus);
		}
}

// unLHCLogging_Save_Delay_Archive
/**
Purpose: Update global variable to set the new state of Timeout

Parameter: 	iIndex, int, input, position in array
			bEdit, bool, output, need to update?
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_Save_Delay_Archive(int iIndex, bool &bEdit)
{
dyn_string exceptionInfo;
time tNow, tLast, tFirst;
int iTime, iNow, iLast;
	
	if(dynlen(g_dbLogging_Alarm_Timeout)>=iIndex && dynlen(g_diLogging_State)>=iIndex && iIndex!=0)
		{	
		tNow=getCurrentTime();
		tFirst=makeTime(1970,1,1,0,0,0);
		tLast=g_dtLogging_LastTime[iIndex];
			
		if (tLast!=tFirst)
			{
			//Get Time in seconds from 1970
			if (isFunctionDefined("ctrlTime"))
				{
				iNow=ctrlTime(tNow);		
				iLast=ctrlTime(tLast);
				}
			else
				{
				iNow=0;		
				iLast=0;
				fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_Save_Delay_Archive(): " + getCatStr(g_sCat_LHCLogging,"DLLERROR"),"");
				}
				
			if (iNow!=0 && iLast!=0)
				iTime=(iNow-iLast)/60;					//in minute		
			else
				iTime=g_iLogging_ArchiveTimeout + 1;	//Error
			}
					
		if (iTime>=g_iLogging_ArchiveTimeout)			//Out of Time
			{
			if (!g_dbLogging_Alarm_Timeout[iIndex])
				{
				bEdit=true;
				g_dbLogging_Alarm_Timeout[iIndex]=true;
				}
			}
		else	
			{											//In Time
			if (g_dbLogging_Alarm_Timeout[iIndex])
				{
				bEdit=true;
				g_dbLogging_Alarm_Timeout[iIndex]=false;
				}
			}
		}
}

// unLHCLogging_Integrity
/**
Purpose: Start checking for PVSS Manager alive 
Parameter: 	sManager, string, input, Manager Name
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_Integrity(string sManager)
{
	int iPosition;
	
	//sManager = substr(sManager,strlen(getSystemName()),strlen(sManager));
	unLHCLogging_GetPosition(sManager, iPosition);
	if (g_diLogging_man_period[iPosition]<=0)
		g_diLogging_man_period[iPosition]=c_Min_Rate;
	
	while (true) 	
		{
	  	unLHCLogging_Check_Integrity(sManager, iPosition); 
		unLHCLogging_GetPosition(sManager, iPosition);
		if (iPosition<1)
			delay(c_Min_Rate);
		else
			delay(g_diLogging_man_period[iPosition]); 
		} 
}


// unLHCLogging_Check_Integrity
/**
Purpose: Function checks period of answer from Manager

Parameter: 	sMan, string, input, Manager Name
			iIndex, int, input, Position in array
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_Check_Integrity(string sMan, int iIndex)
{
int iLastRefresh, iNow, iPeriod, iState, iRes, iMan_num;
time tNow, tValue_2;
string sDatapoint_1, sDatapoint_2, sDatapoint_3;
dyn_string exceptionInfo;
	
	tNow=getCurrentTime();
	sDatapoint_1= sMan + ".Settings.man_Num";

	if (dpExists(sDatapoint_1))
	
		{
		iRes=dpGet(sDatapoint_1, iMan_num);
		
		if(dynContains(g_diLogging_API, iMan_num))
			{
			sDatapoint_2=sMan + ".Settings.man_Num:_online.._stime";
			sDatapoint_3=sMan + ".Settings.man_NumPeriod";	
	
			iRes=dpGet(sDatapoint_2, tValue_2, sDatapoint_3, g_diLogging_man_period[iIndex]);
			if (g_diLogging_man_period[iIndex]<(c_Min_Rate/2))
				g_diLogging_man_period[iIndex]=c_Min_Rate;
			else
				g_diLogging_man_period[iIndex]=g_diLogging_man_period[iIndex]*2;
		
			//Get Time in seconds from 1970
			if (isFunctionDefined("ctrlTime"))
				{
				iNow=ctrlTime(tNow);		
				iLastRefresh=ctrlTime(tValue_2);
				}
			else
				{
				iNow=0;		
				iLastRefresh=0;
				fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_Check_Integrity(): " + getCatStr(g_sCat_LHCLogging,"DLLERROR"),"");
				}
			
			if (iNow!=0 && iLastRefresh!=0)
				iPeriod=iNow-iLastRefresh;	
			else
				iPeriod=g_diLogging_man_period[iIndex]+1;
			
			//DebugN("Period: " + iPeriod + " >? " + g_diLogging_man_period[iIndex]);
			if (iPeriod > g_diLogging_man_period[iIndex])
				iState=2;				//Blocked
			else
				iState=1;				//Started		
			}
		else
			iState=0;					//Stopped
				
		unLHCLogging_Update_PVSS_Status(sMan, iState);	
		}
}

// unLHCLogging_Update_PVSS_Status
/**
Purpose: update Status according to the result 
Parameter: 	sManager, string, input, Manager Name
			iState_PVSS, int, State of Manager in PVSS console
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_Update_PVSS_Status(string sManager, int iState_PVSS)
{
int iPosition, iStatus, iRes, iStatus_Archive;
dyn_string exceptionInfoTemp;
	
	unLHCLogging_GetPosition(sManager, iPosition);
	if (iPosition>0)
		{	
		if (iState_PVSS!=g_diLogging_PVSS[iPosition])			//PVSS Status has changed
			{
			g_diLogging_PVSS[iPosition]=iState_PVSS;			//Update Global
	
			if (g_bPanel)	//for unLHCLoggingStatus.pnl	
				{
				//2.2 Get State for MANAGER!
				unLHCLogging_getStatus(iPosition, iStatus);

				if (iStatus!=c_Error)				//NO Error for Manager
					{
					//2.3 Get Archive(s) State linked with this Manager!
					iRes=dpGet(sManager + ".State.st_Archives", iStatus_Archive);
					if (iStatus_Archive!=c_Ok)		
						{
						if (dynContains(g_diLoggingCritical_Alarm, iStatus_Archive))
							iStatus=c_Error;
						else
							iStatus=c_Warning;
						}
					}
				
				//2.4 Update Table in Panel
			//	DebugN("unLHCLogging_Update_PVSS_Status", sManager, iPosition, iStatus);
				unLHCLogging_setStatusTable(1, sManager, iPosition, iStatus);	
				}
			else			//for SystemIntegrity
				{
				//go to dpConnect (dpSet with the same Value) - Not a good solution!!!!
//				DebugN("g_diLogging_Message[iPosition]", g_diLogging_Message[iPosition]);
				iRes=dpSet(sManager + ".State.ev_MessageOut", g_diLogging_Message[iPosition]);
				}		
			}
		}
	else
		fwException_raise(exceptionInfoTemp, "ERROR", "unLHCLogging_Update_PVSS_Status(): Unknow Manager " + sManager,"");
}

// unLHCLogging_CheckLine
/**
Purpose: Check Line syntax for Config File

Parameter: 	sSelected, string, input, string for the cell Selected 
			sArchive, string, input, string for the cell Archive 
			sAlias, string, input, string for the cell Alias 
			sDpe, string, input, string for the cell Dpe
			sName, string, input, string for the cell Name 
			sDescription, string, input, string for the cell Description
			sHierarchy, string, input, string for the cell Hierarchy
			sFormat, string input, string for the format ("", pvss or other)
			sEr, string, output, Pattern for the error 
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_CheckLine(string sSelected, string sArchive,string sAlias, string sDpe, string sName, string sDescription, string sHierarchy, string sFormat, string &sEr)
{
int iEr,i, length;
dyn_string dsSplitHierarchy;
dyn_string dsNotAllowed = makeDynString("\"","'",">","<","~","&","?","!", "@", "#", "$", "%", "^", "*",
																				"","","","","","","","","","","","","","","","","",
																				"","","","","");
		sEr="";
		length = dynlen(dsNotAllowed);
		if ((sDpe!="") && (sAlias=="")){
		//Case 1 : dpe="something" alias =
				if (!dpExists(sDpe)) sEr="Dpe";	
			}
		else 
			if ((sDpe!="") && (sAlias!="")){
		//Case 2 : dpe="ProcessInput.Post" alias ="Alias"
				sDpe = unGenericDpFunctions_dpAliasToName(sAlias)+sDpe;
				if (!dpExists(sDpe)) sEr="Dpe";	
			}

		if (sArchive=="")
			sEr="Archive";		
		else
			{
			if ((sSelected=="X" || sSelected=="F") && sDpe!="")
				{
				//5. Name	
		/*		iEr=0;
				iEr=strpos(sName,"\"");
				iEr=strpos(sName,"'") + iEr;
				iEr=strpos(sName,">") + iEr;
				iEr=strpos(sName,"<") + iEr;
				iEr=strpos(sName,"~") + iEr;
				iEr=strpos(sName,"&") + iEr;
				iEr=strpos(sName,"?") + iEr;
				iEr=strpos(sName,"!") + iEr; 			*/	
	
				iEr=0;
				for(i=1; i<=length;i++)
				{
				iEr=strpos(sName,dsNotAllowed[i]) + iEr;
				}
	
				if (iEr> -length || sName=="ERROR")		
					{
					sEr="Name";
					return;
					}
				
				//4. Description
				/*
				iEr=0;
				iEr=strpos(sDescription,"\"");
				iEr=strpos(sDescription,"'") + iEr;
				iEr=strpos(sDescription,">") + iEr;
				iEr=strpos(sDescription,"<") + iEr;
				iEr=strpos(sDescription,"~") + iEr;
 				iEr=strpos(sDescription,"&") + iEr;
				iEr=strpos(sDescription,"?") + iEr;
				iEr=strpos(sDescription,"!") + iEr; 				

				if (iEr> -8 || sDescription=="ERROR")		
					{
					sEr="Description";
					return;
					}*/
					iEr=0;
				for(i=1; i<=length;i++)
				{
				iEr=strpos(sDescription,dsNotAllowed[i]) + iEr;
				}
	
				if (iEr> -length || sDescription=="ERROR")		
					{
					sEr="sDescription";
					return;
					}
					
				dsSplitHierarchy=strsplit(sHierarchy,"/");
				//3. Hierarchy
				/*
				iEr=strpos(sHierarchy,"\"");
				iEr=strpos(sHierarchy,"'") + iEr;
				iEr=strpos(sHierarchy,">") + iEr;
				iEr=strpos(sHierarchy,"<") + iEr;
				iEr=strpos(sHierarchy,"~") + iEr; 
				iEr=strpos(sHierarchy,"&") + iEr;
				iEr=strpos(sHierarchy,"?") + iEr;
				iEr=strpos(sHierarchy,"!") + iEr;  
							
				if (iEr> -8 || sHierarchy=="" || sHierarchy=="ERROR" || sHierarchy=="/")
					{
					sEr="Hierarchy";	
					return;
					}
			*/
			
				iEr=0;
				for(i=1; i<=length;i++)
				{
				iEr=strpos(sHierarchy,dsNotAllowed[i]) + iEr;
				}
	
				if (iEr> -length || sHierarchy=="ERROR")		
					{
					sEr="sHierarchy";
					return;
					}
					
				///  Format 
				if ((sFormat!="") & (sFormat!="pvss")){
				iEr=0;
				iEr=strpos(sFormat,"\"");
				iEr=strpos(sFormat,"'") + iEr;
				iEr=strpos(sFormat,">") + iEr;
				iEr=strpos(sFormat,"<") + iEr;
				iEr=strpos(sFormat,"~") + iEr;
 				iEr=strpos(sFormat,"&") + iEr;
				iEr=strpos(sFormat,"?") + iEr;
				iEr=strpos(sFormat,"!") + iEr; 				
				if(!(patternMatch("%*", sFormat)) || (iEr> -8) || (sFormat=="ERROR") )
					sEr="Format";
					}
				}
			}
}

// unLHCLogging_getFromFile
/**
Purpose: Check Line syntax for Config File

Parameter: 	sConfigPath, string, input, Path
			sDp, string, input, Dp for the archive
			dsSelected, dyn_string, output, array for the column Selected 
			dsAlias, dyn_string, output, array for the column Alias 
			dsName, dyn_string, output, array for the column Name 			
			dsArchive, dyn_string, output, array for the column Archive  
			dsDescription, dyn_string, output, array for the column Description			
			dsHierarchy, dyn_string, output, array for the column Hierarchy			
			dsDpe, dyn_string, output, array for the column Dpe
			dsFormat, dyn_string, output, array for the column Format
			bEr, boolean, output, there is an error
			sManager, string, input, name of the checked manager
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_getFromFile(string sConfigPath, dyn_string &dsSelected, dyn_string &dsAlias, dyn_string &dsName, dyn_string &dsArchive, dyn_string &dsDescription, dyn_string &dsHierarchy, dyn_string &dsDpe,dyn_string &dsFormat, bool &bEr, string sManager)
{
	string fileInString, sArchive, sAlias, sHierarchy, sDescription, sName, sDpe, sNode, sPattern, sFormat;
	int i, iPos, len, len2;
	float fMax, fIndex;
	dyn_string dsFileLines, dsSplit, dsDomain;
	bool bChange, bDefined;

	dyn_dyn_string ddsFile;

	if (isFunctionDefined("ctrlConfRead"))
	{
		ctrlConfRead(sConfigPath,ddsFile);
		len=dynlen(ddsFile);
		fMax=len;
		for (i=1;i<=len;i++) //read each statement
		{				
			int l=1;
			dsFileLines=ddsFile[i];

			len2=dynlen(dsFileLines);
//			DebugN(dsFileLines,len2);
			fIndex=i;

			progressBar.scale(fIndex/fMax,1);

			//get Archive
			if(patternMatch("*_ValueArchive_*", dsFileLines[l]))
				{					
					sArchive=substr(dsFileLines[l],strpos(dsFileLines[l],"_ValueArchive_"));
					sArchive=substr(sArchive,0,strpos(sArchive,"]"));
					bDefined = dpExists(sManager + sArchive);
				}
				else
				{
				l=l+1;
					if(patternMatch("*dpe=*", dsFileLines[l]))
					{
						dynAppend(dsArchive,sArchive);
						sDpe=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1);
						dynAppend(dsDpe,sDpe);
						if (bDefined) dynAppend(dsSelected,"X");
							else dynAppend(dsSelected,"L");
						//get alias 
						if(patternMatch("*alias=*", dsFileLines[l+1]))
						{
							sAlias=substr(dsFileLines[l+1], strpos(dsFileLines[l+1],"=")+1);
							dynAppend(dsAlias,sAlias);
						}
						else
						{
							dynAppend(dsAlias,"ERROR");
							dynAppend(dsName,"ERROR");
							dynAppend(dsDescription,"ERROR");
							dynAppend(dsHierarchy,"ERROR");
							dynAppend(dsFormat,"ERROR");
					//		l=len2+1;
							bEr=true;
							return;
						}
						//get name 
						if(patternMatch("*name=*", dsFileLines[l+2]))
						{
							sName=substr(dsFileLines[l+2], strpos(dsFileLines[l+2],"=")+1);
							dynAppend(dsName,sName);
						}
						else
						{
							dynAppend(dsName,"ERROR");
							dynAppend(dsDescription,"ERROR");
							dynAppend(dsHierarchy,"ERROR");
							dynAppend(dsFormat,"ERROR");
					//		l=len2+1;
							bEr=true;
							return;
						}
												
						//get description 
						if(patternMatch("*description=*", dsFileLines[l+3]))
						{
							sDescription=substr(dsFileLines[l+3], strpos(dsFileLines[l+3],"=")+1);
							dynAppend(dsDescription,sDescription);
						}
						else
						{
							dynAppend(dsDescription,"ERROR");
							dynAppend(dsHierarchy,"ERROR");			
							dynAppend(dsFormat,"ERROR");								
					//		l=len2+1;
							bEr=true;
							return;
						}
						
						//get hierarchy
						if(patternMatch("*hierarchy=*", dsFileLines[l+4]))
						{
							sNode="";
							sPattern=dsFileLines[l+4];
							iPos=strpos(sPattern,"node name=");
							while (iPos>=0)
						  {
						    sPattern=substr(sPattern, iPos + 11, strlen(sPattern) - (iPos+11));
						    if (sNode=="")
						    	sNode=substr(sPattern, 0, strlen(sPattern) - (strlen(sPattern) - strpos(sPattern,"\"")));
						    else	
							    sNode=sNode + "/" + substr(sPattern, 0, strlen(sPattern) - (strlen(sPattern) - strpos(sPattern,"\"")));
							    iPos=strpos(sPattern,"node name=");
						  }
							dynAppend(dsHierarchy,sNode);
						}
						else
						{
							dynAppend(dsHierarchy,"ERROR");
							dynAppend(dsFormat,"ERROR");
				//			l=len2+1;
							bEr=true;
							return;								
						}
						if(l+5 <= len2) //line exists
						{
							if(patternMatch("*_ValueArchive_*", dsFileLines[l+5]) || patternMatch("*dpe=*", dsFileLines[l+5])) // no format
									{
										dynAppend(dsFormat,"");	
										i=i+1;		
									}
							else 
								{
								if(patternMatch("*format*", dsFileLines[l+5])) // format is defined
									{
									if(patternMatch("*format=*", dsFileLines[l+5])) // format is defined
										{
											sFormat=substr(dsFileLines[l+5], strpos(dsFileLines[l+5],"=")+1);
											dynAppend(dsFormat,sFormat);
										}
									else  		
										{
											dynAppend(dsFormat,"");	
										}
									}
								else
									{
										dynAppend(dsFormat,"ERROR");
										l=len2+1;
										bEr=true;
										return;						
									}
								}
						}
						else //last line of the file reached and format not defined
						{
							dynAppend(dsFormat,"");			
				//			l=len2+1;
						}
					} // pattern dpe
					else
					{
						dynAppend(dsArchive,sArchive);
						dynAppend(dsSelected,"X");
						dynAppend(dsDpe,"ERROR");
						dynAppend(dsName,"ERROR");
						dynAppend(dsAlias,"ERROR");
						dynAppend(dsDescription,"ERROR");
						dynAppend(dsHierarchy,"ERROR");
						dynAppend(dsFormat,"ERROR");
				//		l=len2+1;
						bEr=true;
					}	
				}		// pattern archive
			}			// for
//					DebugN(dsArchive,dsSelected,dsDpe,dsName,dsAlias,dsDescription,dsHierarchy,dsFormat);
		}					//if configpath

		else
		{
		DebugN("************* DLL not defined *************");
		}
	}

// unLHCLogging_setStatusTable
/**
Purpose: update display to show Status (Manager and Archive)

Parameter: 	bType, bool, input, 0=Archive 1=Manager
			sName, string, input, Manager or Archive name
			iIndex, int, input, Position in Global Array
			iState, int, Manager or Archive State
						
Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unLHCLogging_setStatusTable(bool bType, string sName, int iIndex, int iState)
{
int i, length;
string sContent, sCatalogue, sTable, sText_PVSS, sText_Command, sText_StateOut, sText_Message, sColor, sprefixe, t;
time tLastTime;

	sCatalogue=g_sCat_LHCLogging;
if (bType)	//Manager
	{
	sName=substr(sName,strpos(sName,":")+1);
	length=ManagersList.lineCount();
	sTable="ManagersList";
	sprefixe = "M_"; 
	}
else		//Archive
	{
	sName=substr(sName,strpos(sName,"_ValueArchive_"));
	sprefixe = "A_"; 
	sTable="ArchivesList";
	length=ArchivesList.lineCount();
	}

for (i=0;i<length;i++)
	{
	getValue(sTable, "cellValueRC", i, "Names", sContent);
//	DebugN(sContent, sName);
	if (sContent==sName)			//find Line
		{
		unCoreLHCServices_PVSS(g_diLogging_PVSS[iIndex], sText_PVSS);
		sText_Command=getCatStr(sCatalogue, sprefixe + g_diLogging_Command[iIndex]);
		sText_StateOut = getCatStr(sCatalogue, g_diLogging_State[iIndex]);
		
		if (g_diLogging_Message[iIndex]!=10150)		//->internal purpose for PVSS00LHCLogging
			sText_Message = getCatStr(sCatalogue, g_diLogging_Message[iIndex]);
		else
			getValue(sTable, "cellValueRC", i, "MessageOut", sText_Message);

		switch(iState)
			{
			case 0:
				sColor="unLHCLogging_Ok";
				break;
			
			case c_Warning:
				sColor="unLHCLogging_Warning";
				if (g_dbLogging_Alarm_Timeout[iIndex] && !bType)							//Timeout Warning for archive
					sText_Message=getCatStr(sCatalogue, c_Timeout);	
				break; 
			
			case c_Error:
				sColor="unLHCLogging_Bad";
				break;
			
			default:
				sColor="unLHCLogging_Bad";
			}
		
		tLastTime=g_dtLogging_LastRef[iIndex];
		t=formatTime("%c", tLastTime);
	 	
 		if (_UNIX)
			t+=timeFromGMT(); // ???
		
		setMultiValue (sTable, "cellBackColRC", i, "X", sColor, sTable, "cellValueRC", i, "PVSS", sText_PVSS, sTable, "cellValueRC", i, "Command", sText_Command, sTable, "cellValueRC", i, "StateOut", sText_StateOut, sTable, "cellValueRC", i, "MessageOut", sText_Message, sTable, "cellValueRC", i, "Time", t);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _unSystemIntegrity_LHCLogging_CB
/**
Purpose: Get global settings and API connection for all Managers

	@param sDp_1: string, input, data point name
	@param iTimeout: int, Timeout in minutes
	@param sDp_2: string, input, data point name
	@param iDelay: int, Delay in secondes before checks
	@param sDp_3: string, input, data point name
	@param diApi: dyn_int, API List

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/


_unSystemIntegrity_LHCLogging_CB(string sDp_1, int iTimeout, string sDp_2, int iDelay, string sDp_3, dyn_int diApi)
{
	g_iLogging_ArchiveTimeout=iTimeout;
	g_iLogging_ArchiveDelay=iDelay;
	g_diLogging_API=diApi;
}


// -------------------------------------------------------------------------------------

dyn_int unLHCLogging_getArchiveItems(const string sArchive, const dyn_string dsArchives, const dyn_string dsSelected, const dyn_string dsNames)
{
int iMax, iLen, i, k;
dyn_int diRes;

	if(sArchive!="")
		{
		iMax=0;
		k=0;
		iLen=dynlen(dsSelected);
		for (i=1; i<=iLen; i++)
			{
			if(dsNames[i]!="")
				{
				if(patternMatch(sArchive, dsArchives[i]))
					{
					iMax++;
					if(dsSelected[i]=="X") k++;
					}
				}
			}		
		}

	diRes[1]=k;
	diRes[2]=iMax;
	
return diRes;
}


//@}

