/**

@name LIBRARY: unRecipeFunctions.ctl

@author: Ivan Prieto Barreiro (EN-ICE-SIC)

Creation Date: April 2011 

version 1.0

Modification History:
  01/11/2011: Ivan
    - The device type names have been modified to use the "CPC_" prefix

External Functions: 
  . 
  
Internal Functions:
  .
*/

const int unRecipeFunctions_ERROR_NoRecipeClass=-1;
const int unRecipeFunctions_ERROR_InvalidElements=-2;
const int unRecipeFunctions_ERROR_NoRecipeInstance=-3;


const string unRecipeFunctions_RecipeClassDpType="UnRcpClass";
const string unRecipeFunctions_RecipeClassDpPrefix="";
const string unRecipeFunctions_RecipeInstanceDpPrefix="recipeInstance/";
const string unRecipeFunctions_RecipeInstanceDpType="UnRcpInstance";


/********************************************************** 
 *  Indexes for the elements in the recipe instance data  *
 **********************************************************/
const int RCP_INSTANCE_ID=1;
const int RCP_INSTANCE_NAME=2;
const int RCP_INSTANCE_CLASS=3;
const int RCP_INSTANCE_AUTHOR=4;
const int RCP_INSTANCE_VERSION=5;
const int RCP_INSTANCE_CREATED=6;
const int RCP_INSTANCE_LAST_USAGE=7;
const int RCP_INSTANCE_LAST_MODIFIED=8;
const int RCP_INSTANCE_LAST_ACT_USER=9;
const int RCP_INSTANCE_LAST_MOD_USER=10;
const int RCP_INSTANCE_VALUES=11;
const int RCP_INSTANCE_PREDEFINED=12;


/******************************************************* 
 *  Indexes for the elements in the recipe class data  *
 *******************************************************/
const int RCP_CLASS_NAME=1;
const int RCP_CLASS_TYPE=2;
const int RCP_CLASS_DESC=3;
const int RCP_CLASS_ELEMENTS=4;
const int RCP_CLASS_ELEMENTS_DESC=5;
const int RCP_CLASS_ELEMENTS_RANGE=6;
const int RCP_CLASS_ELEMENTS_UNIT=7;
const int RCP_CLASS_INSTANCES=8;
const int RCP_CLASS_EDITABLE=9;
const int RCP_CLASS_LAST_MODIFIED=10;
const int RCP_CLASS_LAST_ACTIVATED=11;
const int RCP_CLASS_LAST_ACTIVATION_TIME=12;

 

/********************************************************* 
 *  Indexes for the elements in the recipe header buffer *
 *********************************************************/
const int RCP_HEADER_MAX_SIZE	= 15;
const int RCP_HEADER_ID_LOW		= 1;
const int RCP_HEADER_ID_HIGH	= 2;
const int RCP_HEADER_NUM_MANREG	= 3;
const int RCP_HEADER_NUM_DPAR	= 4;
const int RCP_HEADER_DB_DPAR	= 5;
const int RCP_HEADER_NUM_WPAR	= 6;
const int RCP_HEADER_DB_WPAR	= 7;
const int RCP_HEADER_NUM_APAR	= 8;
const int RCP_HEADER_DB_APAR	= 9;
const int RCP_HEADER_NUM_AALARM	= 10;
const int RCP_HEADER_DB_AALARM	= 11;
const int RCP_HEADER_NUM_CONTR	= 12;
const int RCP_HEADER_DB_CONTR	= 13;
const int RCP_HEADER_SENT_VALUE	= 14;
const int RCP_HEADER_CRC		= 15;


/********************************************************* 
 *  Indexes for the elements in the recipe status buffer *
 *********************************************************/
const int RCP_STATUS_MAX_SIZE			= 16;
const int RCP_STATUS_MIRROR_DATA_SIZE	= 13;	// Indicates the last position where the data in the status buffer is a copy
												// of the data in the header buffer
const int RCP_STATUS_ID_LOW				= 1;
const int RCP_STATUS_ID_HIGH			= 2;
const int RCP_STATUS_NUM_MANREG			= 3;
const int RCP_STATUS_NUM_DPAR			= 4;
const int RCP_STATUS_DB_DPAR			= 5;
const int RCP_STATUS_NUM_WPAR			= 6;
const int RCP_STATUS_DB_WPAR			= 7;
const int RCP_STATUS_NUM_APAR			= 8;
const int RCP_STATUS_DB_APAR			= 9;
const int RCP_STATUS_NUM_AALARM			= 10;
const int RCP_STATUS_DB_AALARM			= 11;
const int RCP_STATUS_NUM_CONTR			= 12;
const int RCP_STATUS_DB_CONTR			= 13;
const int RCP_STATUS_STATUS				= 14;
const int RCP_STATUS_CRC				= 15;
const int RCP_STATUS_ERR				= 16;


/********************************************************* 
 *                 Recipe Status                         *
 *********************************************************/
const int RCP_ACTIVATION_TRIGGERED = 0; // The recipe activation has been triggered
const int RCP_ACTIVATED=1;            	// The recipe is activated in the PLCs
const int RCP_NO_PLC_CONNECTION = 2;  	// There is no conection with one or more PLCs
const int RCP_INACTIVE=3;             	// The recipe is inactive
const int RCP_BAD_TYPES=4;            	// The value type is different than the DPE type
const int RCP_BAD_STRUCTURE=5;        	// The number of values is different than the number of DPEs
const int RCP_ACTIVATION_FAILED=6;    	// The recipe activation failed


/********************************************************* 
 *                 Recipe Activation Status              *
 *    (values received in the recipe status buffer)      *
 *********************************************************/
const int RCP_ACT_STATUS_ERROR      	   = -1;  // There was an error during the recipe activation
const int RCP_ACT_STATUS_STANDBY           = 0;   // Recipe activation in standby
const int RCP_ACT_STATUS_NEW_ID_RECEIVED   = 1;   // New recipe identified received
const int RCP_ACT_STATUS_CRC_OK            = 2;   // The CRC is Ok, waiting for activation command 
const int RCP_ACT_STATUS_DONE              = 3;   // Recipe activation done
                                  

/********************************************************* 
 *      Constants used in the recipe activation          *
 *********************************************************/
// Recipe header 'Sent Value'
const int RCP_ACT_HEADER_CANCEL			= -1;
const int RCP_ACT_HEADER_SENDING_DATA	= 1;	// Sending recipe data to the PLC
const int RCP_ACT_HEADER_DATA_SENT		= 2;	// All data sent to the PLC
const int RCP_ACT_HEADER_ACTIVATE		= 3;	// Recipe activation order 
const int RCP_ACT_HEADER_ACTIVATION_OK	= 4;  	// Recipe activation Ok
								  


/****************************** 
 *      Global   Variables    *
 *****************************/
global mapping unRecipe_rangeElements;  // Map where key=device type, value=dpe containing the range definition for the device type
global mapping unRecipe_unRecipe_pidManRegBits;   // Map where key=controller dpe name, value=bit position of the dpe inside the ManReg01 
global mapping unRecipe_aalarmManRegBits;// Map where key=analog alarm dpe name, value=bit position of the dpe inside the ManReg01 
global int g_editingRowNumber = -1;
global string g_editingColName = "";

/****************************** 
 *       Recipe Buffers       *
 *****************************/
global mapping gHeaderMap;			// Header buffer map
global mapping gManRegAddrMap;  	// ManReg address buffer map
global mapping gManRegValMap;		// ManReg values buffer map: this are the values used in the ArmRcp
global mapping gReqAddrMap;			// Req address buffer map
global mapping gReqValMap;			// Req values buffer map

/****************************** 
 *    ManReg Bit positions    *
 *****************************/
// DPAR bit definitions
const int DPAR_ARM_BIT		= 2;
const int DPAR_MONR_BIT 	= 4;
const int DPAR_MOFFR_BIT	= 5;
// WPAR bit definitions
const int WPAR_ARM_BIT		= 2;
const int WPAR_NEWMR_BIT	= 6;
// APAR bit definitions
const int APAR_ARM_BIT		= 2;
const int APAR_NEWMR_BIT	= 6;
// AALARM bit definitions
const int AALARM_ARM_BIT	= 2;
// PID bit definitions
const int PID_ARM_BIT		= 4;



void _unRecipeFunctions_initializeManRegBits()
{
  // Initializes the bit positions of the ManReg01 for the CPC_Controller Objects
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.ArmRcp"]= 4;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MSP"]   = 7;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MSPH"]  = 8;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MSPL"]  = 9;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MOutH"] = 10;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MOutL"] = 11;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MKc"]   = 12;  
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MTi"]   = 14;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MTd"]   = 13;
  unRecipe_unRecipe_pidManRegBits[".ProcessOutput.MTds"]  = 15;  
  
  // Initializes the bit positions of the ManReg01 for the Analog Alarm Objects
  unRecipe_aalarmManRegBits[".ProcessOutput.HH"] = 8;
  unRecipe_aalarmManRegBits[".ProcessOutput.H"]  = 9;
  unRecipe_aalarmManRegBits[".ProcessOutput.L"]  = 12;
  unRecipe_aalarmManRegBits[".ProcessOutput.LL"] = 13;  
  
}

//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to get recipe buffers by PLC name
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

dyn_int _unRecipeFunctions_getHeaderBuffer(string plcName) {
  if (mappingHasKey(gHeaderMap, plcName)) 
    return gHeaderMap[plcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gHeaderMap[plcName] = buffer;
  return buffer;
}

dyn_int _unRecipeFunctions_getManRegAddrBuffer(string plcName) {
  if (mappingHasKey(gManRegAddrMap, plcName)) 
    return gManRegAddrMap[plcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gManRegAddrMap[plcName] = buffer;
  return buffer;
}

dyn_int _unRecipeFunctions_getManRegValBuffer(string plcName) {
  if (mappingHasKey(gManRegValMap, plcName)) 
    return gManRegValMap[plcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gManRegValMap[plcName] = buffer;
  return buffer;
}


dyn_int _unRecipeFunctions_getReqAddrBuffer(string plcName) {
  if (mappingHasKey(gReqAddrMap, plcName)) 
    return gReqAddrMap[plcName];
  
  dyn_int buffer;
  dynAppend(buffer, 0);
  gReqAddrMap[plcName] = buffer;
  return buffer;
}


dyn_float _unRecipeFunctions_getReqValBuffer(string plcName) {
  if (mappingHasKey(gReqValMap, plcName)) 
    return gReqValMap[plcName];
  
  dyn_float buffer;
  dynAppend(buffer, 0.0);
  gReqValMap[plcName] = buffer;
  return buffer;
}
//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to set DP values
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void _unRecipeFunctions_setDpValue(string dpName, string attribute, anytype value, dyn_string &exceptionInfo)
{
  int rc=dpSet(dpName+attribute, value);
  if (rc) {
    fwException_raise(exceptionInfo,"ERROR","Could not set the "+attribute+" value for the DP: "+dpName+attribute,"");
    return;
  }    
}


void _unRecipeFunctions_appendDpValue(string dpName, string attribute, dyn_anytype value, dyn_string &exceptionInfo)
{
  dyn_anytype currentValue;
  int rc=dpGet(dpName+attribute, currentValue);
  if (rc) {
    fwException_raise(exceptionInfo,"ERROR","Could not get the "+attribute+" value for the DP: "+dpName,"");
    return;
  }    
  
  dynAppend(currentValue, value);
  rc = dpSet(dpName+attribute, currentValue);
  if (rc) {
    fwException_raise(exceptionInfo,"ERROR","Could not set the "+attribute+" value for the DP: "+dpName,"");
    return;
  }      
}
//--------------------------------------------------------------------------------------------------------------------------

/** 
 * Gets the formatted values of a set of DPEs.
 * @param dpNames - [IN] List of dp names in the UNICOS format.
 * @param values - [IN] List of values for the specified dpNames.
 * @param formattedValues - [OUT] Formatted values.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_formatValues(dyn_string dpNames, dyn_string values, 
                                     dyn_anytype &formattedValues, dyn_string &exceptionInfo)
{
  string rangeElement, deviceType, rangeMin, rangeMax, pvssDpName;
  dyn_string dsOnlineDpes;
  float fFormattedValue, fValue;
  bool bOk;
  
  if (dynlen(dpNames)!=dynlen(values)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of DPEs is different than the number of values!", "");
    return;
  }  
  
  for (int i=1; i<=dynlen(dpNames); i++) {  
    unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dpNames[i], pvssDpName);  
    // Get the DPE name (e.g. .ProcessOutput.MPosR")
    string dp=unGenericDpFunctions_getDpName(pvssDpName);
    int pos = strpos(pvssDpName, dp);
    string element = substr(pvssDpName, pos+strlen(dp), strlen(pvssDpName)-strlen(dp));   
    deviceType = dpTypeName(pvssDpName);
    
	//	Debug("******************************************\n");
	//DebugTN("Formatting values\n");
	//DebugTN("Dp: " + dpNames[i] + " = " + values[i] +"\n");

	// Removes the white spaces to format properly the exponential values (1.5 e1 => 1.5e1)
	strreplace(values[i], " ", "");
	
	if (deviceType=="CPC_DigitalParameter") {	
		if (values[i]==0) {
			formattedValues[i] = "FALSE";
			//DebugTN("Formatted Value: " + formattedValues[i] + "\n"); 
			continue;
		}
		else if (values[i]==1) {
			formattedValues[i] = "TRUE";
			//DebugTN("Formatted Value: " + formattedValues[i] + "\n"); 
			continue;
		}
			
		string value = strtoupper(values[i]);
		if (value=="TRUE" || value=="FALSE") {
			formattedValues[i] = value;
			//DebugTN("Formatted Value: " + formattedValues[i] + "\n"); 
		} 
		continue;
	}
		
    // Get the dpe where the range is defined for the current recipe element
    if (!mappingHasKey(unRecipe_rangeElements, deviceType+":"+element)){
      _unRecipeFunctions_getRangeElementFromPVSSDPE(pvssDpName, rangeElement, exceptionInfo);  
      unRecipe_rangeElements[deviceType+":"+element] = rangeElement;
    } else {
      rangeElement = unRecipe_rangeElements[deviceType+":"+element];    
    }   
    
    string format="";
    _unRecipeFunctions_getInstanceRange(pvssDpName, rangeElement, rangeMin, rangeMax, exceptionInfo);
    if (rangeMin!="" && rangeMax!=""){
      format = dpGetFormat(unGenericDpFunctions_getDpName(pvssDpName)+rangeElement);
    }
    
    if (format!="") {
      formattedValues[i]=unGenericObject_FormatValue(format, values[i]);
	  continue;
    } 
    
	// if the value is a float, format the value using the online DPE format
	if (dpElementType(dpNames[i]) == DPEL_FLOAT) {
		_unRecipeFunctions_getOnlineValueDpes(makeDynString(unGenericDpFunctions_getDpName(pvssDpName)+rangeElement), dsOnlineDpes);

		format = dpGetFormat(dsOnlineDpes[1]);
		//fFormattedValue = (float)values[i];
		//formattedValues[i] = unGenericObject_FormatValue(format, fFormattedValue);
		unGenericObject_StringToFloat(values[i], fValue, bOk);
		if (!bOk) {
			fwException_raise(exceptionInfo, "ERROR", "Float conversion error :" + values[i], "");
		}
		formattedValues[i] = unGenericObject_FormatValue(format, fValue);
	} else {
		// Applies the default PVSS format
		string s = values[i];  
		formattedValues[i] = s;
	}
  }  
}


void unRecipeFunctions_displayOnlineValues(string sRecipeDpName, dyn_string &exceptionInfo)
{
  dyn_anytype values;
  
  if (!shapeExists("RecipeElements")) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;	
  }
  
  unRecipeFunctions_getOnlineValues(sRecipeDpName, values, exceptionInfo);
  if(dynlen(exceptionInfo))  if (dynlen(exceptionInfo))	{ DebugTN(exceptionInfo); return;}
  
  for (int i=0; i<dynlen(values); i++) {
    setValue("RecipeElements", "cellValueRC", i, "Value", values[i+1]);
	_unRecipeFunctions_cellEditionFinished(sRecipeDpName, i, "Value");
  }  
}


/**
 * Gets the online values of the recipe class DPEs.
 * @param recipeClass - [IN] Name of the recipe class.
 * @param values - [OUT] Online formatted values corresponding to the recipe class DPEs.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */ 
 
void unRecipeFunctions_getOnlineValues(string sRecipeDpName, dyn_anytype &values, dyn_string &exceptionInfo)
{
  dyn_string dpNames, outputDpNames;
  dyn_anytype formattedValues;
  dyn_dyn_mixed recipeObject;

  
  dynClear(values);
  
  // Get the recipe object
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDpName, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo))	{ DebugTN(exceptionInfo); return;}
  
  // Get recipe dpes
  dpNames = recipeObject[fwConfigurationDB_RO_DPE_NAME]; 
  _unRecipeFunctions_getOnlineValueDpes (dpNames, outputDpNames);
  dpGet(outputDpNames, values);
  _unRecipeFunctions_formatValues(dpNames, values, formattedValues, exceptionInfo);  
  
  values = formattedValues;
}



string unRecipeFunctions_getDeviceRange(string dpName, dyn_string &exceptionInfo)
{
  string range, deviceType, dp, element, rangeMin, rangeMax, rangeElement;
  
  if (!dpExists(dpName)) return;
  
  deviceType = dpTypeName(dpName);   
  dp=unGenericDpFunctions_getDpName(dpName);
  int pos = strpos(dpName, dp);
  element = substr(dpName, pos+strlen(dp), strlen(dpName)-strlen(dp));
  dyn_string splitElement = strsplit(element, ".");

  // If the device type is a DigitalParameter the range is [FALSE, TRUE]
  if (deviceType=="CPC_DigitalParameter") {
    range = "[FALSE, TRUE]";
    return range;
  }
       
  if (!mappingHasKey(unRecipe_rangeElements, deviceType+":"+element)){
    _unRecipeFunctions_getRangeElementFromPVSSDPE(dpName, rangeElement, exceptionInfo);
    if (dynlen(exceptionInfo))  return;
    unRecipe_rangeElements[deviceType+":"+element] = rangeElement;
  } else {
    rangeElement = unRecipe_rangeElements[deviceType+":"+element];
  }

  // Get the range values
  _unRecipeFunctions_getInstanceRange(dpName, rangeElement, rangeMin, rangeMax, exceptionInfo);       
  if (rangeMin!="" && rangeMax!="")       
    range = "[" + rangeMin +", " + rangeMax + "]";  
	
  return range;
}


/** 
 * Get the measurement unit of a DPE.
 * @param dpName - [IN] The datapoint name.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
string unRecipeFunctions_getDpeUnit(string dpName, dyn_string &exceptionInfo) {
  string deviceType, dp, element, rangeElement;
  
  if (!dpExists(dpName)) return "";
  
  deviceType = dpTypeName(dpName);   
  dp=unGenericDpFunctions_getDpName(dpName);
  int pos = strpos(dpName, dp);
  element = substr(dpName, pos+strlen(dp), strlen(dpName)-strlen(dp));
  dyn_string splitElement = strsplit(element, ".");
       
  if (!mappingHasKey(unRecipe_rangeElements, deviceType+":"+element)){
    _unRecipeFunctions_getRangeElementFromPVSSDPE(dpName, rangeElement, exceptionInfo);
    if (dynlen(exceptionInfo))  return "";
    unRecipe_rangeElements[deviceType+":"+element] = rangeElement;
  } else {
    rangeElement = unRecipe_rangeElements[deviceType+":"+element];
  }
  
  //DebugTN("The Unit DPE of: " + dpName + " is: " + rangeElement + "\n");
  
  return dpGetUnit(dp + rangeElement);
}


/**
 * Gets the DPEs associated to each device type in a recipe type.
 * @param recipeType - [IN] Recipe type name.
 * @param map - [OUT] Mapping variable where Key=Device type name, Value=List containing the DPEs 
 *              included in the recipe type for the specified key (device type).
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getTypeMapping(string recipeType, mapping &map, dyn_string &exceptionInfo)
{
   dyn_dyn_string rtData;
  	_fwConfigurationDB_getRecipeTypeData(recipeType,rtData,exceptionInfo);
    
   if (dynlen(exceptionInfo)) return;
   
   int len = dynlen(rtData);
   for (int i=1; i<=len; i++){
     if (strtoupper(rtData[i][5])=="TRUE") {
        string deviceType = rtData[i][2];
        dyn_string items;
        if (mappingHasKey(map,deviceType)){
           items = map[deviceType];
        }
        dyn_string splitElement = strsplit(rtData[i][3], ".");
        dynAppend(items, splitElement[dynlen(splitElement)]);
        map[deviceType] = items;
     }
   }
}



/** 
 * Save the recipe values.
 * @param recipeObject - [IN] Recipe instance data.
 * @param values - [IN/OUT] Values for the recipe elements.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */ 
void _unRecipeFunctions_saveRecipeValues(dyn_dyn_mixed recipeObject, dyn_anytype &values, dyn_string &exceptionInfo)
{
  dynClear(exceptionInfo);
  
  dyn_anytype formattedValues;
  string instanceName = recipeObject[fwConfigurationDB_RO_META_ORIGNAME];
  dyn_string elements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  
  
  // The number of elements in the recipe must be the same as the number of values
  if (dynlen(elements)!=dynlen(values)){
    fwException_raise(exceptionInfo,"ERROR","unRecipeFunctions_saveRecipeValues: the number of elements int the recipe: " + 
                      recipeInstance + " is different than the number of values.","");
    return;      
  }  
  
  
/*  for (int i=1; i<=dynlen(elements); i++){
    string pvssDpe;
    unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(elements[i], pvssDpe);
    int type = dpElementType(pvssDpe);
    try {    
      int ival;
      float fval;
      bool bval;
      switch(type)
      {
        case DPEL_FLOAT: 
          Debug("Converting to FLOAT: " + values[i] + "\n");
          //fval = (float)values[i];
          sscanf(values[i], "%f", fval);
          Debug("Conversion OK: " + fval +"\n");
          break;        
        case DPEL_INT:
          Debug("Converting to INT: " + values[i] + "\n");
          //ival = (int)values[i];
          sscanf(values[i], "%d", ival);
          Debug("Conversion OK: " + ival +"\n");
          break;
        case DPEL_BOOL:
          Debug("Converting to BOOL: " + values[i] + "\n");
          //bval = (bool)values[i];
          sscanf(values[i], "%d", bval);
          Debug("Conversion OK: " + bval +"\n");
          break;
      }
    } catch {
      fwException_raise(exceptionInfo,"ERROR","The values introduced don't match the DPE types: "+recipeInstance,"");
      return;
    }     
  }  
*/  
//  _unRecipeFunctions_setDpValue(dpName, ".Values", values, exceptionInfo);
//  if (dynlen(exceptionInfo))  return;

  
  _unRecipeFunctions_formatValues(elements, values, formattedValues, exceptionInfo);
  if (dynlen(exceptionInfo))  return;

  //Debug("Values : " + values +"\n");
  //Debug("FValues: " + formattedValues + "\n");
  
  
  // Save the recipe values in the JCOP recipe
  recipeObject[fwConfigurationDB_RO_VALUE] = formattedValues;
  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONTIME] = getCurrentTime();
  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONUSER] = getUserName();
  fwConfigurationDB_saveRecipeToCache(recipeObject, "", instanceName, exceptionInfo);
  if (dynlen(exceptionInfo))   return;
  
  values = formattedValues;
  
  /*
  string recipeName = recipeClass +"/"+ recipeInstance;
  dyn_dyn_mixed recipeObject;
  fwConfigurationDB_loadRecipeFromCache (recipeName, "", "", recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo))   return;
  recipeObject[fwConfigurationDB_RO_VALUE] = formattedValues;
  fwConfigurationDB_saveRecipeToCache (recipeObject, "", recipeName, exceptionInfo);
  if (dynlen(exceptionInfo))   return;

  
  _unRecipeFunctions_setDpValue(dpName, ".LastModified", getCurrentTime(), exceptionInfo);
  if (dynlen(exceptionInfo))  return;
  
  string userName;
  fwAccessControl_getUserName(userName);
  int rc = dpSet(dpName+".LastModifiedUser", userName);
  if (rc) {
    fwException_raise(exceptionInfo,"ERROR","unRecipeFunctions_saveRecipeValues: " + 
                      "Error while setting the last modification user of the recipe: " + recipeInstance,"");
    return;    
  }
  */
}


/**
 * Get the element where the range is defined for a PVSS DPE
 * @param dpName - DPE name in the PVSS format (e.g. dist_1:gcs-CFP_STP_CVSF18-SF18-CPC_AnalogParameter-00001.ProcessOutput.ValRq)
 * @param element - Variable to store the element name that contains the range (e.g. .ProcessInput.CurValSt)
 * @param exceptionInfo - Standard exception handling variable.
 */
void _unRecipeFunctions_getRangeElementFromPVSSDPE(string dpName, string &element, dyn_string &exceptionInfo)
{
    string deviceType;
    dyn_string dpeInfo;
    
    element = "";
    dynClear(exceptionInfo);

//    Debug("Searching the Range Element of: " + dpName +"\n");
    
    if (!dpExists(dpName)) {
      //Debug("The DPE doesn't exist: " + dpName +"\n");
      fwException_raise(exceptionInfo,"ERROR","The data point element: "+dpName+
                        " doesn't exist.",unRecipeFunctions_ERROR_InvalidElements);
      return;
    }
    
    deviceType = dpTypeName(dpName);
    int pos = strpos(dpName, ".");
    if (pos<0) {
 //     Debug("The elements defined in the recipe class: "+recipeClass+" are wrong.\n");
 /*     fwException_raise(exceptionInfo,"ERROR","The elements defined in the recipe class: "+recipeClass+
                        " are wrong.",unRecipeFunctions_ERROR_InvalidElements);
   */   return;
    }

    if (!dpExists(deviceType+"_RcpInfo.DpeInfo")) {
      // If the information about what is the DPE containing the range definition doesn't exist,
      // returns the DPE name
	  //DebugTN("The " + deviceType+"_RcpInfo.DpeInfo DPE doesn't exist!");	  
      string dp=unGenericDpFunctions_getDpName(dpName);
      int pos = strpos(dpName, dp);
      element = substr(dpName, strlen(dp), strlen(dpName)-strlen(dp));
      //DebugTN("RANGE ELEMENT: " + element +"\n");
      return;
    }
        
    element = substr(dpName, pos, strlen(dpName)-pos);
    dpGet(deviceType+"_RcpInfo.DpeInfo", dpeInfo);

    for (int i=1; i<=dynlen(dpeInfo); i++){
      string currDpeInfo = dpeInfo[i];
      dyn_string splitInfo = strsplit(currDpeInfo, ":");
      if (dynlen(splitInfo)!=2) {
 //       Debug("The recipe info for the device type: "+deviceType+" is wrong.\n");
        fwException_raise(exceptionInfo,"ERROR","The recipe info for the device type: "+deviceType+
                          " is wrong.",unRecipeFunctions_ERROR_InvalidElements);
        return;        
      }
      
      splitInfo[1] = strltrim(splitInfo[1]);
      splitInfo[1] = strrtrim(splitInfo[1]);
      if (splitInfo[1] == element) {
        element = strltrim(splitInfo[2]);
        element = strrtrim(element);
        return;
      }
    }
    
 //   Debug("The range element for: " + dpName + " is: " +element +"\n");
}


/**
 * Gets the min. and max. values defined in a range for a DPE.
 * @param dpe - [IN] Data point element for whom the range values are required.
 * @param rangeElement - [IN] DPE where the range is defined for the 'dpe' parameter (e.g.: ".ProcessInput.CurValSt")
 * @param rangeMin - [OUT] Minimum value of the range definition.
 * @param rangeMax - [OUT] Maximim value of the range definition.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getInstanceRange(string dpe, string rangeElement, string &rangeMin, 
                                         string &rangeMax, dyn_string &exceptionInfo)
{
  //Debug ("Getting instance range for DPE: " + dpe +"\n");
  string dpName = unGenericDpFunctions_getDpName(dpe)+rangeElement;
  if (!dpExists(dpName)){
    fwException_raise(exceptionInfo,"ERROR","The specified data point doesn't exist: "+dpName,"");
    return;        
  }
  
  string min="",max="";
  dpGet(dpName+":_pv_range.._min", min,
        dpName+":_pv_range.._max", max);
  
  //Debug("DpGet: " + dpName+":_pv_range.._min" +"\n");
  //Debug("DpGet: " + dpName+":_pv_range.._max" +"\n");
  
  if (min=="" && max=="") {
    rangeMin="";
    rangeMax="";
  } else {
    string format = dpGetFormat(dpName);
    rangeMin = unGenericObject_FormatValue(format, min);
    rangeMax = unGenericObject_FormatValue(format, max);     
  }
  
  //Debug("RangeMin: " + rangeMin +"\n");
  //Debug("RangeMax " + rangeMax +"\n");
}  



/** 
 * Activate a recipe.
 * @param sRcpInstanceDp - [IN] Data point name of the recipe which activation is triggered.
 * @param exceptionInfo  - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_activateRecipe(string sRcpInstanceDp, dyn_string &exceptionInfo) 
{
  mapping systemDpes, plcDpes;
  dyn_dyn_mixed rcpInstance;
  dyn_string sStatusMessage;
  string sRcpClassName, sRcpInstanceName;
  time t0, t1;
  bool bActive;

  // Disables the recipe class combo box, the recipe instances table and the recipe elements table
  setValue("RecipeClasses", "enabled", false);
  setValue("RecipeInstances", "enabled", false);
  setValue("RecipeElements", "enabled", false);
  
  dpSet(sRcpInstanceDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_ACTIVATE);
  
  dynClear(exceptionInfo);  
  t0 = getCurrentTime();
   
  if (mappinglen(unRecipe_unRecipe_pidManRegBits)==0 || mappinglen(unRecipe_aalarmManRegBits)==0)
    _unRecipeFunctions_initializeManRegBits();
  
  dpGet(sRcpInstanceDp+".ProcessInput.ClassName", sRcpClassName,
		sRcpInstanceDp+".ProcessInput.InstanceName", sRcpInstanceName);  
  
  fwConfigurationDB_loadRecipeFromCache(sRcpClassName + "/" +sRcpInstanceName, makeDynString(), "", rcpInstance, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
    setValue("RecipeClasses", "enabled", true);
    setValue("RecipeInstances", "enabled", true);
    setValue("RecipeElements", "enabled", true);
	
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	// Show the error message  
	DebugTN(exceptionInfo); 
	return; 
  }
  
  // Check if all the PLCs are connected
  unRecipeFunctions_getPlcStatus(rcpInstance[fwConfigurationDB_RO_DPE_NAME], bActive, sStatusMessage, exceptionInfo);
  if (!bActive) {
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
    setValue("RecipeClasses", "enabled", true);
    setValue("RecipeInstances", "enabled", true);
    setValue("RecipeElements", "enabled", true);
	// Show the error message
	_unRecipeFunctions_writeInRecipeLog("There isn't connection with the PLC(s): " + sStatusMessage + ". The recipe " + sRcpClassName + " / " + sRcpInstanceName + " can't be activated");
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	return;
  }
  
  unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_TRIGGERED);
  _unRecipeFunctions_writeInRecipeLog("Activating recipe: " + sRcpClassName + " / " + sRcpInstanceName);
  
  // Format all the recipe values and saves the formatted values 
  // This is necessary to avoid problems if the formats have been modified and the recipe values don't have the correct formats
  _unRecipeFunctions_formatValues(rcpInstance[fwConfigurationDB_RO_DPE_NAME], rcpInstance[fwConfigurationDB_RO_VALUE], 
                                     rcpInstance[fwConfigurationDB_RO_VALUE], exceptionInfo);
  fwConfigurationDB_saveRecipeToCache 	(rcpInstance, "", sRcpClassName +"/" + sRcpInstanceName, exceptionInfo);
  if (dynlen(exceptionInfo)) {
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
    setValue("RecipeClasses", "enabled", true);
    setValue("RecipeInstances", "enabled", true);
    setValue("RecipeElements", "enabled", true);
	// Show the error message
	DebugTN(exceptionInfo);
	_unRecipeFunctions_writeInRecipeLog("Error formatting the recipe values: the recipe activation is cancelled.");
	unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	return;
  }
  
  // Gets the recipe elements by PLC
  _unRecipeFunctions_getElementsByPLC(rcpInstance[fwConfigurationDB_RO_DPE_NAME], 
									  rcpInstance[fwConfigurationDB_RO_VALUE], 
									  plcDpes, 
									  exceptionInfo);
									  
  if (dynlen(exceptionInfo))  { 
    // Enables the recipe class combo box, the recipe instances table and the recipe elements table
    setValue("RecipeClasses", "enabled", true);
    setValue("RecipeInstances", "enabled", true);
    setValue("RecipeElements", "enabled", true);
	// Show the error message
    _unRecipeFunctions_writeInRecipeLog("Recipe can't be activated: The number of recipe elements is bigger than the recipe buffer size.");
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	DebugTN(exceptionInfo); 
	return;
  }
  
  // Recipe activation
  _unRecipeFunctions_activateRecipeByPLC(plcDpes, sRcpInstanceDp, exceptionInfo);  
  
  // Enables the recipe class combo box, the recipe instances table and the recipe elements table
  setValue("RecipeClasses", "enabled", true);
  setValue("RecipeInstances", "enabled", true);
  setValue("RecipeElements", "enabled", true);
  
  // Reset the recipe state
  if (dynlen(exceptionInfo))  {
	dpSet(sRcpInstanceDp+".ProcessInput.State", "");
	DebugTN(exceptionInfo); 
	return;
  }
  
  
  t1 = getCurrentTime();
  string elapsedTime = formatTime("%S", t1-t0, " milli: %04d");
  //Debug ("Activation Elapsed Time: " + elapsedTime +"\n");	
}



/**
 * Gets the recipe elements according to the PLC where the elements are located.
 * @param elements - [IN] Recipe data point elements.
 * @param value - [IN] Recipe values.
 * @param plcDpes - [OUT] Map containing the recipe elements and values for each PLC.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */  
void _unRecipeFunctions_getElementsByPLC(dyn_string elements, dyn_anytype values, 
                                         mapping &plcDpes, dyn_string &exceptionInfo)
{
  int rc, len, iElements, iBufferLength;
  string sBufferDpname, sPlcName, sSystemName;
  dyn_dyn_string data;
  mappingClear(plcDpes);
  
  int len = dynlen(elements);
  if (len!=dynlen(values)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of DPEs and values of the recipe must be equal.", "");
    return;
  }  
  
  // Inserts the DPEs in the mapping according to the PLC name
  for (int i=1; i<=len; i++) {
    sSystemName = unGenericDpFunctions_getSystemName(elements[i]);
    sPlcName = unGenericObject_GetPLCNameFromDpName(elements[i]);
    
    dynClear(data);
    if (mappingHasKey(plcDpes, sSystemName+sPlcName)) {
      data = plcDpes[sSystemName+sPlcName];
    }

    dynAppend(data[1], elements[i]);
    dynAppend(data[2], values[i]);
    plcDpes[sSystemName+sPlcName] = data;
  }    
  
  // Check that the number of elements in the recipe (for each PLC) isn't bigger than the recipe buffers size
  len = mappinglen(plcDpes);
  for (int i=1; i<=len; i++) {
    data = mappingGetValue(plcDpes, i);
	iElements = dynlen(data[1]);
	sPlcName = mappingGetKey(plcDpes, i);
	
	_unRecipeFunctions_getRecipeBuffersName(sPlcName, sBufferDpname, exceptionInfo);
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }

	rc = dpGet(sBufferDpname + "ProcessInput.BuffersLength", iBufferLength);
	if (iElements > iBufferLength) {
		fwException_raise(exceptionInfo, "ERROR", "The number of recipe elements (" + iElements + ") is bigger than the PLC buffers length (" + iBufferLength + ") for the PLC: " + sPlcName, "");
		return;
	}
  }
}


/** 
 * Activates a recipe in each PLC where the recipe is defined.
 * @param plcDpes - [IN] Mapping where Key=PLC name, Value=Array with two positions where:
 *                   1: contains the DPEs of the recipe in the PLC,
 *                   2: contains the recipe values for each DPE.
 * @param exceptionInfo - [OUT] Standard exception handling variable.                     
 */ 
void _unRecipeFunctions_activateRecipeByPLC(mapping plcDpes, string sRcpInstanceDp, dyn_string &exceptionInfo)
{  
  //Debug("Init: Activate Recipe by PLC\n");
  int len = mappinglen(plcDpes);
  for (int i=1; i<=len; i++) {
    string plcName = mappingGetKey(plcDpes, i);
    string plcType = dpTypeName(plcName); 
    dyn_dyn_string recipeData = mappingGetValue(plcDpes, i);  
    
    if (plcType==UN_PLC_DPTYPE){
      // Recipe activation for SCHNEIDER PLC
      _unRecipeFunctions_activateSchneiderRecipe(plcName, sRcpInstanceDp, recipeData[1], recipeData[2], exceptionInfo);
    } else if (plcType==S7_PLC_DPTYPE) {
      // Recipe activation for SIEMENS PLC 
      _unRecipeFunctions_activateSiemensRecipe(plcName, sRcpInstanceDp, recipeData[1], recipeData[2], exceptionInfo);
    } else {
      // Unknown PLC
      fwException_raise(exceptionInfo, "ERROR", "The PLC type is unknown for the PLC:" + plcName, "");
      return;      
    }
  }
 // Debug("End: Activate Recipe by PLC\n");
}


/** 
 * Activates a recipe in a Schneider PLC.
 * @param plcName - [IN] PLC name.
 * @param dpes - [IN] List of recipe DPEs for the 'plcName'.
 * @param values - [IN] List of values for the recipe DPEs.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_activateSchneiderRecipe(string plcName, string sRcpInstanceDp, dyn_string dpes, 
                                                dyn_string values, dyn_string &exceptionInfo)
{
  string sSystemName, sRecipeState, dpBuffersAlias, dpBuffersName, sRemainingTime;
  bool bIsRemote, bIsConnected, bActivationProgressBarExists, bSelectionProgressBarExists;
  time tStart, tEnd, tCurrent;
  int rc, iActivationTimeout, iProgressBarValue=0, iSleepTime=1, iTimeOut;
  
  //Debug("Init: Activate Schneider PLC\n");
  
  // Check if the current PLC is in a distributed system and if it's connected
  sSystemName = unGenericDpFunctions_getSystemName(plcName);
  unDistributedControl_isRemote(bIsRemote, sSystemName);
  if (bIsRemote) {
	unDistributedControl_isConnected(bIsConnected, sSystemName);
	if (!bIsConnected){
		fwException_raise(exceptionInfo, "ERROR", "The distributed system '" + sSystemName + "' isn't connected.", "");
		return;
	}
  }
  
  // Get the recipe buffers name for the current PLC
  _unRecipeFunctions_getRecipeBuffersName(plcName, dpBuffersAlias, exceptionInfo);  
  if (dynlen(exceptionInfo))  { DebugTN(exceptionInfo); return; }
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dpBuffersAlias, dpBuffersName);

 
  // Get the activation timeout from the recipe buffers
  rc = dpGet(dpBuffersName+"ProcessInput.ActivationTimeout", iActivationTimeout);
  
  // Activation timeout
  tStart = getCurrentTime();
  tEnd   = makeTime((unsigned)year(tStart), 
                    (unsigned)month(tStart), 
                    (unsigned)day(tStart), 
                    (unsigned)hour(tStart), 
                    (unsigned)minute(tStart), 
                    (unsigned)second(tStart)+(unsigned)iActivationTimeout);
			
			
  // Initializes the recipe activation progress bar
  if (bActivationProgressBarExists = shapeExists("ActivationProgressBar")) { 
    unProgressBar_setMax("ActivationProgressBar", iActivationTimeout);
	iProgressBarValue = iActivationTimeout;
	unProgressBar_setPosition("ActivationProgressBar", iProgressBarValue);
  }  
  
  // if the selection timeout is less than the activation timeout, selects the recipe instance
  dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime,
        getSystemName() + "_unSelectDeselect.timeOut", iTimeOut);
  if (iActivationTimeout > (int)(strsplit(sRemainingTime, ";")[3])) {
	unGenericObject_ButtonSelectAction($sDpName, true, exceptionInfo);
	if (shapeExists("SelectionProgressBar")) {
		unProgressBar_setPosition("SelectionProgressBar", iTimeOut);
	}
  }
 
  _unRecipeFunctions_createBuffersData(plcName, dpes, values, exceptionInfo);
  if (dynlen(exceptionInfo)) {DebugTN(exceptionInfo); return;}
  _unRecipeFunctions_writeInRecipeLog("Sending recipe data to the PLC " + plcName);
  _unRecipeFunctions_sendRecipeHeader(plcName, exceptionInfo);

  //DebugTN("DPConnect on recipe bufers: " + dpBuffersName+"ProcessInput.Status.recipeStatus");
  //Debug("DP CONNECT FALSE\n");
  dpConnect("_unRecipeFunctions_checkRecipeStatusBufferCB", false,
				dpBuffersName+"ProcessInput.Status.RecipeStatus", 
				sRcpInstanceDp+".ProcessInput.State");
	
  // Loop until the recipe activation is finished or the timeout has expired
  while ((tCurrent = getCurrentTime()) < tEnd) {
    dpGet(sRcpInstanceDp+".ProcessInput.State", sRecipeState);
	if (sRecipeState!=UN_RCP_INSTANCE_STATE_ACTIVATE) {
		// Recipe activation finished
		break;
	}
	
	delay(iSleepTime);
	
	// Updates the Activation Timeout progress bar
	if (bActivationProgressBarExists) {	
		iProgressBarValue = iProgressBarValue - iSleepTime;
		unProgressBar_setPosition("ActivationProgressBar", iProgressBarValue);
	}	
	
	// if the selection timeout is less than the activation timeout, selects the recipe instance
	dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime);
	if (iProgressBarValue > (int)(strsplit(sRemainingTime, ";")[3])) {
		unGenericObject_ButtonSelectAction($sDpName, true, exceptionInfo);
	}
  } 
  
  
  // if the activation timeout has expired, cancel the recipe activation
  if (tCurrent >= tEnd && sRecipeState==UN_RCP_INSTANCE_STATE_ACTIVATE) {
	_unRecipeFunctions_cancelRecipeActivation (plcName, dpBuffersName, sRcpInstanceDp+".ProcessInput.State", "Timeout expired");
  }
  
  // Resets the progress bar
  if (bActivationProgressBarExists) {
	unProgressBar_setPosition("ActivationProgressBar", 0);
  }
	
  //Debug("End: Activate Schneider PLC\n");
}


/** 
 * Activates a recipe in a Siemens PLC.
 * @param plcName - [IN] PLC name.
 * @param recipeId - [IN] Recipe ID number.
 * @param dpes - [IN] List of recipe DPEs for the 'plcName'.
 * @param values - [IN] List of values for the recipe DPEs.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_activateSiemensRecipe(string plcName, string sRcpInstanceDp, dyn_string dpes, 
                                              dyn_string values, dyn_string &exceptionInfo)
{
  string sSystemName, sRecipeState, dpBuffersAlias, dpBuffersName, sRemainingTime;
  bool bIsRemote, bIsConnected, bActivationProgressBarExists, bSelectionProgressBarExists;
  time tStart, tEnd, tCurrent;
  int rc, iActivationTimeout, iProgressBarValue=0, iSleepTime=1, iTimeOut;
  
  //Debug("Init: Activate Siemens PLC\n");
  
  // Check if the current PLC is in a distributed system and if it's connected
  sSystemName = unGenericDpFunctions_getSystemName(plcName);
  unDistributedControl_isRemote(bIsRemote, sSystemName);
  if (bIsRemote) {
	unDistributedControl_isConnected(bIsConnected, sSystemName);
	if (!bIsConnected){
		fwException_raise(exceptionInfo, "ERROR", "The distributed system '" + sSystemName + "' isn't connected.", "");
		return;
	}
  }
  
  // Get the recipe buffers name for the current PLC
  _unRecipeFunctions_getRecipeBuffersName(plcName, dpBuffersAlias, exceptionInfo);  
  if (dynlen(exceptionInfo))  { DebugTN(exceptionInfo); return; }
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dpBuffersAlias, dpBuffersName);

 
  // Get the activation timeout from the recipe buffers
  rc = dpGet(dpBuffersName+"ProcessInput.ActivationTimeout", iActivationTimeout);
  
  // Activation timeout
  tStart = getCurrentTime();
  tEnd   = makeTime((unsigned)year(tStart), 
                    (unsigned)month(tStart), 
                    (unsigned)day(tStart), 
                    (unsigned)hour(tStart), 
                    (unsigned)minute(tStart), 
                    (unsigned)second(tStart)+(unsigned)iActivationTimeout);
			
			
  // Initializes the recipe activation progress bar
  if (bActivationProgressBarExists = shapeExists("ActivationProgressBar")) { 
    unProgressBar_setMax("ActivationProgressBar", iActivationTimeout);
	iProgressBarValue = iActivationTimeout;
	unProgressBar_setPosition("ActivationProgressBar", iProgressBarValue);
  }  
  
  // if the selection timeout is less than the activation timeout, selects the recipe instance
  dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime,
        getSystemName() + "_unSelectDeselect.timeOut", iTimeOut);
  if (iActivationTimeout > (int)(strsplit(sRemainingTime, ";")[3])) {
	unGenericObject_ButtonSelectAction($sDpName, true, exceptionInfo);
	if (shapeExists("SelectionProgressBar")) {
		unProgressBar_setPosition("SelectionProgressBar", iTimeOut);
	}
  }
 
  _unRecipeFunctions_createBuffersData(plcName, dpes, values, exceptionInfo);
  if (dynlen(exceptionInfo)) {DebugTN(exceptionInfo); return;}
  _unRecipeFunctions_createRecipeHeaderDBNumber(plcName, dpes, exceptionInfo); 
  if (dynlen(exceptionInfo)) {DebugTN(exceptionInfo); return;}  
  _unRecipeFunctions_writeInRecipeLog("Sending recipe data to the PLC " + plcName);
  _unRecipeFunctions_sendRecipeHeader(plcName, exceptionInfo);

  //DebugTN("DPConnect on recipe bufers: " + dpBuffersName+"ProcessInput.Status.recipeStatus");
  //Debug("DP CONNECT FALSE\n");
  dpConnect("_unRecipeFunctions_checkRecipeStatusBufferCB", false,
				dpBuffersName+"ProcessInput.Status.RecipeStatus", 
				sRcpInstanceDp+".ProcessInput.State");
	
  // Loop until the recipe activation is finished or the timeout has expired
  while ((tCurrent = getCurrentTime()) < tEnd) {
    dpGet(sRcpInstanceDp+".ProcessInput.State", sRecipeState);
	if (sRecipeState!=UN_RCP_INSTANCE_STATE_ACTIVATE) {
		// Recipe activation finished
		break;
	}
	
	delay(iSleepTime);
	
	// Updates the Activation Timeout progress bar
	if (bActivationProgressBarExists) {	
		iProgressBarValue = iProgressBarValue - iSleepTime;
		unProgressBar_setPosition("ActivationProgressBar", iProgressBarValue);
	}	
	
	// if the selection timeout is less than the activation timeout, selects the recipe instance
	dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime);
	if (iProgressBarValue > (int)(strsplit(sRemainingTime, ";")[3])) {
		unGenericObject_ButtonSelectAction($sDpName, true, exceptionInfo);
	}
  } 
  
  
  // if the activation timeout has expired, cancel the recipe activation
  if (tCurrent >= tEnd && sRecipeState==UN_RCP_INSTANCE_STATE_ACTIVATE) {
	_unRecipeFunctions_cancelRecipeActivation (plcName, dpBuffersName, sRcpInstanceDp+".ProcessInput.State", "Timeout expired");
  }
  
  // Resets the progress bar
  if (bActivationProgressBarExists) {
	unProgressBar_setPosition("ActivationProgressBar", 0);
  }
	
  //Debug("End: Activate Siemens PLC\n");
}



void _unRecipeFunctions_sendRecipeHeader(string sPlcName, dyn_string &exceptionInfo)
{
  string sDpName, sSystemName;
  dyn_string recipeHeaderDpes;
  
  _unRecipeFunctions_getRecipeBuffersName(sPlcName, sDpName, exceptionInfo);
  //DebugTN("_unRecipeFunctions_sendRecipeHeader - START\n");
  
  // Creates the strings to represent the recipe header dpes
  for (int i=1; i<=RCP_HEADER_MAX_SIZE; i++) {
	recipeHeaderDpes[i] = sDpName + "ProcessOutput.Header[" + i + "]";
  }
 
  if (strpos(sPlcName, ":") < 0) {
	// Adds the system name to the PLC name
	sSystemName = unGenericDpFunctions_getSystemName(sPlcName);
	sPlcName = sSystemName + sPlcName;
  }
  
/*  DebugTN("PLCName: " + sPlcName +"\n");
  DebugTN("RcpHeaderDPEs: " + dynlen(recipeHeaderDpes) + "   RcpHeaderValues: " + dynlen(_unRecipeFunctions_getHeaderBuffer(sPlcName)) + "\n");
  DebugTN("Recipe Header DPEs: " + recipeHeaderDpes + " \n");
  DebugTN("RcpHeader: " + _unRecipeFunctions_getHeaderBuffer(sPlcName) + "\n");
*/
  dpSetWait(recipeHeaderDpes, _unRecipeFunctions_getHeaderBuffer(sPlcName));
 
//  DebugTN("_unRecipeFunctions_sendRecipeHeader - END\n");
}



void _unRecipeFunctions_checkRecipeStatusBufferCB(string sRecipeStatusDp, int recipeStatus,
												  string sRecipeState, string sState) 
{
  int pos;
  bool checkResult;
  string sBuffersDpName, plcName, sActivationStatus, errDpe, pvssDpe;
  dyn_int recipeHeaderBuffer;
  dyn_string exceptionInfo;
  dyn_dyn_mixed recipeObject; 
    	
  // Get the PLC name
  plcName = unGenericObject_GetPLCNameFromDpName(sRecipeStatusDp);
  if (strpos(plcName, ":")<0)
	plcName = unGenericDpFunctions_getSystemName(sRecipeStatusDp) + plcName;
  
  //DebugTN("PLC: " + plcName + "\n");
  
  // Get the PLC buffers dp name
  pos = strpos(sRecipeStatusDp, "ProcessInput");
  if (pos>=0) {
	sBuffersDpName = substr(sRecipeStatusDp, 0, pos);
  }
	  
  switch (recipeStatus){
    case RCP_ACT_STATUS_ERROR:
      //DebugTN("Recipe activation - error");
	  _unRecipeFunctions_writeInRecipeLog("Recipe activation error in PLC " + plcName);
      dpDisconnect("_unRecipeFunctions_checkRecipeStatusBufferCB", sRecipeStatusDp, sRecipeState);
	  strreplace(sRecipeState, ":_online.._value", "");
	  dpSet(sRecipeState, "");
	  _unRecipeFunctions_cancelRecipeActivation(plcName, sBuffersDpName, sRecipeState, "PLC Error");
	  unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
      break;
  
    case RCP_ACT_STATUS_STANDBY:
      //DebugTN("Recipe activation - in standby");
	 // _unRecipeFunctions_writeInRecipeLog("Recipe activation - in standby");
      break;
      
    case RCP_ACT_STATUS_NEW_ID_RECEIVED:
      //DebugTN("Recipe activation - New Id received");
	 // _unRecipeFunctions_writeInRecipeLog("Recipe activation - new id received in PLC");
	  checkResult = _unRecipeFunctions_checkRecipeStatusBufferData(sBuffersDpName, exceptionInfo);
	  if (false==checkResult) {
		// Wrong data in the recipe status buffer
		_unRecipeFunctions_writeInRecipeLog("Activation error in PLC " + plcName + ": " + exceptionInfo[2]);
	    //DebugTN("The data in the status buffer is Wrong: ", exceptionInfo);
		strreplace(sRecipeState, ":_online.._value", "");
		_unRecipeFunctions_cancelRecipeActivation(plcName, sBuffersDpName, sRecipeState, "Bad status data");
	  } else {
		// The data in the recipe status buffer is Ok
		// 1. Send the recipe data (ManRegAddr, ManRegVal, ReqAddr and ReqVal buffers)
		_unRecipeFunctions_sendData(plcName, exceptionInfo);
		// 2. Send the recipe header buffer
		recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(plcName);
		recipeHeaderBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_DATA_SENT;
		gHeaderMap[plcName] = recipeHeaderBuffer;
		_unRecipeFunctions_sendRecipeHeader(plcName, exceptionInfo);
	  }
      break;

    case RCP_ACT_STATUS_CRC_OK:
      //DebugTN("Recipe activation - CRC Ok");
	  //_unRecipeFunctions_writeInRecipeLog("Recipe activation - CRC Ok");
	  checkResult = _unRecipeFunctions_checkRecipeStatusBufferData(sBuffersDpName, exceptionInfo);
	  if (false==checkResult) {
	    // Wrong data in the recipe status buffer
		_unRecipeFunctions_writeInRecipeLog("Activation error in PLC " + plcName + ": " + exceptionInfo[2]);
	    //DebugTN("The data in the status buffer is Wrong: ", exceptionInfo);
		strreplace(sRecipeState, ":_online.._value", "");
		_unRecipeFunctions_cancelRecipeActivation(plcName, sBuffersDpName, sRecipeState, "Bad status data");
	  } else {
		// 1. Send the activation order 
		recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(plcName);
		recipeHeaderBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_ACTIVATE;
		gHeaderMap[plcName] = recipeHeaderBuffer;
		_unRecipeFunctions_sendRecipeHeader(plcName, exceptionInfo);  
	  }
      break;      

    case RCP_ACT_STATUS_DONE:
      dpDisconnect("_unRecipeFunctions_checkRecipeStatusBufferCB", sRecipeStatusDp, sRecipeState);
	  
	  // TODO: Is enough this delay?
	  delay(1);
	  
	  // Check if the recipe has been activated properly
	  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeState, recipeObject, exceptionInfo);
	  if (dynlen(exceptionInfo)) {DebugTN(exceptionInfo); return;}
	  dyn_string rcpElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
	  dyn_string rcpValues   = recipeObject[fwConfigurationDB_RO_VALUE];
	  
	  if (_unRecipeFunctions_checkRecipeOnlineValues(rcpElements, rcpValues, errDpe)) {
		// Updates the recipe meta info
		recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONTIME] = getCurrentTime();
		recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONUSER] = getUserName();
		_fwConfigurationDB_activateRecipeUpdateMetaInfo(recipeObject, exceptionInfo);
		string sRcpDp = sRecipeState;
		strreplace(sRcpDp, ".ProcessInput.State:_online.._value", "");
	    _unRecipeFunctions_loadPropertiesTableInfo(sRcpDp, recipeObject);
	    _unRecipeFunctions_loadHistoryTableInfo(sRcpDp, recipeObject);
	    _unRecipeFunctions_writeInRecipeLog("Recipe activation completed in PLC " + plcName);
	  } else {
	    unGenericDpFunctions_convert_PVSSDPE_to_UNICOSDPE(errDpe, errDpe);
		unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
	    _unRecipeFunctions_writeInRecipeLog("Recipe activation failed in PLC " + plcName + ": Recipe values are different than online values");		
		_unRecipeFunctions_writeInRecipeLog("The first different value corresponds to the data point: " + errDpe);
		
		if (patternMatch("*.MOutH", errDpe) || patternMatch("*.MOutL", errDpe)) {
		  _unRecipeFunctions_writeInRecipeLog("Please, verify that the recipe value is defined in the controlled object range.");
		} 
		else if (patternMatch("*.MSPH", errDpe) || patternMatch("*.MSPL", errDpe)) {
		  _unRecipeFunctions_writeInRecipeLog("Please, verify that the recipe value is defined in the measured value range.");
		}
	  }
	  
	  // Resets the recipe instance state
	  strreplace(sRecipeState, ":_online.._value", "");
	  dpSet(sRecipeState, "");
      break;
	  
	default:
	  //DebugTN("Recipe activation - ERROR CODE RECEIVED!!!");
	  _unRecipeFunctions_writeInRecipeLog("Recipe activation error: Unknown recipe status value (" + recipeStatus + ")");
  }
}


/**
 * Checks the data in the recipe status buffer to verify:
 * <ul>
 * 	<li>The received data corresponds to the data in the recipe header buffer.</li>
 *	<li>The timestamp received is synchronized with the PLC: PVSS Time +/- PLC Delay = received timestamp
 * </ul>
 * @param sBuffersDpName - [IN] DpName of the recipe buffers.
 * @param exceptionInfo  - [OUT] Standard exception handling variable.
 */
bool _unRecipeFunctions_checkRecipeStatusBufferData(string sBuffersDpName, dyn_string &exceptionInfo)
{
	string sPlcName, sPlcType, sDelayDpName;
	time timeMin, timeMax, currentTime = getCurrentTime();
	dyn_string timestampDpes, statusValueDpes, headerValueDpes;
	dyn_string statusValues, headerValues, sDelay;
	dyn_time timestamps;
	
	// Creates strings containing the DPEs for: 
	//	 - Timestamp of the status buffer data
	//   - Status buffer data
	//   - Header buffer data
	int i=1;
	while (i<=RCP_STATUS_MAX_SIZE && i<=RCP_STATUS_MAX_SIZE) {
		timestampDpes[i]   = sBuffersDpName + "ProcessInput.Status[" + i + "]:_online.._stime";
		statusValueDpes[i] = sBuffersDpName + "ProcessInput.Status[" + i + "]:_online.._value";
		headerValueDpes[i] = sBuffersDpName + "ProcessOutput.Header[" + i + "]:_online.._value";
		i++;
	}
	
	// Get the PLC Delay DPName
	string sPlcName = unGenericObject_GetPLCNameFromDpName(sBuffersDpName);
	sPlcType = dpTypeName(sPlcName);
	if (sPlcType==S7_PLC_DPTYPE) {
		sDelayDpName = "unicosS7PLC_systemIntegrityInfo.config.data";
	} else if (sPlcType==UN_PLC_DPTYPE) {
		sDelayDpName = "unicosPLC_systemIntegrityInfo.config.data";
	} else {
		// Unknown PLC type
		fwException_raise(exceptionInfo, "ERROR", "The PLC type is unknown: " + sPlcType +".", "");
		return false;
	}

	// Get all the necessary values
	delay(2);
	int rc = dpGet(timestampDpes, timestamps, statusValueDpes, statusValues, headerValueDpes, headerValues, sDelayDpName, sDelay);
	if (rc<0) {
		fwException_raise(exceptionInfo, "ERROR", "DpGet failed in _unRecipeFunctions_checkRecipeStatusBufferData() function.", "");
		return false;	
	}
	
	// Check that the received timestamp is in a valid range
	timeMin = makeTime((unsigned)year(currentTime), 
                       (unsigned)month(currentTime), 
                       (unsigned)day(currentTime), 
                       (unsigned)hour(currentTime), 
                       (unsigned)minute(currentTime), 
                       (unsigned)second(currentTime)-(unsigned)sDelay[1]);
	timeMax = makeTime((unsigned)year(currentTime), 
                       (unsigned)month(currentTime), 
                       (unsigned)day(currentTime), 
                       (unsigned)hour(currentTime), 
                       (unsigned)minute(currentTime), 
                       (unsigned)second(currentTime)+(unsigned)sDelay[1]);
	
	if (timestamps[RCP_STATUS_STATUS]<timeMin || timestamps[RCP_STATUS_STATUS]>timeMax){
		fwException_raise(exceptionInfo, "ERROR", "The timestamp of the received data is out of range.", "");
		return false;
	}
		
	// Check that all the 'mirrored' values corresponds to the values in the recipe header buffer
	for (int i=1; i<=RCP_STATUS_MAX_SIZE && i<=RCP_STATUS_MIRROR_DATA_SIZE; i++) {
		if (headerValues[i]!=statusValues[i]) {
			fwException_raise(exceptionInfo, "ERROR", "The status buffer values are different than the header buffer values.", "");
			return false;
		}
	}

	return true;
}



void _unRecipeFunctions_cancelRecipeActivation (string sPlcName, string sBuffersDpName, string sRecipeState, string reason, bool disconnect=true) 
{
	dyn_string exceptionInfo;

	//Debug("_Cancelling recipe activation\n");
	//Debug("BuffersDpName: " + sBuffersDpName + "\n");
	//Debug("sRecipeState: " + sRecipeState + "\n");
	
	if (strpos(sPlcName, ":") < 0) {
		// Adds the system name to the PLC name
		sSystemName = unGenericDpFunctions_getSystemName(sPlcName);
		sPlcName = sSystemName + sPlcName;
    }
  
	dyn_int headerBuffer = _unRecipeFunctions_getHeaderBuffer(sPlcName);
	
	if (disconnect) {
		dpDisconnect("_unRecipeFunctions_checkRecipeStatusBufferCB", 
						sBuffersDpName+"ProcessInput.Status.RecipeStatus", 
						sRecipeState);  
	}

	headerBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_CANCEL;
	gHeaderMap[sPlcName] = headerBuffer;
	_unRecipeFunctions_sendRecipeHeader(sPlcName, exceptionInfo);
	dpSet(sRecipeState, "");
	unRecipeFunctions_setRecipeStatusInfo(RCP_ACTIVATION_FAILED);
		
	//Debug("_Recipe activation cancelled!\n");
	_unRecipeFunctions_writeInRecipeLog("Recipe activation cancelled in PLC " + sPlcName + ": " + reason);
}


void unRecipeFunctions_cancelRecipeActivation(string sRecipeDpName, dyn_string &exceptionInfo) 
{
	int i, iLen;
	string sRcpBuffersDp, sPlcName, sRecipeState;
	dyn_dyn_mixed recipeObject;
	mapping rcpPlcs;
	
	//Debug("Cancelling recipe activation\n");
	
	_unRecipeFunctions_getRecipeObjectFromDp (sRecipeDpName, recipeObject, exceptionInfo);
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
	
	_unRecipeFunctions_getElementsByPLC(recipeObject[fwConfigurationDB_RO_DPE_NAME], recipeObject[fwConfigurationDB_RO_VALUE], rcpPlcs, exceptionInfo);
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
	
	iLen = mappinglen(rcpPlcs);
	for (int i=1; i<=iLen; i++) {
		sPlcName = mappingGetKey(rcpPlcs, i);
		if (strpos(sPlcName, ":") < 0 ) {
			// Adds the system name to the PLC name
			sSystemName = unGenericDpFunctions_getSystemName(sPlcName);
			sPlcName = sSystemName + sPlcName;
		}
		
		_unRecipeFunctions_getRecipeBuffersName(sPlcName, sRcpBuffersDp, exceptionInfo);
		if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
		
		sRecipeState = sRecipeDpName + ".ProcessInput.State";
		_unRecipeFunctions_cancelRecipeActivation (sPlcName, sRcpBuffersDp, sRecipeState, "Cancelled by user", false);
	}
	
	//Debug("Recipe activation cancelled!\n");
}



// TODO: Deal with the errors (in exceptionInfo variable)
void _unRecipeFunctions_createBuffersData(string sPlcName, dyn_string dpes, dyn_string values, dyn_string &exceptionInfo)
{
  dyn_int dparManRegAddr, wparManRegAddr, aparManRegAddr, aalarmManRegAddr, pidManRegAddr;
  dyn_int dparManRegVal,  wparManRegVal,  aparManRegVal,  aalarmManRegVal,  pidManRegVal;
  dyn_int dparReqAddr, wparReqAddr, aparReqAddr, aalarmReqAddr, pidReqAddr;
  dyn_float dparReqVal,  wparReqVal,  aparReqVal,  aalarmReqVal,  pidReqVal;
  mapping controllerMap, aalarmMap;
  dyn_dyn_string list;
  int manRegAddr, manRegVal, manRegValAct, reqAddr, reqVal;
  float fval;
  int rc, ival, pos, numDPar=0, numWPar=0, numAPar=0, numAAlarm=0, numPid=0;  
  bool bval, bOk;
  time t0, t1;
  
  t0 = getCurrentTime();
  
  if (strpos(sPlcName, ":") < 0) {
	// Adds the system name to the PLC name
	sSystemName = unGenericDpFunctions_getSystemName(sPlcName);
	sPlcName = sSystemName + sPlcName;
  }
  
  //Debug("Inside _unRecipeFunctions_createBuffersData\n");
  //Debug("Values: " + values + "\n");
  int len = dynlen(dpes);
  for (int i=1; i<=len; i++){    
    // Get the DPE device type name
    string dpe = dpes[i];
    string deviceType = dpTypeName(dpe);
    string deviceName = unGenericDpFunctions_getDpName(dpe);
    string dpeName = substr(dpe, strlen(deviceName), strlen(dpe)-strlen(deviceName));
/*    
    Debug("DPE: " + dpe +"\n");
    Debug("DeviceType: " + deviceType +"\n");
    Debug("DeviceName: " + deviceName +"\n");    
    Debug("DpName: " + dpeName +"\n");
  */  
    // Get the DPE address
    string addressType, address, temp;
    _unRecipeFunctions_getDpeAddress(dpe, deviceType, address, exceptionInfo);
    if (dynlen(exceptionInfo)) return;
	
    switch (deviceType){
      case "CPC_DigitalParameter":    
        numDPar++;    
        // Get the ManReg01 address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
        // Set the new ManReg01 values 
        bval = (strtoupper(values[i])=="TRUE")? true : false;    
		//DebugTN ("DPAR VALUE: " + bval + "   values[i] : " + values[i]);
		// Set the ManReg values for the ArmRcp
        manRegVal=0;
		_unRecipeFunctions_setBit(manRegVal, DPAR_ARM_BIT, true);
		_unRecipeFunctions_setBit(manRegVal, DPAR_MONR_BIT, bval);
        _unRecipeFunctions_setBit(manRegVal, DPAR_MOFFR_BIT, !bval);  		
        // Insert the address and values in the lists
        dynAppend(dparManRegAddr, manRegAddr);       
        dynAppend(dparManRegVal, manRegVal);
        break;
      case "CPC_WordParameter":
        numWPar++;
        // Get the ManReg01 address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
        // Set the new ManReg01 values for the ArmRcp
		manRegVal=0;
		_unRecipeFunctions_setBit(manRegVal, WPAR_ARM_BIT, true);
		_unRecipeFunctions_setBit(manRegVal, WPAR_NEWMR_BIT, true);
        // Insert the address and values in the lists
        dynAppend(wparManRegAddr, manRegAddr);       
        dynAppend(wparManRegVal, manRegVal);
        // Get the MPosR address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.MPosR", deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(wparReqAddr, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        dynAppend(wparReqVal, fval);
        break; 
      case "CPC_AnalogParameter":
        numAPar++;
        // Get the ManReg01 address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
        // Set the new ManReg01 values for the ArmRcp
        manRegVal=0;
		_unRecipeFunctions_setBit(manRegVal, APAR_ARM_BIT, true);
		_unRecipeFunctions_setBit(manRegVal, APAR_NEWMR_BIT, true);
        // Insert the address and values in the lists
        dynAppend(aparManRegAddr, manRegAddr);       
        dynAppend(aparManRegVal, manRegVal);
        // Get the MPosR address
        _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.MPosR", deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(aparReqAddr, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        dynAppend(aparReqVal, fval);
        break;        
        
      case "CPC_AnalogAlarm":
		numAAlarm++; 
        dynClear(list);
        // Get the ManReg01 address and ManReg01 value
        if (!mappingHasKey(aalarmMap, deviceName)){                  
          _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		  if (dynlen(exceptionInfo)) return;
          list[1][1] = manRegAddr;
          list[1][2] = 0;
          aalarmMap[deviceName]=list;
          manRegVal=0;    
        } else {
          list=aalarmMap[deviceName];
          manRegAddr=list[1][1];
          manRegVal = list[1][2];
        }
        
		// Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
		 
        // Modifies the ManReg Value
		// Get the MNewMR bit position
        if (!mappingHasKey(unRecipe_aalarmManRegBits, dpeName)){
          fwException_raise(exceptionInfo, "ERROR", "The analog alarm dpe:" + dpeName + 
                            " hasn't a bit possition defined in the ManReg01.", "");
          return;  
        }	
        pos = unRecipe_aalarmManRegBits[dpeName];
		
		// Set the new ManReg01 values for the ArmRcp
		_unRecipeFunctions_setBit(manRegVal, AALARM_ARM_BIT, true); 
		_unRecipeFunctions_setBit(manRegVal, pos, true); 
        list[1][2]=manRegVal;   
        		
        // Set the Req. Address and Req. Value
        _unRecipeFunctions_getDpeAddress(dpe, deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(list, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        list[dynlen(list)][2] = fval; 
        
        aalarmMap[deviceName] = list;
        break;
        
      case "CPC_Controller":
	    numPid++; 
        dynClear(list);
        // Get the ManReg01 address and ManReg01 value
        if (!mappingHasKey(controllerMap, deviceName)){     
          _unRecipeFunctions_getDpeAddress(deviceName+".ProcessOutput.ManReg01", deviceType, manRegAddr, exceptionInfo);
		  if (dynlen(exceptionInfo)) return;		  
          list[1][1] = manRegAddr;
          list[1][2] = 0;
          controllerMap[deviceName]=list;
          manRegVal    = 0;          
        } else {
          list=controllerMap[deviceName];
          manRegAddr   = list[1][1];
          manRegVal    = list[1][2];
        }
        
		//Debug("ManRegAddr of : " + deviceName +".ProcessOutput.ManReg01 : " + manRegAddr +"\n");
		
        // Modifies the ManReg Value
        if (!mappingHasKey(unRecipe_unRecipe_pidManRegBits, dpeName)){
          fwException_raise(exceptionInfo, "ERROR", "The CPC_Controller dpe:" + dpeName + 
                            " hasn't a bit possition defined in the ManReg01.", "");
          return;  
        }
        pos = unRecipe_unRecipe_pidManRegBits[dpeName];
		_unRecipeFunctions_setBit(manRegVal, PID_ARM_BIT, true); 
		_unRecipeFunctions_setBit(manRegVal, pos, true); 
        list[1][2]=manRegVal;   
		
        // Set the Req. Address and Req. Value
        _unRecipeFunctions_getDpeAddress(dpe, deviceType, reqAddr, exceptionInfo);
		if (dynlen(exceptionInfo)) return;
        dynAppend(list, reqAddr);
		unGenericObject_StringToFloat(values[i], fval, bOk);
        list[dynlen(list)][2] = fval; 
        
		//DebugTN("Assigning Controller value - Address: " + reqAddr + "  Value: " + values[i] + "   listValue: " + list[dynlen(list)][2] + "\n");
		
        controllerMap[deviceName] = list;
        break;
    }
  }
  
  // Gets all the recipe buffers
  dyn_int recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(sPlcName);
  dyn_int manRegAddrBuffer   = _unRecipeFunctions_getManRegAddrBuffer(sPlcName);
  dyn_int manRegValBuffer    = _unRecipeFunctions_getManRegValBuffer(sPlcName);
  dyn_int reqAddrBuffer      = _unRecipeFunctions_getReqAddrBuffer(sPlcName);
  dyn_float reqValBuffer   	 = _unRecipeFunctions_getReqValBuffer(sPlcName);
  
  
  // Inserts the data in the recipe header buffer 
  // Split the current time in high part and low part
  dynClear(recipeHeaderBuffer);
  bit64 t = getCurrentTime();
  bit32 bLow  = 0x0000FFFFu & t;
  bit32 bHigh = 0x0000FFFFu & (t >> 16);
  // TODO: Fix this!
  recipeHeaderBuffer[RCP_HEADER_ID_LOW] 	= (int)bLow % 32000;
  recipeHeaderBuffer[RCP_HEADER_ID_HIGH] 	= (int)bHigh % 32000;
  recipeHeaderBuffer[RCP_HEADER_NUM_MANREG] = (numDPar + numWPar + numAPar + numAAlarm + numPid);
  recipeHeaderBuffer[RCP_HEADER_NUM_DPAR] 	= numDPar;
  recipeHeaderBuffer[RCP_HEADER_DB_DPAR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_WPAR] 	= numWPar;
  recipeHeaderBuffer[RCP_HEADER_DB_WPAR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_APAR] 	= numAPar;
  recipeHeaderBuffer[RCP_HEADER_DB_APAR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_AALARM] = numAAlarm;
  recipeHeaderBuffer[RCP_HEADER_DB_AALARM]  = 0;
  recipeHeaderBuffer[RCP_HEADER_NUM_CONTR] 	= numPid;
  recipeHeaderBuffer[RCP_HEADER_DB_CONTR] 	= 0;
  recipeHeaderBuffer[RCP_HEADER_SENT_VALUE] = RCP_ACT_HEADER_SENDING_DATA;
  recipeHeaderBuffer[RCP_HEADER_CRC] 		= 0;
  gHeaderMap[sPlcName] 	  = recipeHeaderBuffer;
  
  // Process the controller devices in the recipe to create the ManReg address, ManReg values, ReqAddress and ReqValues
  //DebugTN("Processing the controller values\n");
  _unRecipeFunctions_processMultiDpeDevices(controllerMap, pidManRegAddr, pidManRegVal, pidReqAddr, pidReqVal);
  //DebugTN("PID Req. values: ", pidReqVal);
  
  // Process the analog alarm devices in the recipe to create the ManReg address, ManReg values, ReqAddress and ReqValues
  //DebugTN("Processing the aalarm values\n");
  _unRecipeFunctions_processMultiDpeDevices(aalarmMap, aalarmManRegAddr, aalarmManRegVal, aalarmReqAddr, aalarmReqVal);

  // Inserts the data in the ManReg Address buffer
  dynClear(manRegAddrBuffer);
  dynAppend(manRegAddrBuffer, dparManRegAddr);
  dynAppend(manRegAddrBuffer, wparManRegAddr);
  dynAppend(manRegAddrBuffer, aparManRegAddr);
  dynAppend(manRegAddrBuffer, aalarmManRegAddr);
  dynAppend(manRegAddrBuffer, pidManRegAddr);
  gManRegAddrMap[sPlcName] = manRegAddrBuffer; 
/* 
  DebugTN("ManRegAddr DPAR: ", dparManRegAddr);
  DebugTN("ManRegAddr WPAR: ", wparManRegAddr);
  DebugTN("ManRegAddr APAR: ", aparManRegAddr);
  DebugTN("ManRegAddr AALARM: ", aalarmManRegAddr);
  DebugTN("ManRegAddr PID: ", pidManRegAddr);
  DebugTN("Setting Man Reg Addr for PLC: " + sPlcName, gManRegAddrMap[sPlcName]);
*/ 
  // Inserts the data in the ManReg Values buffer  (ArmRcp values)
  dynClear(manRegValBuffer);
  dynAppend(manRegValBuffer, dparManRegVal);
  dynAppend(manRegValBuffer, wparManRegVal);
  dynAppend(manRegValBuffer, aparManRegVal);
  dynAppend(manRegValBuffer, aalarmManRegVal);
  dynAppend(manRegValBuffer, pidManRegVal);
  gManRegValMap[sPlcName] = manRegValBuffer;
  // Inserts the data in the Requested Address buffer
  dynClear(reqAddrBuffer);
  dynAppend(reqAddrBuffer, dparReqAddr);
  dynAppend(reqAddrBuffer, wparReqAddr);
  dynAppend(reqAddrBuffer, aparReqAddr);
  dynAppend(reqAddrBuffer, aalarmReqAddr);
  dynAppend(reqAddrBuffer, pidReqAddr);
  gReqAddrMap[sPlcName] = reqAddrBuffer;
  // Inserts the data in the Requested Values buffer
  dynClear(reqValBuffer);  
  //dynAppend(reqValBuffer, dparReqVal);
  dynAppend(reqValBuffer, wparReqVal);
  dynAppend(reqValBuffer, aparReqVal);
  dynAppend(reqValBuffer, aalarmReqVal);
  dynAppend(reqValBuffer, pidReqVal);
 // DebugTN("Req Val Buffer: ", reqValBuffer);
  gReqValMap[sPlcName] = reqValBuffer;
  
/*  
  DebugTN("PLCName: " + sPlcName +"\n");
  DebugTN("ManReq Addr: " + manRegAddrBuffer +"\n");
  DebugTN("ManReq Vals (ArmRcp): " + manRegValBuffer +"\n");
  DebugTN("Req Addr: " + reqAddrBuffer +"\n");
  DebugTN("Req Vals: " + reqValBuffer +"\n");  
 */ 
  
  t1 = getCurrentTime();
  
 // string elapsedTime = formatTime("%S", t1-t0, " milli: %04d");
 // Debug ("Elapsed time creating buffers data: " + elapsedTime +"\n");
}



void _unRecipeFunctions_sendData(string plcName, dyn_string &exceptionInfo)
{  
  bool done = false;
  dyn_string dpeManRegAddr, dpeManRegVal, dpeReqAddr, dpeReqVal;
  dyn_int manRegAddr, manRegVal, reqAddr;
  dyn_float reqVal;
  string sTmp, sBufferDpName;
  
  _unRecipeFunctions_getRecipeBuffersName(plcName, sBufferDpName, exceptionInfo);
  manRegAddr = _unRecipeFunctions_getManRegAddrBuffer(plcName);
  manRegVal  = _unRecipeFunctions_getManRegValBuffer(plcName);
  reqAddr    = _unRecipeFunctions_getReqAddrBuffer(plcName);
  reqVal     = _unRecipeFunctions_getReqValBuffer(plcName);
  
 /* 
  Debug("0 - START - _unRecipeFunctions_sendData\n");
  Debug("PLCName: " + plcName +"\n");
  Debug("ManRegAddr: " + manRegAddr + "\n");
  Debug("ManRegVal : " + manRegVal + "\n");
  Debug("ReqAddr: " + reqAddr + "\n");
  Debug("ReqVal : " + reqVal + "\n");
*/  
  
  int count=0;
  int numManReg  = dynlen(manRegAddr);
  int numReqAddr = dynlen(reqAddr);  
  int numStructures = ((numManReg-1)/CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_WORD_STRUCTURE)+1;
  
//  Debug("1 - Creating " + numStructures + " WORD structure DPES _unRecipeFunctions_sendData\n");
  for (int i=1; i<=numStructures && !done; i++){
    for (int j=1; j<=CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_WORD_STRUCTURE; j++){
      string dpe = ".Structure"+i+".Value"+(count+1);
      dynAppend(dpeManRegAddr, sBufferDpName+"ProcessOutput.ManRegAddr"+dpe);
      dynAppend(dpeManRegVal, sBufferDpName+"ProcessOutput.ManRegVal"+dpe);
            
      if (count<numReqAddr)
        dynAppend(dpeReqAddr, sBufferDpName+"ProcessOutput.ReqAddr"+dpe);  
      count++;   
      if (count==numManReg){
        done=true;
        break;     
      }
    }
  }
  
  count=0; 
  done=false;
  
  if (numReqAddr>0) {
//    Debug("NumReqAddr: " + numReqAddr + "\n");
	numStructures = ((numReqAddr-1)/CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_FLOAT_STRUCTURE)+1;
	//Debug("2 - Creating " + numStructures + " FLOAT structure DPES _unRecipeFunctions_sendData\n");
	for (int i=1; i<=numStructures && !done; i++){
	  for (int j=1; j<=CPC_CONFIG_RECIPEBUFFERS_MAX_ELEMENTS_FLOAT_STRUCTURE; j++){
		string dpe = ".Structure"+i+".Value"+(count+1);
		dynAppend(dpeReqVal, sBufferDpName+"ProcessOutput.ReqVal"+dpe);
		count++;
		if (count==numReqAddr){
		  done=true;
		  break;
		}
	  }
	}
  }
  //Debug("3 - _unRecipeFunctions_sendData\n");

/*  
  Debug("DpeManRegAddr: " + dpeManRegAddr +"\n");
  Debug("DpeManRegVal: " + dpeManRegVal +"\n");
  Debug("DpeReqAddr: " + dpeReqAddr +"\n");
  Debug("DpeReqVal: " + dpeReqVal +"\n");
  
  Debug("ManRegAddr: " + manRegAddr +"\n");
  Debug("ManRegVal: " + manRegVal +"\n");
  Debug("ReqAddr: " + reqAddr +"\n");
  Debug("ReqVal: " + reqVal +"\n");
*/ 
  dpSetWait(dpeManRegAddr, manRegAddr);
  dpSetWait(dpeManRegVal, manRegVal);
  if (numReqAddr > 0) {
	dpSetWait(dpeReqAddr, reqAddr);
	dpSetWait(dpeReqVal, reqVal);
  }
  
//  Debug("4 - All recipe data sent to the PLC " + plcName + " _unRecipeFunctions_sendData\n");
  _unRecipeFunctions_writeInRecipeLog("All recipe data sent to the PLC " + plcName);
}


void _unRecipeFunctions_processMultiDpeDevices(mapping map, dyn_int &manRegAddresses, dyn_int &manRegValues, 
                                               dyn_int &reqAddresses, dyn_float &reqValues)
{
  dynClear(manRegAddresses);
  dynClear(manRegValues);
  dynClear(reqAddresses);
  dynClear(reqValues);  
  
  int len = mappinglen(map);
  dyn_string keys = mappingKeys(map);
  
  // Iterates through all the devices in the map
  for (int i=1; i<=len; i++){
    string deviceName = keys[i];
    dyn_dyn_string data = map[deviceName];
    int manRegAddr   = data[1][1];
    int manRegVal    = data[1][2];
    int datalen = dynlen(data);

	//DebugTN("Data: ", data);
    // Iterates through all the device DPES     
    for (int j=2; j<=datalen; j++){
      int reqAddr = data[j][1];
      float reqVal  = data[j][2];
	//  DebugTN("ManRegAddr: " + manRegAddr + "  ManRegVal: " + manRegVal + "\n");
    //  DebugTN("ReqAddr: " + reqAddr + "  ReqVal: " + reqVal + "\n");
      // Inserts the data in the buffers
      dynAppend(manRegAddresses, manRegAddr);
      dynAppend(manRegValues, manRegVal);
      dynAppend(reqAddresses, reqAddr);
      dynAppend(reqValues, reqVal);
    }
  }
}




void _unRecipeFunctions_getDpeAddress(string dpe, string deviceType, 
                                     string &address, dyn_string &exceptionInfo)
{
  string addressType, temp;
//  DebugTN("Getting address of: " + dpe + ":_address.._drv_ident\n");
 // Debug("Getting address of: " + dpe + ":_address.._drv_ident\n");
  dpGet(dpe + ":_address.._drv_ident", addressType);
  string sFunction = "DRV_" + addressType + "_convertToUnicosAddress";
  if(!isFunctionDefined(sFunction)) {
    //Debug("Function not defined: " + sFunction +"\n");
    fwException_raise(exceptionInfo, "ERROR", "The function " +sFunction+ " isn't defined.", "");  
    return;
  }	

  evalScript(address,	"string main(string sDeviceDpeName, string sDeviceType)"+
  										"{	string temp;"+
  										"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
  										"	return temp;"+
  										"}", makeDynString(), dpe, deviceType);     
  
  // If it's an S7 address, gets the offset
  if (addressType=="S7"){
    string fullAddress = address;
    string offset;
    
    int pos=strpos(fullAddress, ".");
    if(pos<0) {
      //Debug("Invalid address format: " + fullAddress +"\n");
      fwException_raise(exceptionInfo, "ERROR", "The S7 address format is invalid: " + fullAddress, "");
      return;
    }
  
    string temp = substr(fullAddress, pos+1, strlen(fullAddress)-pos);
    int len = strlen(temp);
    int j=0;
    for (int i=0; i<=len; i++){
      if ((temp[i]>='0' && temp[i]<='9') || temp[i]=='\0'){
        offset = offset + temp[i];
        j++;
      }
    }
    address = offset;  
  }
  
 // unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "The Address of : " + dpe + " is: " + address +"\n");
//  Debug("The Address of : " + dpe + " is: " + address +"\n"); 
}



void _unRecipeFunctions_setBit(anytype &manRegVal, int position, bool value)
{
  bit64 bitVariable;
  
  bitVariable = manRegVal;
  setBit(bitVariable, position, value);
  manRegVal = bitVariable;
}


void _unRecipeFunctions_getBit(anytype manRegVal, int position, int &value)
{
  bit64 bitVariable;
  
  bitVariable = manRegVal;
  value = getBit(bitVariable, position);
}



/** 
 * Gets the recipe buffers DPE name for an specific PLC.
 * @param plcName - [IN] PLC name.
 * @param dpName - [OUT] Name of the DPE where the recipe buffers of the PLC are located.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getRecipeBuffersName(string plcName, string &dpName, dyn_string &exceptionInfo)
{
  dpName = plcName;
  
  //DebugTN("PlcName: ", plcName);
  
  // Removes the system name of the plcName
  int pos = strpos(plcName, ":");
  if (pos>=0 && strlen(plcName)>pos) {
    plcName = substr(plcName, pos+1);
  }  
  
  string buffersName = dpName + "_" + CPC_CONFIG_RECIPEBUFFERS_DPT_NAME;
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(buffersName, dpName);  

  //DebugTN("BuffersName: " + buffersName + "   DPName: " + dpName);  
  
  if (!dpExists(dpName)) {
    //Debug("DP doesn't exist: " + dpName +"\n");
    fwException_raise(exceptionInfo, "ERROR", "The PLC recipe buffers don't exist: " + dpName, "");
  }
}


/** 
 * Inserts the DB number for each recipe device type in the recipe header buffer (Siemens only).
 * @param dpes - [IN] List of DPEs of a recipe.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_createRecipeHeaderDBNumber(string plcName, dyn_string dpes, dyn_string &exceptionInfo)
{
  int len;
  string dbNum;
  mapping dbMapping;  
  dyn_int recipeHeaderBuffer = _unRecipeFunctions_getHeaderBuffer(plcName);  
  
  
  len=dynlen(dpes);
  for (int i=1; i<=len; i++) {
    string deviceType = dpTypeName(dpes[i]);
    if (mappingHasKey(dbMapping, deviceType))
      continue;
    
    _unRecipeFunctions_getS7AddressDBNumber(dpes[i], deviceType, dbNum, exceptionInfo);
    if (dynlen(exceptionInfo))  return;
       
    dbMapping[deviceType] = dbNum;
    //Debug("DB of " + deviceType + ": " + dbMapping[deviceType] +"\n");
  }
  
  // Insert the data in the recipe header
  if (mappingHasKey(dbMapping, "CPC_DigitalParameter"))
    recipeHeaderBuffer[RCP_HEADER_DB_DPAR] = dbMapping["CPC_DigitalParameter"];
  else 
    recipeHeaderBuffer[RCP_HEADER_DB_DPAR] = 0;
  if (mappingHasKey(dbMapping, "CPC_WordParameter"))
    recipeHeaderBuffer[RCP_HEADER_DB_WPAR] = dbMapping["CPC_WordParameter"];
  else
    recipeHeaderBuffer[RCP_HEADER_DB_WPAR] = 0;
  if (mappingHasKey(dbMapping, "CPC_AnalogParameter"))
    recipeHeaderBuffer[RCP_HEADER_DB_APAR] = dbMapping["CPC_AnalogParameter"];  
  else 
    recipeHeaderBuffer[RCP_HEADER_DB_APAR] = 0;
  if (mappingHasKey(dbMapping, "CPC_AnalogAlarm"))
    recipeHeaderBuffer[RCP_HEADER_DB_AALARM] = dbMapping["CPC_AnalogAlarm"];
  else 
    recipeHeaderBuffer[RCP_HEADER_DB_AALARM] = 0;
  if (mappingHasKey(dbMapping, "CPC_Controller"))
    recipeHeaderBuffer[RCP_HEADER_DB_CONTR] = dbMapping["CPC_Controller"];  
  else
    recipeHeaderBuffer[RCP_HEADER_DB_CONTR] = 0;
  
  gHeaderMap[plcName] = recipeHeaderBuffer;
  //Debug("Header Buffer: " + recipeHeaderBuffer + "\n");
}



/** 
 * Gets the DB number of a S7 address for a DPE.
 * @param dpe - [IN] DPE which DB number is required.
 * @param deviceType - [IN] Device type name of the dpe.
 * @param db - [OUT] DB number of the DPE S7 address.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getS7AddressDBNumber(string dpe, string deviceType, string &db, dyn_string &exceptionInfo)
{
  string fullAddress = DRV_S7_convertToUnicosAddress(dpe, deviceType);

  int pos=strpos(fullAddress, ".");
  if(pos<0) {
    fwException_raise(exceptionInfo, "ERROR", "The S7 address format is invalid: " + fullAddress, "");
    return;
  }
  
  db = substr(fullAddress, 0, pos);
  strreplace(db, "DB", "");
 // Debug ("DB Number of: " + fullAddress + " is " + db + "\n");
}


/** 
 * Gets the offset of an S7 address where a DPE is located.
 * @param dpe - [IN] DPE which offset is required.
 * @param deviceType - [IN] Device type name of the dpe.
 * @param offset - [OUT] Offset of the DPE S7 address.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void _unRecipeFunctions_getS7AddressOffset(string dpe, string deviceType, string &offset, dyn_string &exceptionInfo)
{
  string fullAddress = DRV_S7_convertToUnicosAddress(dpe, deviceType);

  int pos=strpos(fullAddress, ".");
  if(pos<0) {
    fwException_raise(exceptionInfo, "ERROR", "The S7 address format is invalid: " + fullAddress, "");
    return;
  }
  
  string temp = substr(fullAddress, pos+1, strlen(fullAddress)-pos);
  int len = strlen(temp);
  int j=0;
  for (int i=0; i<=len; i++){
    if ((temp[i]>='0' && temp[i]<='9') || temp[i]=='\0'){
      offset = offset + temp[i];
      j++;
    }
  }
  
  //Debug ("Offset of: " + fullAddress + " is " + offset + "\n");
}


/**
 * Gets the recipe elements according to the distributed system.
 * @param elements - [IN] Recipe data point elements.
 * @param value - [IN] Recipe values.
 * @param systemDpes - [OUT] Map containing the recipe elements and values for each distributed system.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */  
void _unRecipeFunctions_getElementsBySystem(dyn_string elements, dyn_anytype values, 
                                            mapping &systemDpes, dyn_string &exceptionInfo)
{
  string pvssDpe;
  dyn_dyn_string data;
  mappingClear(systemDpes);
  
  int len = dynlen(elements);
  if (len!=dynlen(values)) {
    fwException_raise(exceptionInfo, "ERROR", "The number of DPEs and values of the recipe must be equal.", "");
    return;
  }  
  
  // Inserts the DPEs in the mapping according to the system name
  for (int i=1; i<=len; i++) {
    unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(elements[i], pvssDpe);
    string systemName = unGenericDpFunctions_getSystemName(pvssDpe);
    
    dynClear(data);
    if (mappingHasKey(systemDpes, systemName)) {
      data = systemDpes[systemName];
    }

    dynAppend(data[1], pvssDpe);
    dynAppend(data[2], values[i]);
    systemDpes[systemName] = data;
  }  
}


/** 
 * Get the DPEs that contains the online values (ProcessInput) for the specified DPEs (ProcessOutput)
 * @param dpNames - [IN] List of DPEs which online value DPEs are required.
 * @param outputDpes - [OUT] List of online value DPEs.
 */
void _unRecipeFunctions_getOnlineValueDpes (dyn_string dpNames, dyn_string &outputDpes) {
	int len = dynlen(dpNames);
	for (int i=1; i<=len; i++) {
		string dpType = dpTypeName(dpNames[i]);
		
		if (dpType=="CPC_DigitalParameter") {
			strreplace(dpNames[i], "ProcessOutput.ManReg01", "ProcessInput.PosSt");
		} else if (dpType=="CPC_AnalogAlarm") {
			strreplace(dpNames[i], "ProcessOutput", "ProcessInput");
			dpNames[i] += "St";
		} else if (dpType=="CPC_Controller") {
			strreplace(dpNames[i], "ProcessOutput", "ProcessInput");
			if (strpos(dpNames[i], "MSPH") >= 0) {
				strreplace(dpNames[i], "MSPH", "ActSPH");
			} else if (strpos(dpNames[i], "MSPL") >= 0) {
				strreplace(dpNames[i], "MSPL", "ActSPL");
			} else if (strpos(dpNames[i], "MSP") >= 0) {
				strreplace(dpNames[i], "MSP", "MSPSt");
			} else if (strpos(dpNames[i], "MOutH") >= 0) {
				strreplace(dpNames[i], "MOutH", "ActOutH");
			} else if (strpos(dpNames[i], "MOutL") >= 0) {
				strreplace(dpNames[i], "MOutL", "ActOutL");
			} else if (strpos(dpNames[i], "MKc") >= 0) {
				strreplace(dpNames[i], "MKc", "ActKc");
			} else if (strpos(dpNames[i], "MTi") >= 0) {
				strreplace(dpNames[i], "MTi", "ActTi");
			} else if (strpos(dpNames[i], "MTd") >= 0) {
				strreplace(dpNames[i], "MTd", "ActTd");
			} else if (strpos(dpNames[i], "MTds") >= 0) {
				strreplace(dpNames[i], "MTds", "ActTds");
			} 
		} else if (dpType=="CPC_WordParameter" || dpType=="CPC_AnalogParameter") {
			strreplace(dpNames[i], "ProcessOutput.MPosR", "ProcessInput.PosSt");
		} 
	}
	
	outputDpes = dpNames;
}


/** 
 * Check if the recipe values are the same than the online values.
 * @param dpes - [IN] Recipe elements data points.
 * @param values - [IN] Recipe values.
 * @param errDpe - [OUT] If the recipe values are different than the online values, this variable
 * 				   will contain the first dpe where the values are different.
 * @return TRUE if the recipe values are the same than the online values, otherwise FALSE.
 */
 bool _unRecipeFunctions_checkRecipeOnlineValues(dyn_string dpes, dyn_string values, string &errDpe)
{ 
  dyn_string outputDpes;
  dyn_string onlineValues, formattedValues;
  dyn_string exceptionInfo;
  string sFormat, sOnlineValue;
  float fFormattedValue;
  
  // Get the DPEs for the online values
  _unRecipeFunctions_getOnlineValueDpes (dpes, outputDpes); 
  dpGet(outputDpes, onlineValues);
  
 // DebugTN("Rcp Values: ", values);
 // DebugTN("Online Val: ", onlineValues);
  
  int len = dynlen(onlineValues);
  for (int i=1; i<=len; i++) {
      
	  float fval, frcpval, fDiff;
      int ival, ircpval;
      bool bval, brcpval;        
	  
	  string dpType = dpTypeName(outputDpes[i]);
	  if (dpType=="CPC_DigitalParameter") {
		bval = onlineValues[i];
		brcpval = values[i];
		//DebugTN("RcpVal: '" + brcpval + "'  OnlineVal: '" + bval +"'\n");
		if (bval!=brcpval) {
			errDpe = dpes[i];
			return false;
		}
		continue;	
	  }
		
      int elementType = dpElementType(outputDpes[i]); 
      switch(elementType){
        case DPEL_FLOAT:
			sFormat = dpGetFormat(outputDpes[i]);
			sOnlineValue = unGenericObject_FormatValue(sFormat, onlineValues[i]);
			float fValue = (float)sOnlineValue;
			if (values[i] != sOnlineValue) {
				errDpe = dpes[i];
				return false;
			}
          break;
        case DPEL_INT:
          ircpval = (int)values[i];
          if (onlineValues[i]!=ircpval) {
			errDpe = dpes[i];
            return false;
          }             
          break;        
        case DPEL_BOOL:
          brcpval = (bool)values[i];
          if (onlineValues[i]!=brcpval) {
			errDpe = dpes[i];
            return false;
          }  
          break;        
        default:
          value = (string)values[j];           
      }
  }
  
  return true;
} 



void unRecipeFunctions_getRecipeStatus(dyn_dyn_mixed recipeObject, string lastActivated, int &status, 
                                       dyn_string &errMsg, dyn_string &exceptionInfo)
{
  bool active;
  dyn_string statusMessage;
  string errDpe;
  string rcpName 		 = recipeObject[fwConfigurationDB_RO_META_ORIGNAME];
  dyn_string rcpElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  dyn_string rcpValues   = recipeObject[fwConfigurationDB_RO_VALUE];
  dyn_string rcpTypes	 = recipeObject[fwConfigurationDB_RO_ELEMENT_TYPE];
  
  // Check the PLCs status
  //Debug("unRecipeFunctions_getRecipeStatus - Checking PLC status \n");
  unRecipeFunctions_getPlcStatus(rcpElements, active, errMsg, exceptionInfo);
  if(active==false){
    status = RCP_NO_PLC_CONNECTION;
    return;
  }
  
  // Check if all the DPEs used in the recipe exist
  //unRecipeFunctions_checkRecipeDpes(exceptionInfo);
  //Debug("unRecipeFunctions_getRecipeStatus - Checking recipe structure \n");
  unRecipeFunctions_checkRecipeStructure(rcpName, rcpElements, rcpValues, rcpTypes, errMsg, exceptionInfo);
  if (dynlen(exceptionInfo)){
   // Debug("ExceptionInfo: " + exceptionInfo);
    if (dynlen(errMsg))
      status = RCP_BAD_STRUCTURE;
    else
      status = RCP_BAD_TYPES;
    return;
  }  
  
  // Check if the last activated recipe of this type is the current one
  //string lastActivated;
 // Debug("unRecipeFunctions_getRecipeStatus - Getting last activated recipe \n");

  if (lastActivated!=rcpName){
    status = RCP_INACTIVE;
    return;
  }      
  
  //Debug("unRecipeFunctions_getRecipeStatus - Getting online values \n");
  if(false == _unRecipeFunctions_checkRecipeOnlineValues(rcpElements, rcpValues, errDpe)){
    status = RCP_INACTIVE;  
    return;  
  }  

  status = RCP_ACTIVATED;
}



void unRecipeFunctions_checkRecipeStructure(string rcpName, dyn_string rcpElements, dyn_string rcpValues, dyn_string rcpTypes,
                                            dyn_string &errMsg, dyn_string &exceptionInfo)
{
  float fval;
  int ival, elementType;
  bool bval;
  dyn_string exception;
  string deviceType = "", lowerValue;
  
  dynClear(errMsg);
  dynClear(exceptionInfo);
  
  // Check if the number of elements is equal to the number of values
  if (dynlen(rcpElements)!=dynlen(rcpValues)){
    fwException_raise(exceptionInfo, "ERROR", "unRecipeFunctions_checkRecipeStructure: " +
                      "The number of elements is different than the number of values: " + rcpName, "");
    return;
  }
  
  // Check if the recipe value types match the data point element types
  int numElements = dynlen(rcpElements);
  for (int i=1; i<=numElements; i++){    

	deviceType = dpTypeName(rcpElements[i]);
	//Debug("DeviceType: " + deviceType +"\n");
	// TODO: Avoid the type hardcoding
	if (deviceType=="CPC_DigitalParameter") 
		elementType = DPEL_BOOL;
	else 
		elementType = rcpTypes[i];
    dynClear(exception);
	//Debug("Checking rcp element: " + rcpElements[i] + "  type: " + rcpTypes[i] + "\n");
	
    switch(elementType){
      case DPEL_FLOAT:      
		//Debug("Checking float data\n");	  
        unConfigGenericFunctions_checkFloat(rcpValues[i], fval, exception); 
        if(dynlen(exception)){ 
          dynAppend(exceptionInfo, exception);  
        }
        break;
      case DPEL_INT:
		//Debug("Checking int data\n");
        unConfigGenericFunctions_checkInt(rcpValues[i], ival, exception); 
        if(dynlen(exception)) {
          dynAppend(exceptionInfo, exception);      
        }
        break;
      case DPEL_BOOL:
	   // Debug("Checking boolean data\n");
		lowerValue = strtolower(rcpValues[i]);
		if (lowerValue!="true" && lowerValue!="false")
			fwException_raise(exceptionInfo, "ERROR", "unRecipeFunctions_checkRecipeStructure: " +
			          "Wrong boolean value: " + rcpValues[i], "");
                //          "Wrong boolean value: " + dpTypeName(rcpValue[i]), "");
						  
						  
   /*     unConfigGenericFunctions_checkInt(rcpValues[i], bval, exception);   
        if(dynlen(exception)) {
          dynAppend(exceptionInfo, exception);  
        }*/
        break;
      default:
        fwException_raise(exceptionInfo, "ERROR", "unRecipeFunctions_checkRecipeStructure: " +
                          "Unknown DPType: " + dpTypeName(rcpElements[i]), "");
        return;
    }
  }
}



/** 
 * Check if the recipe values are valid according to the defined ranges.
 * @param recipeObject - [IN] Recipe object.
 * @param values - [IN/OUT] Recipe values.
 * @param errorLines - [OUT] Positions where the values are wrong according to the defined ranges.
 * @param exceptionInfo - [OUT] Standard exception handling routine.
 */
void unRecipeFunctions_checkRecipeValues (dyn_dyn_mixed recipeObject, dyn_anytype &values, 
                                          dyn_int &errorLines, dyn_string &exceptionInfo)
{
  string min, max, range, sVal;
  dyn_string exception;
  dynClear(errorLines);
  dynClear(exceptionInfo);
  int retVal;
  int len = dynlen(recipeObject[fwConfigurationDB_RO_DPE_NAME]);
  
  for (int i=1; i<=len; i++){
	// Check if the recipe value is valid
    retVal = unRecipeFunctions_checkRecipeValue (recipeObject[fwConfigurationDB_RO_DPE_NAME][i], values[i], exception);

	if (dynlen(exception)) {
      dynAppend(exceptionInfo, exception);
      dynClear(exception);
    }	  
	  
	if (retVal == -1) {
      dynAppend(errorLines, i);
    } 
  }
}


/** 
 * Check if the recipe value is valid according to the defined range.
 * @param sDpName - [IN] Data point name of the recipe element.
 * @param values - [IN] Value of the recipe element.
 * @param exceptionInfo - [OUT] Standard exception handling routine.
 * @return 0 if the value is in the defined range or if no range is defined, otherwise -1.
 */
int unRecipeFunctions_checkRecipeValue (string sDpName, anytype value, dyn_string &exceptionInfo)
{
	string sVal, range, min, max;
	
	range = unRecipeFunctions_getDeviceRange(sDpName, exceptionInfo);	
	if (dynlen(exceptionInfo)) {
		return -1;
	}
	
	// No range defined => Value is valid
	if (range=="")
		return 0;		
		
	// Check if the range is FALSE, TRUE
	if (range=="[FALSE, TRUE]") {
		if (value=="0" || value=="1" || strtoupper(value)=="TRUE" || strtoupper(value)=="FALSE")
			return 0;
		return -1;
    }	
		
    _unRecipeFunctions_getRangeValues(range, min, max, exceptionInfo);
	
    if (dynlen(exceptionInfo)) {
      return -1;
    } 
	
    float fmin, fmax, fval;
    fmin = (float)min;
    fmax = (float)max;
	
	// Removes the white space in the exponential value (1.5 e1 => 1.5e1) before converting to float
	sVal = value;
	strreplace(sVal, " ", "");	
    fval = (float)sVal;

    if (fmin>fval || fval>fmax){
      return -1;
    } 
	
	return 0;
}


/** 
 * Get the min. and max. values defined in a range string
 * @param range - [IN] String containing a range definition (i.e.: [0.0, 100.0])
 * @param min - [OUT] Minimum value defined in the range.
 * @param max - [OUT] Maximum value defined in the range.
 * @param exceptionInfo - [OUT] Standard exception handling routine.
 */
void _unRecipeFunctions_getRangeValues(string range, string &min, string &max, dyn_string &exceptionInfo)
{  
  string rangeElement, deviceType, rangeMin, rangeMax;

  min = "";
  max = "";
  
  string tmpRange = range;
  strreplace(tmpRange, "[", "");
  strreplace(tmpRange, "]", "");
  strreplace(tmpRange, " ", "");
  

  dyn_string rangeSplit = strsplit(tmpRange, ",");
  if (dynlen(rangeSplit)!=2)  {
    fwException_raise(exceptionInfo, "ERROR", "The specified range format is incorrect: " + range, "");
    return;
  }

  min = rangeSplit[1];
  max = rangeSplit[2];
  
  strreplace(min, " ", "");
  strreplace(max, " ", "");
  
  min = strrtrim(min);
  min = strltrim(min);
  max = strrtrim(max);
  max = strltrim(max);
}


/**
 * Gets the PLC status of a recipe.
 * @param recipeElements - [IN] List of DPEs.
 * @param bActive - [OUT] True if the PLCs are active, otherwise false.
 * @param statusmessage - [OUT] PLC status message.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_getPlcStatus(dyn_string recipeElements, bool &bActive, 
                                    dyn_string &statusMessage, dyn_string &exceptionInfo)
{
  int iRes;
  bActive = true;
	
  // Get the list of PLCs used in the recipe
  dyn_string plcList;
  for (int i=1; i<=dynlen(recipeElements); i++){
     string plc = unGenericObject_GetPLCNameFromDpName(recipeElements[i]);
     if (!dynContains(plcList, plc)){
       dynAppend(plcList, plc);
     }
  }
	
  //check PLC status
  for (int i=1; i<=dynlen(plcList); i++){
    //sPlcName - corresponding dp name in "_Mod_Plc" internal DPT
    int iSystemIntegrityAlarmValue = 0;
    bool bSystemIntegrityAlarmEnabled = false;
    string sPlcName = plcList[i];	
    
    iRes = dpGet(c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm", iSystemIntegrityAlarmValue);
    //Debug("Checking system integrity: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm\n");
    if (iRes < 0)	{
      dynAppend(exceptionInfo, "unRecipeFunctions_getPlcStatus: failed to get current state of plc - " + sPlcName +"\n");
  		 return;
    }
	
    iRes = dpGet(c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled", bSystemIntegrityAlarmEnabled);
    //Debug("Checking system integrity 2: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled\n");
    if (iRes < 0)	{
     	dynAppend(exceptionInfo, "unRecipeFunctions_getPlcStatus: failed to get current state of plc - " + sPlcName+"\n");
  		 return;
    }	
    
    if((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)) {
      bActive = false;
      dynAppend(statusMessage, sPlcName);
    }
  }
}


void _unRecipeFunctions_plcStatusCB (string sAlarmDpe, int iSystemIntegrityAlarmValue,
									 string sEnabledDpe, bool bSystemIntegrityAlarmEnabled)
{	
	if (!shapeExists("LoadedRecipeDp")) {
		return;
	}

	_unRecipeFunctions_loadPropertiesTableInfo();
}


// TODO: How to do this with several PLCs?
void unRecipeFunctions_disconnectPlcStatus(string sRcpInstanceDp, dyn_string &exceptionInfo)
{
  dyn_string dsRecipeElements, dsPlcList;
  dyn_dyn_mixed recipeObject;
  string sPlcName;
  int rc;

  _unRecipeFunctions_getRecipeObjectFromDp(sRcpInstanceDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }  
  dsRecipeElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  
  // Get the list of PLCs used in the recipe
  for (int i=1; i<=dynlen(dsRecipeElements); i++){
     sPlcName = unGenericObject_GetPLCNameFromDpName(dsRecipeElements[i]);
     if (!dynContains(dsPlcList, sPlcName)){
       dynAppend(dsPlcList, sPlcName);
     }
  }
  
  // connect to PLC status
  for (int i=1; i<=dynlen(dsPlcList); i++){
    sPlcName = dsPlcList[i];	
	//DebugTN("DpDisconnect: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm" );
    rc = dpDisconnect("_unRecipeFunctions_plcStatusCB", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled" );
  }	
}


// TODO: How to do this with several PLCs?
void unRecipeFunctions_connectPlcStatus(string sRcpInstanceDp, dyn_string &exceptionInfo)
{
  dyn_string dsRecipeElements, dsPlcList;
  dyn_dyn_mixed recipeObject;
  string sPlcName;

  _unRecipeFunctions_getRecipeObjectFromDp(sRcpInstanceDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
  
  dsRecipeElements = recipeObject[fwConfigurationDB_RO_DPE_NAME];
  
  // Get the list of PLCs used in the recipe
  for (int i=1; i<=dynlen(dsRecipeElements); i++){
     sPlcName = unGenericObject_GetPLCNameFromDpName(dsRecipeElements[i]);
     if (!dynContains(dsPlcList, sPlcName)){
       dynAppend(dsPlcList, sPlcName);
     }
  }
  
  // connect to PLC status
  for (int i=1; i<=dynlen(dsPlcList); i++){
    //DebugTN("DpConnect: " + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm" );
    sPlcName = dsPlcList[i];	
	dpConnect("_unRecipeFunctions_plcStatusCB", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm", 
			  c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
  }
}




bool unRecipeFunctions_selectRecipeInstance(string sRcpInstanceDp, dyn_string &exceptionInfo)
{
  int iPos, iSelected;
  dyn_int selectedLines;
  string sFaceplatePanel, sButtonPanel, sClassName, sInstanceName, sLoadedRcpDp, sState;
  dyn_string rcpClassesLoaded, rcpInstancesDp;  
  
  //DebugTN("SelectRecipeInstance - Start\n");
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances") || 
      !shapeExists("RecipeClassLabel") || !shapeExists("RecipeInstanceLabel")) 
  {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return false;
  }

  // Get the recipe classes loaded in the combo box
  getValue("RecipeClasses", "items", rcpClassesLoaded);
  
  // Get the recipe class name of the recipe instance to select
  dpGet(sRcpInstanceDp+".ProcessInput.ClassName", sClassName,
		sRcpInstanceDp+".ProcessInput.InstanceName", sInstanceName,
		sRcpInstanceDp+".ProcessInput.State", sState);
  
  iPos = dynContains(rcpClassesLoaded, sClassName);  
  if (iPos<=0) {
    // The recipe class of the recipe instance doesn't exist!
    iPos = dynContains(rcpClassesLoaded, "-None-");
    if (iPos<=0) {
      setValue("RecipeClasses", "appendItem", "-None-");
      setValue("RecipeClasses", "selectedPos", dynlen(rcpClassesLoaded)+1);
    } else {
      setValue("RecipeClasses", "selectedPos", iPos);
    }
    return false;
  } 

  // Selects the recipe class in the combo box
  getValue("RecipeClasses", "selectedPos", iSelected);
  if ((iPos)!=iSelected) {   
	//Debug("Selecting recipe class number: " + iPos + "\n");
    setValue("RecipeClasses", "selectedPos", iPos);
    _unRecipeFunctions_loadRecipeInstances(sClassName);
  } 
  
  
  // Selects the recipe instance in the table
  getValue("RecipeInstances", "getSelectedLines", selectedLines);
  getValue("RecipeInstances", "getColumnN", 1, rcpInstancesDp);
  iPos = dynContains(rcpInstancesDp, sRcpInstanceDp);
  if (iPos<=0) { 
	// The required recipe doesn't exist, an error occurred in the dynContains method
	// or there aren't selected lines in the table
	return false;
  }
  

  getValue("LoadedRecipeDp", "text", sLoadedRcpDp);
  if (dynlen(selectedLines)<=0 || (iPos-1)!=selectedLines[1] || sLoadedRcpDp=="" || sLoadedRcpDp!=sRcpInstanceDp) {
	  //DebugTN("Selecting recipe instance: " + (iPos-1) + "\n");
      setValue("RecipeInstances", "selectLineN", (iPos-1));
   
	  // add  device action
	  if (shapeExists("buttonActivate")) {
		removeSymbol(myModuleName(), myPanelName(), "RCPButton");
	  }
			
	  // get the button panel of the device from UNICOS configuration
	  unGenericDpFunctions_getContextPanel(sRcpInstanceDp, sButtonPanel, exceptionInfo);
	  if(dynlen(exceptionInfo)) {DebugTN(exceptionInfo); return;}
	  addSymbol(myModuleName(), myPanelName(), sButtonPanel, "RCPButton", makeDynString("$sDpName:"+sRcpInstanceDp), 190, 610, 0, 1, 1);
		
	  unRecipeFunctions_loadRecipeInstanceData(); 
	  setValue("RecipeClassLabel", "text", sClassName);
	  setValue("RecipeInstanceLabel", "text", sInstanceName);
	  setValue("LoadedRecipeDp", "text", sRcpInstanceDp);	
  } 	  
	  
  // Forces the reload of the recipe state to animate the Select button
  dpSet(sRcpInstanceDp+".ProcessInput.State", sState);

  // Animate the Selection Progress Bar
  startThread("unRecipeFunctions_animateSelectionProgressBar", sRcpInstanceDp);
  
  return true;
}


/**
 * Method used to animate the selection progress bar in the recipe instance panel.
 * @param sDpName - [IN] Data point name of the selected recipe instance.
 */
void unRecipeFunctions_animateSelectionProgressBar (string sDpName) 
{
  int iTimeOut;
  dyn_string exceptionInfo;
  string sRemainingTime, sSelectedManager, sSelectedRecipeDp;
  bool bSelect;

  // Get the selected recipe instance in the panel
  unRecipeFunctions_getSelectedRecipeDp(sSelectedRecipeDp, exceptionInfo); 
  if (sDpName != sSelectedRecipeDp) {
	unProgressBar_setPosition("SelectionProgressBar", 0);  
	return;
  }
	
  /** Selection Progress Bar animation **/
  if (shapeExists("SelectionProgressBar") && shapeExists("buttonSelect")) {
    delay (0, 50);
    dpGet(sDpName + ".statusInformation.selectedManager", sSelectedManager);
    bSelect = (sSelectedManager != "");
    if (bSelect) {
      dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime,
		         getSystemName() + "_unSelectDeselect.timeOut", iTimeOut);      
      
      unProgressBar_setMax("SelectionProgressBar", iTimeOut);
      unProgressBar_setPosition("SelectionProgressBar", iTimeOut);

	  while (strsplit(sRemainingTime, ";")[3] != "0" && bSelect) {
		unRecipeFunctions_getSelectedRecipeDp(sSelectedRecipeDp, exceptionInfo);
		if (sDpName != sSelectedRecipeDp) {
			return;
		}
        dpGet(getSystemName() + "_unSelectDeselect.remainingTime", sRemainingTime);
        unProgressBar_setPosition("SelectionProgressBar", strsplit(sRemainingTime, ";")[3]);
		delay(5);
		// If the DP was removed, return
		if (!dpExists(sDpName + ".statusInformation.selectedManager"))
			return;
		dpGet(sDpName + ".statusInformation.selectedManager", sSelectedManager);
		bSelect = (sSelectedManager != "");
	  }
    }
    
	unRecipeFunctions_getSelectedRecipeDp(sSelectedRecipeDp, exceptionInfo);
	if (sDpName != sSelectedRecipeDp) {
		return;
	}
	unProgressBar_setPosition("SelectionProgressBar", 0);     
  } 
}


//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to populate the recipes panel
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void unRecipeFunctions_populateRecipePanel () 
{
	_unRecipeFunctions_loadRecipeClassList();
	_unRecipeFunctions_loadRecipeInstances();	
} 
 
 
void _unRecipeFunctions_loadRecipeClassList()
{
	dyn_dyn_anytype resultTable;
	dyn_string recipeClassNames, recipeClassComments, exceptionInfo;
	dynClear(recipeClassNames);
	dynClear(recipeClassComments);

	// Check that all the necessary shapes exist
	if (!shapeExists("RecipeClasses")) {
		fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
		return;
	}
	
	// Get all the recipe classes
	dpQuery("SELECT '.ProcessInput.ClassName:_online.._value' FROM '*' WHERE _DPT=\""+UN_CONFIG_UNRCPCLASS_DPT_NAME+"\"", resultTable); 
  
    if (dynlen(resultTable)>1) {
      dynRemove(resultTable, 1);
      for (int i=1; i<=dynlen(resultTable); i++) {
        setValue("RecipeClasses", "appendItem", resultTable[i][2]);
      }
    }
} 
 

synchronized void _unRecipeFunctions_loadRecipeInstances(string sRecipeClass="")
{
  dyn_string dsRcpNames, dsRcpDps, exceptionInfo;
  dyn_dyn_anytype resultTable;
    	
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeInstances") || !shapeExists("RecipeClasses")) 
  {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }  	
	
  if (sRecipeClass=="") {
	getValue("RecipeClasses", "text", sRecipeClass); 
	if (sRecipeClass=="")  return;
  }
  
  setValue("RecipeInstances", "deleteAllLines");
    
  // Get all the instances of the recipe class
  dpQuery("SELECT '.ProcessInput.InstanceName:_online.._value' FROM '*' WHERE '.ProcessInput.ClassName:_online.._value' = \""+sRecipeClass+"\" AND _DPT=\""+UN_CONFIG_UNRCPINSTANCE_DPT_NAME+"\"", resultTable); 
  
  if (dynlen(resultTable)>1) {
    dynRemove(resultTable, 1);
    for (int i=1; i<=dynlen(resultTable); i++) {
		dynAppend(dsRcpNames, resultTable[i][2]);
		dynAppend(dsRcpDps, resultTable[i][1]);
    }
  }
  
  setValue("RecipeInstances", "appendLines", dynlen(dsRcpNames),
           "RecipeInstances", dsRcpNames, 
           "RecipeInstancesDP", dsRcpDps);
} 
 
 
 
synchronized void _unRecipeFunctions_loadPropertiesTableInfo(string sDpName="", dyn_dyn_mixed recipeObject=makeDynMixed())
{
  int status;
  bool bActive;
  dyn_int lines;
  dyn_string message, exceptionInfo;
  dyn_mixed recipeClassObject;

  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeProperties")) 
  {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }     
  
  // Get the selected recipe DP name
  if (sDpName=="") {
	unRecipeFunctions_getSelectedRecipeDp(sDpName, exceptionInfo);
	if (dynlen(exceptionInfo) || sDpName=="") { DebugTN(exceptionInfo); return; }
  }
  
  // Loads the recipe object data
  if (dynlen(recipeObject)==0) {
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo); 
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
  }

  // Loads the recipe class info
  fwConfigurationDB_getRecipeClassInfo(recipeObject[fwConfigurationDB_RO_META_CLASSNAME], recipeClassObject, exceptionInfo);
  if (dynlen(exceptionInfo))  { DebugTN(exceptionInfo); return; }
  
  // Fill the data in the recipe properties table
  setValue("RecipeProperties", "deleteAllLines");
  dyn_string names = makeDynString("Class:", "Last Activated:", "Status:", "Class Desc.:", "Rcp. Desc.:");
  dyn_string values= makeDynString(recipeClassObject[fwConfigurationDB_RCL_CLASSNAME], 
                                   recipeClassObject[fwConfigurationDB_RCL_LAST_ACTIVATED_RECIPE], 
                                   "", 
                                   recipeClassObject[fwConfigurationDB_RCL_DESCRIPTION], 
								   recipeObject[fwConfigurationDB_RO_META_COMMENT]);
  setValue("RecipeProperties", "appendLines", 5,
           "Name", names,
           "Value", values);
  
  // Get the recipe instance status
  unRecipeFunctions_getRecipeStatus(recipeObject, 
                                    recipeClassObject[fwConfigurationDB_RCL_LAST_ACTIVATED_RECIPE],
                                    status, 
                                    message, 
                                    exceptionInfo);
									
  unRecipeFunctions_setRecipeStatusInfo(status, message);
} 
 
 
synchronized void unRecipeFunctions_setRecipeStatusInfo(int status, dyn_string message=makeDynString())
{
  int len;
  string sCurrentStatus;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("StatusColor") || !shapeExists("buttonActivate") || !shapeExists("RecipeProperties")) 
  {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }     

  StatusColor.visible(true);

  switch(status){
    case RCP_NO_PLC_CONNECTION:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [No connection to PLCs: "+message+"]");
      break;
    case RCP_BAD_STRUCTURE:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Wrong recipe structure]");
      break;     
    case RCP_BAD_TYPES:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Wrong value types]");
	  _unRecipeFunctions_setInvalidRows(message);
      break;       
    case RCP_INACTIVE:
      StatusColor.backCol("{251,255,72}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Inactive]");
      break;
    case RCP_ACTIVATED:
      StatusColor.backCol("{0,245,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Active]");
      break;
    case RCP_ACTIVATION_FAILED:
      StatusColor.backCol("{255,0,0}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Activation failed]");
      break;	  
	case RCP_ACTIVATION_TRIGGERED:
      StatusColor.backCol("{251,255,72}");
      setValue("RecipeProperties", "cellValueRC", 2, "Value", "     [Activating]");  	
	  break;
  }
}
 
 
synchronized void unRecipeFunctions_loadRecipeInstanceData()
{
  string sRecipeInstance, sRecipeClass, sDpName;
  dyn_int diIndex;
  dyn_string exceptionInfo, dsDeviceDps, dsDeviceAlias, dsDescriptions, dsUnits, dsRanges;  
  dyn_dyn_mixed recipeObject;
  dyn_anytype formattedValues;
  mapping typeMapping;	 
				 
  // Get the selected recipe DP name
  unRecipeFunctions_getSelectedRecipeDp(sDpName, exceptionInfo);
  if (dynlen(exceptionInfo) || sDpName=="") { 
    _unRecipeFunctions_writeInRecipeLog("Error loading the recipe. Check the log window to get the error details.");
	DebugTN(exceptionInfo); 
	return; 
  }
  
  _unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
	_unRecipeFunctions_writeInRecipeLog("Error loading the recipe. Check the log window to get the error details.");
	DebugTN(exceptionInfo); 
	return; 
  }
	  

  string predefined = recipeObject[fwConfigurationDB_RO_META_PREDEFINED];

  // Load the RecipeProperties table
  _unRecipeFunctions_loadPropertiesTableInfo(sDpName, recipeObject);
  
  // Load the History table
  _unRecipeFunctions_loadHistoryTableInfo(sDpName, recipeObject);

  // Inserts the recipe elements in the table
  int len = dynlen(recipeObject[fwConfigurationDB_RO_DPE_NAME]);
  setValue("RecipeElements", "deleteAllLines");
  
  for (int i=1; i<=len; i++) {  
	dyn_string tmp;
    string dpName, alias, deviceType, description, dpe, value, unit="", range="";    
   
    dpName = recipeObject[fwConfigurationDB_RO_DPE_NAME][i];
	alias = unGenericDpFunctions_getAlias(unGenericDpFunctions_getDpName(dpName));
    deviceType = dpTypeName(dpName);
	tmp = strsplit(dpName, ".");
	dpe = tmp[dynlen(tmp)];
    description = unGenericDpFunctions_getDescription(unGenericDpFunctions_getDpName(dpName)) + " : " + dpe;

    range = unRecipeFunctions_getDeviceRange(recipeObject[fwConfigurationDB_RO_DPE_NAME][i], exceptionInfo);
    unit = unRecipeFunctions_getDpeUnit(dpName, exceptionInfo);
    
	dynAppend(diIndex, i);
	dynAppend(dsDeviceDps, dpName);
	dynAppend(dsDeviceAlias, alias);
	dynAppend(dsDescriptions, description);
	dynAppend(dsUnits, unit);
	dynAppend(dsRanges, range);
  }
  
  if (len>0) {
	  setValue("RecipeElements", "appendLines", len,
				 "Index", diIndex,
				 "DeviceDp", dsDeviceDps,
				 "Alias", dsDeviceAlias,
				 "Description", dsDescriptions,
				 "Value",  recipeObject[fwConfigurationDB_RO_VALUE],
				 "Unit", dsUnits,
				 "Range", dsRanges);
  }
  
  _unRecipeFunctions_resetRecipeElementsBgColor();
}
 
 
synchronized void _unRecipeFunctions_loadHistoryTableInfo(string sDpName="", dyn_dyn_mixed recipeObject=makeDynMixed())
{
  string recipeClass, recipeInstance;
  dyn_int lines;
  dyn_string exceptionInfo;

  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeInstances") || !shapeExists("History") || !shapeExists("RecipeClasses")) {
	  fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	  return;
  }     

  // Get the selected recipe DP name
  if (sDpName=="") {
	unRecipeFunctions_getSelectedRecipeDp(sDpName, exceptionInfo);
	if (dynlen(exceptionInfo) || sDpName=="") { DebugTN(exceptionInfo); return; }
  }
  
  // Loads the recipe object data
  if (dynlen(recipeObject)==0) {
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo); 
	if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return; }
  }
  
  
  setValue("History", "deleteAllLines");
  dyn_string col1 = makeDynString("Creator:", "Last modifier:", "Last activator:");
  dyn_string col2 = makeDynString(recipeObject[fwConfigurationDB_RO_META_AUTHOR], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONUSER], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONUSER]);
  dyn_string col3 = makeDynString("Creation time:", "Last modification time:", "Last activation time:");
  dyn_string col4 = makeDynString(recipeObject[fwConfigurationDB_RO_META_CREATIONTIME], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTMODIFICATIONTIME], 
                                  recipeObject[fwConfigurationDB_RO_META_LASTACTIVATIONTIME]);  
  setValue("History", "appendLines", 3,
           "#1", col1,
           "#2", col2,
           "#3", col3,
           "#4", col4);
}  


/** 
 * Remove the data from the tables.
 */
synchronized void _unRecipeFunctions_clearTables() {
  setValue("RecipeProperties", "deleteAllLines");
  setValue("History", "deleteAllLines");
  setValue("RecipeElements", "deleteAllLines"); 
  setValue("LoadedRecipeDp", "text", "");
  setValue("StatusColor", "visible", false);
}

//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to edit a recipe instance
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
void _unRecipeFunctions_editRecipeInstance(string sRecipeDp, int rowNumber, string colName, dyn_string &exceptionInfo)
{  
  //Debug("unRecipeFunctions_editRecipeInstance - Start\n");
  string sSelectedDpName, sRecipeState;
  
  // Recipe instance in EDIT mode
  dpSet(sRecipeDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_EDIT);

  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeElements") || !shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
    dpSet(sRecipeDp+".ProcessInput.State", "");
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	//Debug("Shape doesn't exist!\n");
	return;
  }
  
  unRecipeFunctions_getSelectedRecipeDp(sSelectedDpName, exceptionInfo);
  if (sSelectedDpName!=sRecipeDp) {
	  // If the instance selection failes, returns
	  if (!unRecipeFunctions_selectRecipeInstance(sRecipeDp, exceptionInfo)) {
		dpSet(sRecipeDp+".ProcessInput.State", "");
		return;
	  }
  }
	
  _unRecipeFunctions_highlightEditedValue(sSelectedDpName, rowNumber, colName); 
  
//  dpGet(sRecipeDp+".ProcessInput.State", sRecipeState);
//  if (sRecipeState=="") {	
	  // Recipe instance in EDIT mode
	  //dpSet(sRecipeDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_EDIT);
	  
	  // Modifies the color of the recipe values
	  _unRecipeFunctions_setTableColumnBGColor("RecipeElements", "Value", "{204,255,153}");
	  setValue("RecipeClasses", "enabled", false);
	  setValue("RecipeInstances", "enabled", false);
	  setValue("RecipeElements", "namedColumnEditable", "Value", true);
//  }
}


void unRecipeFunctions_editRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo, int row=-1, string column="")
{  
  string sSelectedDpName, sRecipeState;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeElements") || !shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  unRecipeFunctions_getSelectedRecipeDp(sSelectedDpName, exceptionInfo);
  if (sSelectedDpName!=sRecipeDp) {
	  // If the instance selection failes, returns
	  if (!unRecipeFunctions_selectRecipeInstance(sRecipeDp, exceptionInfo)) 
		return;
  }
	
  // Recipe instance in EDIT mode
  dpSet(sRecipeDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_EDIT);
	  
  // Modifies the color of the recipe values
  _unRecipeFunctions_setTableColumnBGColor("RecipeElements", "Value", "{204,255,153}");
  setValue("RecipeClasses", "enabled", false);
  setValue("RecipeInstances", "enabled", false);
  setValue("RecipeElements", "namedColumnEditable", "Value", true);
  
  g_editingRowNumber = row;
  g_editingColName = column;
}



void unRecipeFunctions_saveRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo)
{
  int colCount, colIndex = -1, status;
  string colName;
  dyn_float df;
  dyn_string ds, message;
  dyn_int errorLines;
  dyn_anytype values;
  dyn_dyn_mixed recipeObject;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeElements") || !shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	//Debug("Shape doesn't exist!\n");
	return;
  } 
  
  // Get the column index for the values
  getValue("RecipeElements", "columnCount", colCount);
  for (int i=0; i<colCount; i++) {
    colName = RecipeElements.columnName(i);
	if (colName == "Value") {
		colIndex = i;
		break;
	}
  }
  
  if (colIndex<0) {
	fwException_raise(exceptionInfo, "ERROR", "The 'RecipeElements' table doesn't have a column named 'Value'.","");
	return;	
  }
  getValue("RecipeElements", "getColumnN", colIndex, values);
  
  
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { DebugTN(exceptionInfo); return;}
  
  // Check the recipe values
  unRecipeFunctions_checkRecipeValues(recipeObject, values, errorLines, exceptionInfo);
  //Debug("Values: " + values);
  //Debug("ErrorLines: " + errorLines);

  // check if the range of any value is wrong
  if (dynlen(errorLines)) {
    int len=dynlen(errorLines);
    _unRecipeFunctions_writeInRecipeLog("The value of some recipe elements is out of range");
	
    for (int i=1; i<=len; i++) {
      int line = (int)errorLines[i];
      setValue("RecipeElements", "cellBackColRC", (line-1), "Value", "{255,64,64}");
    }
    return;
  }
  
  // Asks for user confirmation
  ChildPanelOnCentralReturn("fwGeneral/fwOkCancel.pnl",
							"Save recipe values",
							makeDynString("$sTitle:Save recipe values ",
										  "$text: Do you want to save the recipe values?"  ),
							df,
							ds);
  
  if (df[1]<=0) return;
  
  _unRecipeFunctions_saveRecipeValues(recipeObject, values, exceptionInfo); 
  recipeObject[fwConfigurationDB_RO_VALUE] = values;
  
  // sets the formatted values in the table
  for (int i=0; i<dynlen(values); i++) {
    setValue("RecipeElements", "cellValueRC", i, "Value", values[i+1]);
  }
  _unRecipeFunctions_resetRecipeElementsBgColor();
  _unRecipeFunctions_loadHistoryTableInfo(sRecipeDp, recipeObject);
  _unRecipeFunctions_loadPropertiesTableInfo(sRecipeDp, recipeObject);
  
  setValue("RecipeClasses", "enabled", true);
  setValue("RecipeInstances", "enabled", true);
  setValue("RecipeElements", "namedColumnEditable", "Value", false);
  
  dpSet(sRecipeDp+".ProcessInput.State", "");
  g_editingRowNumber = -1;
  g_editingColName = "";
}


void _unRecipeFunctions_writeInRecipeLog(string message) {
  int iElements;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeLog")) {
	return;
  } 

  string sTime = formatTime("[%Y.%m.%d %H:%M:%S]", getCurrentTime());
  RecipeLog.appendItem(sTime + " " + message);
  
  getValue("RecipeLog", "itemCount", iElements);
  setValue("RecipeLog", "bottomPos", iElements);
}


void unRecipeFunctions_cancelEditRecipeInstance(string sRecipeDp) {
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeElements") || !shapeExists("RecipeClasses") || !shapeExists("RecipeInstances") ) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  } 
  
  dpSet(sRecipeDp+".ProcessInput.State", "");
  
  setValue("RecipeClasses", "enabled", true);
  setValue("RecipeInstances", "enabled", true);
  setValue("RecipeElements", "namedColumnEditable", "Value", false);
  _unRecipeFunctions_resetRecipeElementsBgColor(); 
  unRecipeFunctions_loadRecipeInstanceData();
  
  g_editingRowNumber = -1;
  g_editingColName = "";
}



void _unRecipeFunctions_getRecipeObjectFromDp (string sRecipeDp, dyn_dyn_mixed &recipeObject, dyn_string &exceptionInfo)
{
  int pos;
  string recipeClass, recipeInstance;
  
  if ((pos=strpos(sRecipeDp, "."))>=0) {
	sRecipeDp = substr(sRecipeDp, 0, pos);
  }
  
  if (!dpExists(sRecipeDp+".ProcessInput.ClassName") || 
      !dpExists(sRecipeDp+".ProcessInput.InstanceName"))
  {
	fwException_raise(exceptionInfo, "ERROR", "The recipe DP '" + sRecipeDp + "' doesn't exist.","");
	return;
  }  
  
  dpGet(sRecipeDp+".ProcessInput.ClassName", recipeClass,
		sRecipeDp+".ProcessInput.InstanceName", recipeInstance);
		
  fwConfigurationDB_loadRecipeFromCache(recipeClass+"/"+recipeInstance, makeDynString(), "", recipeObject, exceptionInfo);
}


/** 
 * Creates a new recipe instance of the selected recipe class
 */
void unRecipeFunctions_createRecipeInstance()
{
  string sRecipeClass;
  dyn_string exceptionInfo, ds;
  dyn_float df;
  
  // Check that all the necessary shapes exist
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances") ) {
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  } 
  
  getValue("RecipeClasses", "selectedText", sRecipeClass);
  if (sRecipeClass=="" || sRecipeClass=="-None-") {
    fwException_raise(exceptionInfo,"ERROR", "Please, select a recipe class before creating a recipe.","");
    return;
  }
			
  ChildPanelOnCentralReturn("vision/unRecipe/UnRcpInstance/unUnRcpInstance_Duplicate.pnl",
			"Create Recipe Instance",
			makeDynString("$sTitle:Create Recipe of class: " + sRecipeClass,
				      "$sDescription: Input the name for the new recipe instance"),
			df,ds);
  
  if (!dynlen(df) || df[1]<0) return;
  if (!dynlen(ds) || ds[1]=="") {
    fwException_raise(exceptionInfo,"ERROR", "The recipe name can't be empty. ","");
    return;     
  }  			
  
  _unRecipeFunctions_writeInRecipeLog("Creating the recipe '" + ds[1] + "' of class '" + sRecipeClass + "' ...");
  _unRecipeFunctions_createRecipeInstance(sRecipeClass, ds[1], ds[2], exceptionInfo);
  if (dynlen(exceptionInfo)) { 
    _unRecipeFunctions_writeInRecipeLog("Error creating the recipe '" + ds[1] + "'. Check the log window to get the error details.");
	DebugTN(exceptionInfo); 
	return; 
  }
  _unRecipeFunctions_writeInRecipeLog("The recipe '" + ds[1] + "' of class '" + sRecipeClass + "' has been created.");
  
  _unRecipeFunctions_loadRecipeInstances(sRecipeClass);
}


/** 
 * Creates a new recipe instance.
 * @param sClassName - [IN] Name of the recipe class.
 * @param sInstanceName - [IN] Name of the new recipe instance.
 * @param sDescription - [IN] Description for the new recipe instance.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 * @param dsRcpValues - [IN] Recipe values. If the parameter isn't specified, the recipe elements will have the online values.
 */ 
void _unRecipeFunctions_createRecipeInstance(string sClassName, string sInstanceName, string sDescription, 
											 dyn_string &exceptionInfo, dyn_string dsRcpValues=makeDynString())
{
  int iDeviceNumber, iDriverNumber, iLen, iNumInstances;
  string sDpName, sPrefix, sDpeValue = "", sPvssDpe, sFrontEndName, sFrontEndDp, sAppName, sDomain, sNature;
  dyn_mixed recipeClass;
  dyn_string dsConfig, dsRunningArchive, dsSplitDp, dsOnlineDpes, dsRcpNames, dsArchBool, dsArchAnalog, dsArchEvent, dsParameters;
  dyn_dyn_anytype resultTable;
  bool bEvent;
  
  // check if the recipe instance already exists
  // Get all the instances of the recipe class
  dpQuery("SELECT '.ProcessInput.InstanceName:_online.._value' FROM '*' WHERE '.ProcessInput.ClassName:_online.._value' = \""+sClassName+"\" AND _DPT=\""+UN_CONFIG_UNRCPINSTANCE_DPT_NAME+"\"", resultTable); 
  
  if (dynlen(resultTable)>1) {
    dynRemove(resultTable, 1);
    for (int i=1; i<=dynlen(resultTable); i++) {
		// If the recipe instance already exists, show an error message and return
		if (resultTable[i][2] == sInstanceName) {
			fwException_raise(exceptionInfo,"ERROR", "The recipe " + sClassName + "/" + sInstanceName +" already exists. ","");
			return;	
		}
    }
  }

  // Get necessary data to create a new recipe instance
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sClassName, sPvssDpe);
  sFrontEndName = unGenericObject_GetPLCNameFromDpName(sPvssDpe);
  strreplace(sFrontEndName, SOFT_FE_DPTYPE+"_", "");
  sAppName = unGenericDpFunctions_getApplication(sPvssDpe);
  unGenericDpFunctions_getParameters(sPvssDpe, dsParameters);
  sDomain = dsParameters[4];
  sNature = dsParameters[5];

  
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sFrontEndName, sFrontEndDp);
  dpGet(sFrontEndDp + "configuration.archive_bool", dsArchBool,
		sFrontEndDp + "configuration.archive_analog", dsArchAnalog,
		sFrontEndDp + "configuration.archive_event", dsArchEvent,
		sFrontEndDp + "communication.driver_num", iDriverNumber);
  
  fwConfigurationDB_getRecipeClassInfo(sClassName, recipeClass, exceptionInfo);
  if (dynlen(exceptionInfo) || dynlen(recipeClass[fwConfigurationDB_RCL_ELEMENTS])==0 ) return;
  
  // Creates a unique recipe DP that is not existing yet
  _unRecipeFunctions_createNewRecipeDp(UN_CONFIG_UNRCPINSTANCE_DPT_NAME, sFrontEndName, sAppName, sDpName, iDeviceNumber);
 
  // Initialize the fwDevice and value archive global variables
  unImportDevice_initialize();
  unImportDevice_getApplicationCharacteristics(bEvent, sPrefix);

  iLen = dynlen(recipeClass[fwConfigurationDB_RCL_ELEMENTS]);
  int i=1;
  if (dynlen(dsRcpValues)==0 || dynlen(dsRcpValues)!=iLen) { 
	  // If no values specified, get the online values
	  do {
		dsSplitDp = strsplit(recipeClass[fwConfigurationDB_RCL_ELEMENTS][i], ".");
		unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dsSplitDp[1] + "." + dsSplitDp[dynlen(dsSplitDp)], sPvssDpe);
		_unRecipeFunctions_getOnlineValueDpes (sPvssDpe, dsOnlineDpes);
		dpGet(dsOnlineDpes, dsRcpValues);
		if (i>1) sDpeValue += ",";
		sDpeValue += dsSplitDp[1] + "." + dsSplitDp[dynlen(dsSplitDp)] + "|" + dsRcpValues[1];
		i++;
	  } while (i<=iLen);
  } else {
	  // Set the specified values for the recipe 
	  do {
		dsSplitDp = strsplit(recipeClass[fwConfigurationDB_RCL_ELEMENTS][i], ".");
		if (i>1) sDpeValue += ",";
		sDpeValue += dsSplitDp[1] + "." + dsSplitDp[dynlen(dsSplitDp)] + "|" + dsRcpValues[i];
		i++;
	  } while (i<=iLen);
  }
  
  
  // Creates the dsConfig for the device
  dynAppend(dsConfig, UN_CONFIG_UNRCPINSTANCE_DPT_NAME);	// Recipe instance DPType name
  dynAppend(dsConfig, iDeviceNumber);						// Device number
  dynAppend(dsConfig, sClassName+"/"+sInstanceName);		// Device alias
  dynAppend(dsConfig, sDescription);						// Description
  dynAppend(dsConfig, "");									// Diagnostics
  dynAppend(dsConfig, "");									// WWWLink
  dynAppend(dsConfig, "");									// Synoptic
  dynAppend(dsConfig, sDomain);								// Domain
  dynAppend(dsConfig, sNature);								// Nature
  dynAppend(dsConfig, "UnRcpInstanceState");				// Widget Type
  dynAppend(dsConfig, sClassName);							// Recipe class
  dynAppend(dsConfig, "FALSE");								// Initial recipe
  dynAppend(dsConfig, sDpeValue);							// Recipe DPE-Value definitions


  unImportDevice_import(dsConfig, exceptionInfo, SOFT_FE_DPTYPE, sFrontEndName,
					    sAppName, sPrefix, iDriverNumber,
						dsArchBool[1], dsArchAnalog[1], dsArchEvent[1], bEvent,
						dsRunningArchive);
  if (dynlen(exceptionInfo)) return; 
  
  unImportDevice_triggerUpdate();
  
  // Set the recipe instance as not predefined
  dpSet(fwConfigurationDB_RecipeCacheDpPrefix + sClassName + "/" + sInstanceName + ".MetaInfo.Predefined", false);
}




/** 
 * Modify the font type of the edited rows to Bold.
 * @param sDpName - [IN] The data point name of the edited row.
 */
void _unRecipeFunctions_highlightEditedValue(string sDpName)
{
	int retVal;
	string newValue;
	dyn_string exceptionInfo, formattedValues;
	dyn_dyn_mixed recipeObject;
	
	if (g_editingRowNumber < 0 || g_editingColName=="") {
		return;
	}
	
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo);
	getValue("RecipeElements", "cellValueRC", g_editingRowNumber, g_editingColName, newValue);
					
	_unRecipeFunctions_formatValues(makeDynString(recipeObject[fwConfigurationDB_RO_DPE_NAME][g_editingRowNumber+1]), 
					makeDynString(newValue), formattedValues, exceptionInfo);
								
	setValue ("RecipeElements", "cellValueRC", g_editingRowNumber, g_editingColName, formattedValues[1]);
	if (formattedValues[1] != recipeObject[fwConfigurationDB_RO_VALUE][g_editingRowNumber+1]) {
		RecipeElements.rowFontType (g_editingRowNumber+1, makeDynInt(1));
	}
	
	g_editingRowNumber = -1;
	g_editingColName = "";	
}


/** 
 * The edition of a cell value has been completed.
 * @param sDpName - [IN] The data point name of the recipe instance.
 * @param row	  - [IN] Row number of the edited cell.
 * @param column  - [IN] Column name of the edited cell.
 */
void _unRecipeFunctions_cellEditionFinished(string sDpName, int row, string column)
{
	int retVal;
	string newValue;
	dyn_string exceptionInfo, formattedValues;
	dyn_dyn_mixed recipeObject;
	
	_unRecipeFunctions_getRecipeObjectFromDp (sDpName, recipeObject, exceptionInfo);
	getValue("RecipeElements", "cellValueRC", row, column, newValue);
	
	// Check if the recipe value is in the defined range.
	retVal = unRecipeFunctions_checkRecipeValue (
					recipeObject[fwConfigurationDB_RO_DPE_NAME][row+1], 
					newValue, 
					exceptionInfo);
	
	// If the value is out of range shows an error message and modifies the cell color
	if (retVal == -1) {
	  _unRecipeFunctions_writeInRecipeLog("The new value is out of range");
	  setValue("RecipeElements", "cellBackColRC", (row), "Value", "{255,64,64}");
	  return;
	} else {
		// Restore the cell color
		setValue("RecipeElements", "cellBackColRC", (row), "Value", "{204,255,153}");
	}
	
	_unRecipeFunctions_formatValues(makeDynString(recipeObject[fwConfigurationDB_RO_DPE_NAME][row+1]), 
								    makeDynString(newValue), formattedValues, exceptionInfo);
	setValue ("RecipeElements", "cellValueRC", row, column, formattedValues[1]);
	if (formattedValues[1] != recipeObject[fwConfigurationDB_RO_VALUE][row+1]) {
		RecipeElements.rowFontType (row, makeDynInt(1));
	} 
	
	g_editingRowNumber = -1;
	g_editingColName = "";	
}


/** 
 * Duplicate the selected recipe instance.
 * @param sRecipeDp - [IN] DP name of the recipe instance to duplicate.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_duplicateRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo)
{
  string sRecipeInstance, sRecipeClass, sRecipeDesc;
  dyn_float df;
  dyn_string ds;
  dyn_dyn_mixed recipeObject;  
  
  // Recipe instance in DUPLICATE mode
  dpSet(sRecipeDp+".ProcessInput.State", UN_RCP_INSTANCE_STATE_DUPLICATE);
  
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
    dpSet(sRecipeDp+".ProcessInput.State", "");
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
	dpSet(sRecipeDp+".ProcessInput.State", "");
	DebugTN(exceptionInfo);	
	return; 
  }
  
  ChildPanelOnCentralReturn("vision/unRecipe/UnRcpInstance/unUnRcpInstance_Duplicate.pnl",
			"Duplicate Recipe Instance",
			makeDynString("$sTitle:Duplicate Recipe Instance",
				      "$sDescription: Input the name for the new recipe instance"),
			df,ds);
  
  if (!dynlen(df) || df[1]<0) {
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  if (!dynlen(ds) || ds[1]=="") {
    dpSet(sRecipeDp+".ProcessInput.State", "");
    fwException_raise(exceptionInfo,"ERROR", "The recipe name can't be empty. ","");
    return;     
  }  
  
  sRecipeInstance = ds[1];
  sRecipeDesc = ds[2];
  sRecipeClass = recipeObject[fwConfigurationDB_RO_META_CLASSNAME];   
  
  _unRecipeFunctions_writeInRecipeLog("Creating the recipe '" + sRecipeInstance + "' of class '" + sRecipeClass + "' ...");
  _unRecipeFunctions_createRecipeInstance(sRecipeClass, sRecipeInstance, sRecipeDesc, 
										  exceptionInfo, recipeObject[fwConfigurationDB_RO_VALUE]);
  
  if (dynlen(exceptionInfo)) { 
	dpSet(sRecipeDp+".ProcessInput.State", "");
	DebugTN(exceptionInfo); 
	_unRecipeFunctions_writeInRecipeLog("Error creating the recipe '" + sRecipeInstance + "'. Check the log window to get the error details.");
	return; 
  }
  _unRecipeFunctions_writeInRecipeLog("The recipe '" + sRecipeInstance + "' of class '" + sRecipeClass + "' has been created.");
  _unRecipeFunctions_loadRecipeInstances(sRecipeClass);
  dpSet(sRecipeDp+".ProcessInput.State", "");
}


/** 
 * Delete a recipe instance.
 * @param sRecipeDp - [IN] Data point name of the recipe instance to delete.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_deleteRecipeInstance(string sRecipeDp, dyn_string &exceptionInfo)
{
  int rc, iPos;
  dyn_dyn_mixed recipeObject;
  dyn_string recipeInstances, ds;
  dyn_float df;
  string sClassName, sInstanceName, sPvssDpe;
  bool bPredefined, bInitial;
  
  // Set the recipe State to "Delete"
  dpSet($sDpName+".ProcessInput.State", UN_RCP_INSTANCE_STATE_DELETE);
  
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances") || 
	  !shapeExists("SelectionProgressBar") || !shapeExists("PreviousRecipeDp") ) 
  {
	dpSet($sDpName+".ProcessInput.State", "");
	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  _unRecipeFunctions_getRecipeObjectFromDp(sRecipeDp, recipeObject, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
    dpSet($sDpName+".ProcessInput.State", "");
	DebugTN(exceptionInfo);	
	return; 
  }
  
  // Get the predefined and inital values of the recipe instance
  bPredefined = recipeObject[fwConfigurationDB_RO_META_PREDEFINED][1];
  // TODO: Get the initial flag when it's included in the FwRecipeCache DP
  bInitial = false;
  if (bPredefined || bInitial) {
	_unRecipeFunctions_writeInRecipeLog("Error: The predefined and initial recipes can't be deleted.");
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  sClassName = recipeObject[fwConfigurationDB_RO_META_CLASSNAME];
  sInstanceName = recipeObject[fwConfigurationDB_RO_META_ORIGNAME];
  if ((iPos=strpos(sInstanceName, "/")) >=0 ) {
	sInstanceName = substr(sInstanceName, iPos+1);
  }
  
  // Asks for user confirmation
  ChildPanelOnCentralReturn("fwGeneral/fwOkCancel.pnl",
							"Delete Recipe",
							makeDynString("$sTitle:Delete recipe ",
										  "$text: Do you want to delete the recipe " + sClassName + "/" + sInstanceName + "?"  ),
							df,
							ds);
  
  if (df[1]<=0)  {
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  setValue("LoadedRecipeDp", "text", "");
  unSelectDeselectHMI_select(sRecipeDp, false, exceptionInfo);
  unProgressBar_setPosition("SelectionProgressBar", 0);    
  
  // Delete the recipe instance from the fwConfigurationDB
  fwConfigurationDB_deleteRecipeOfClass(sInstanceName, sClassName, exceptionInfo);
  if (dynlen(exceptionInfo)) { 
	DebugTN(exceptionInfo);	
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return; 
  }
  
  // Gets the list of recipe instances of the recipe class
  unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(sClassName, sPvssDpe, exceptionInfo);
  rc = dpGet(sPvssDpe+"ProcessInput.Recipes",recipeInstances);
  if (rc) {
  	DebugTN("unRecipeFunctions_deleteRecipeInstance: Error while getting the data of the recipe class '" +sClassName+"'","");
	dpSet(sRecipeDp+".ProcessInput.State", "");
	return;
  }
  
  // Remove the recipe instance of the list of recipes that belong to the recipe class
  if ((iPos=dynContains(recipeInstances, sInstanceName)) > 0) {
	dynRemove(recipeInstances, iPos);
	rc = dpSet(sPvssDpe+"ProcessInput.Recipes", recipeInstances);
	if (rc) {
		DebugTN("unRecipeFunctions_deleteRecipeInstance: Error while setting the recipe instances of the recipe class '" +sClassName+"'","");
		dpSet(sRecipeDp+".ProcessInput.State", "");
		return;	
	}
  }
  
  // Delete the recipe instance
  rc = dpDelete(sRecipeDp);
  if (rc < 0) {
	DebugTN ("Error deleting the recipe instance " + sClassName + "/" + sInstanceName);
	dpSet(sRecipeDp+".ProcessInput.State", "");
  }
   
  _unRecipeFunctions_loadRecipeInstances(sClassName);
}



/**
 * Creates a dp name that is not existing already
 * @param sDeviceType - [IN] Recipe device type name.
 * @param sFrontEndName - [IN] Front end name where the recipe will be linked.
 * @param sAppName - [IN] Application name.
 * @param sDpName - [OUT] Data point name for the recipe instance.
 * @param iDeviceNumber - [OUT] Device number for the recipe.
 */
void _unRecipeFunctions_createNewRecipeDp(string sDeviceType, string sFrontEndName, string sAppName, string &sDpName, int &iDeviceNumber)
{
  string sRandomNumber, sTmp, sPrefix;
  
  dpGet("_unApplication.devicePrefix", sPrefix);
  sDpName = sPrefix + "-" + sFrontEndName + "-" + sAppName + "-" + sDeviceType + "-";
  
  do {
	  sRandomNumber="";
	  for (int i=1; i<=5; i++) {
		sTmp = rand();
		sRandomNumber += sTmp[strlen(sTmp)-1];    
	  }
  } while (dpExists(sDpName+sRandomNumber));
    
  sDpName += sRandomNumber;
  iDeviceNumber = (int) sRandomNumber;
}


/** 
 * Get the data point name of the recipe instance selected in the panel.
 * @param sSelected - [OUT] DP name of the recipe instance selected in the panel.
 * @param exceptionInfo - [OUT] Standard exception handling variable.
 */
void unRecipeFunctions_getSelectedRecipeDp(string &sSelected, dyn_string &exceptionInfo)
{
  sSelected = "";
  string sRecipeClass;
  int iLineCount;
  dyn_int diLines;
  
  if (!shapeExists("RecipeClasses") || !shapeExists("RecipeInstances")) {
  	fwException_raise(exceptionInfo, "ERROR", "Some graphical shapes are missing.","");
	return;
  }
  
  getValue("RecipeClasses", "text", sRecipeClass);
  if (sRecipeClass=="")	return; 
  
  getValue("RecipeInstances", "lineCount", iLineCount);
  
  getValue("RecipeInstances", "getSelectedLines", diLines);
  if (!dynlen(diLines) || (diLines[1] >= iLineCount))  return;

  getValue("RecipeInstances", "cellValueRC", diLines[1], "RecipeInstancesDP", sSelected);
}



//--------------------------------------------------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//		Methods to edit the color on a table 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void _unRecipeFunctions_setTableColumnBGColor(string tableName, string columnName, string color)
{
  int lines;
  int rc=getValue(tableName, "lineCount", lines);
  if(rc)  return;
  
  for (int i=0; i<=lines; i++) {
    setValue(tableName, "cellBackColRC", i, columnName, color);
  }  
}

void _unRecipeFunctions_setTableRowBGColor(string tableName, int rowIndex, string color)
{
  int cols; 
  int rc=getValue(tableName, "columnCount", cols);
  if(rc)  return;
  
  for (int i=0; i<=cols; i++) {
    string colName;
    getValue(tableName, "columnName", i, colName);
    setValue(tableName, "cellBackColRC", rowIndex, colName, color);
  }  
}


void _unRecipeFunctions_resetRecipeElementsBgColor() {
  int lines;

  getValue("RecipeElements", "lineCount", lines);
  for (int i=1; i<=lines; i++){
    if (i%2==1)
      _unRecipeFunctions_setTableRowBGColor("RecipeElements", (i-1), "{227,230,253}");  
    else
      _unRecipeFunctions_setTableRowBGColor("RecipeElements", (i-1), "{255,255,255}");
	// Resets the font style
	RecipeElements.rowFontType (i-1, makeDynInt(0));
  }
}


void _unRecipeFunctions_setInvalidRows(dyn_string rows)
{
  int len=dynlen(rows);
  for (int i=1; i<=len; i++) {
    int row = rows[i];
    _unRecipeFunctions_setTableRowBGColor("RecipeElements", (i-1), "{255,0,0}"); 
  }
}
