/**@file

// unSystemIntegrity_LHCLogging.ctl
This library contains the system integrity functions of the LHCLogging manager.

@par Creation Date
  12/07/2002

@par Modification History
  03/10/2011: Herve
  - IS-600  unLHCServices  system name and component version added in the MessageText log and in the diagnostic 
 
  27/09/2011: Herve
  - IS-621  unLHCServices  LHCLogging systemIntegrity: no more kill of the PVSS00LHCLogging in case of alarm 
 
  16/06/2011: Herve
  - IS-556: get the list of systemAlarm pattern
  
  - IS-508: LHCLogging systemIntegrity: allow few times an error before raising an alarm, number of allowed errors configurable, 
  automatic kill and restart the LHCLogging if needed, extended diagnostic
  
  - 29/11/2004: Herve
    move the LHCLogging_pattern into unicosObjects_declarations.ctl
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@author 
  Herve Milcent (EN-ICE)
*/

//@{
//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//------------------
const string UN_SYSTEM_INTEGRITY_LHCLogging = "LHCLogging";
const int UN_LHCLOGGING_NBALLOWEDALARM = 4;
const int UN_LHCLOGGING_TIMEBEFORERESTART = 2;

const int c_systemIntegrityLHCLogging_defaultCheckingDelay = 10; // minimum 10sec.

const int UN_LHCLOGGING_STATEOUT = 1;
const int UN_LHCLOGGING_MESSAGEOUT = 2;
const int UN_LHCLOGGING_ARCHIVE_STATE =3;

const string LHCLOGGING_MANAGER_ARCHIVE_ALARMSTATE = "../msg/en_US.iso88591/unLHCLoggingManagerArchive_AlarmState.cat";
const string LHCLOGGING_MANAGER_ARCHIVE_OKSTATE = "../msg/en_US.iso88591/unLHCLoggingManagerArchive_OKState.cat";

// 
//------------------

// global declaration
global dyn_string g_unSystemIntegrity_LHCLoggingList; // list of the LHCLogging dp checked
global dyn_string g_unSystemIntegrity_LHCLoggingManagerList; // list of the LHCLogging dp checked
global dyn_int g_unSystemIntegrity_LHCLoggingManagerNumber; // list of the LHCLogging manager num
global int g_iLHCLogging_nbAllowedAlarm;        // nb of delay iteration with alarm before raising a systemIntegrity alarm
global mapping g_m_unSystemIntegrity_LHCLogging_archive; // one entry per manager containing the list of archives
global mapping g_m_unSystemIntegrity_LHCLogging_state; // one entry per manager with 2 int per manager
global mapping g_m_unSystemIntegrity_Archive_state; // one entry per archive with 3 int per archive
global mapping g_m_unSystemIntegrity_Archive_LastRefStart; // one entry per archive with last time of start query
global mapping g_m_unSystemIntegrity_Archive_LHCLoggingManager; // one entry per archive containing the corresponding LHCLogging manager
global dyn_int g_diLogging_API;          //API started List
global int g_iLogging_ArchiveTimeout;      //Max Timeout between Now and last RefStart for Archive
global int g_iLogging_ArchiveDelay;        //Check LHCLogging states period ....
global dyn_int g_diLogging_Thread;
global bool g_b_Connection_done;
global dyn_int g_di_unSystemIntegrity_LHCLogging_oldState;
global dyn_int g_diAlarmState, g_diOKState;
global int g_iLHCLogging_nbTimeAlarmBeforeRestart;        // nb of delay iteration with alarm before raising a systemIntegrity alarm

global bool g_bPanel;							//SystemIntegrity = false / unLHCLoggingStatus.pnl = True

//------------------------------------------------------------------------------------------------------------------------
// LHCLogging_systemIntegrityInfo
/** Return the list of systemAlarm pattern. 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@return list of systemAlarm pattern
*/
dyn_string LHCLogging_systemIntegrityInfo()
{
  return makeDynString(LHCLogging_pattern);
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLogging_Initialize
/** Get the list of LHCLogging check by the systemIntegrity. The ones that are enabled.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param dsResult output, list of enabled front-end systemIntegrity
*/
unSystemIntegrity_LHCLogging_Initialize(dyn_string &dsResult)
{
  dyn_string dsList;
  int len, i, iRes;
  string dpToCheck;
  bool enabled;
  string sMessage;
  dyn_string exceptionInfo;
  
  sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("unLHCLogging"));

  //initial Values which will be see as erros
  unCoreLHCServices_initialize(c_Logging_Pattern);
  unSystemIntegrity_LHCLogging_load(LHCLOGGING_MANAGER_ARCHIVE_ALARMSTATE, g_diAlarmState);
  unSystemIntegrity_LHCLogging_load(LHCLOGGING_MANAGER_ARCHIVE_OKSTATE, g_diOKState);
   
  // get the list of LHCLogging manager dp to checked
  dsList = dpNames(c_unSystemAlarm_dpPattern+LHCLogging_pattern+"*", c_unSystemAlarm_dpType);
  //DebugN("LHCLogging:", dsList);
  len = dynlen(dsList);
  for(i = 1; i<=len; i++) {
    dpToCheck = dpSubStr(dsList[i], DPSUB_DP);  
    dpToCheck = substr(dpToCheck, strpos(dpToCheck,LHCLogging_pattern));
    dpGet(dsList[i]+".enabled", enabled);
    if(enabled)
      dynAppend(dsResult, dpToCheck);
  }
  unMessageText_send("*", "*", g_SystemIntegrity_sSystemName+"systemIntegrity", "user", "*", "INFO", "systemIntegrity "+sMessage+" loaded", exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLogging_load
/** load the data from a catalog
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param sFileName  input, catalog name
@param diValue outinput, values from the catalog
*/
unSystemIntegrity_LHCLogging_load(string sFileName, dyn_int &diValue)
{
  string sBuffer;
  dyn_string dsBuffer, dsTemp;
  int i, len;
  int iStatus;
  dyn_int diTemp;

  dynClear(diValue);
  fileToString(sFileName, sBuffer);
  strreplace(sBuffer,"\r","");
  dsBuffer=strsplit(sBuffer, "\n");
  len = dynlen(dsBuffer);
  for(i=1;i<=len;i++)
  {
    dsTemp = strsplit(dsBuffer[i], ",");
    if(dynlen(dsTemp) >=2) {
      sscanf(dsTemp[2],"%d", iStatus);
      dynAppend(diTemp, iStatus);
    }
  }
  diValue = diTemp;
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLogging_DataCallback
/** Callback on the configuration data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param sDpe1  input, dpe
@param dsConfigData input, configuration data
*/
unSystemIntegrity_LHCLogging_DataCallback(string sDpe1, dyn_string dsConfigData)
{
  dyn_string dsTemp=dsConfigData;
  int iLHCLogging_nbAllowedAlarm, iBeforeTimeOut;
  
  while(dynlen(dsTemp) <=2)
    dynAppend(dsTemp, "0");
  iLHCLogging_nbAllowedAlarm = dsTemp[1];
  if(iLHCLogging_nbAllowedAlarm == 0 )
    iLHCLogging_nbAllowedAlarm = UN_LHCLOGGING_NBALLOWEDALARM;
  g_iLHCLogging_nbAllowedAlarm = iLHCLogging_nbAllowedAlarm;
  iBeforeTimeOut = dsTemp[2];
  if(iBeforeTimeOut == 0)
    iBeforeTimeOut = UN_LHCLOGGING_TIMEBEFORERESTART;
  g_iLHCLogging_nbTimeAlarmBeforeRestart = iBeforeTimeOut;
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLogging_DataCallback", dsConfigData, "ndAllo:"+g_iLHCLogging_nbAllowedAlarm, "nbRes:"+g_iLHCLogging_nbTimeAlarmBeforeRestart);
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLogging_HandleCommand
/** Callback on the systemIntegrity command.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param sDpe1  input, dpe
@param command input, type of command
@param sDpe2  input, dpe
@param parameters input, parameters of the command
*/
unSystemIntegrity_LHCLogging_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
  dyn_string exceptionInfo;
  int i, len =dynlen(parameters),j, lenj;
  string sResult;
  dyn_string dsResult, dsTemp;
  string sMessage;
    
  switch(command) {
    case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        unSystemIntegrity_LHCLogging_checking(parameters[i], true, true, exceptionInfo);
      }
      break;
    case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        unSystemIntegrity_LHCLogging_checking(parameters[i], false, false, exceptionInfo);
        // remove dp from the alert list of applicationDP if it is in
        _unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+parameters[i], false, exceptionInfo);
        // delete the _UnSystemAlarm dps
        if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
          dpDelete(c_unSystemAlarm_dpPattern+parameters[i]);
      }
      break;
    case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
          unSystemIntegrity_LHCLogging_checking(parameters[i], false, true, exceptionInfo);
      }
      break;
    case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
          unSystemIntegrity_LHCLogging_checking(parameters[i], false, false, exceptionInfo);
      }
      break;
    case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
      dpSet(UN_SYSTEM_INTEGRITY_LHCLogging+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
          UN_SYSTEM_INTEGRITY_LHCLogging+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_unSystemIntegrity_LHCLoggingList);
      break;
    case UN_SYSTEMINTEGRITY_EXTENDED_DIAGNOSTIC:
      synchronized(g_unSystemIntegrity_LHCLoggingList)
      {
        dynAppend(dsResult, getCurrentTime());
        len=dynlen(g_unSystemIntegrity_LHCLoggingList);
        dynAppend(dsResult, "g_unSystemIntegrity_LHCLoggingList: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_unSystemIntegrity_LHCLoggingList[i];
          dynAppend(dsResult, sResult);
        }
        len=dynlen(g_unSystemIntegrity_LHCLoggingManagerList);
        dynAppend(dsResult, "g_unSystemIntegrity_LHCLoggingManagerList: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_unSystemIntegrity_LHCLoggingManagerList[i];
          dynAppend(dsResult, sResult);
        }
        len=dynlen(g_unSystemIntegrity_LHCLoggingManagerNumber);
        dynAppend(dsResult, "g_unSystemIntegrity_LHCLoggingManagerNumber: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_unSystemIntegrity_LHCLoggingManagerNumber[i];
          dynAppend(dsResult, sResult);
        }
        len=mappinglen(g_m_unSystemIntegrity_LHCLogging_archive);
        dynAppend(dsResult, "g_m_unSystemIntegrity_LHCLogging_archive: "+len);
        for(i=1;i<=len;i++)
        {
          dsTemp = mappingGetValue(g_m_unSystemIntegrity_LHCLogging_archive, i);
          lenj=dynlen(dsTemp);
          dynAppend(dsResult, "g_m_unSystemIntegrity_LHCLogging_archive archive: "+lenj);
          for(j=1;j<=lenj;j++)
          {
            sResult="["+mappingGetKey(g_m_unSystemIntegrity_LHCLogging_archive,i)+"]: "+dsTemp[j];
            dynAppend(dsResult, sResult);
          }
        }
        len=mappinglen(g_m_unSystemIntegrity_LHCLogging_state);
        dynAppend(dsResult, "g_m_unSystemIntegrity_LHCLogging_state: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+mappingGetKey(g_m_unSystemIntegrity_LHCLogging_state, i)+"]: "+mappingGetValue(g_m_unSystemIntegrity_LHCLogging_state, i)[UN_LHCLOGGING_STATEOUT];
          dynAppend(dsResult, sResult);
          sResult = "["+mappingGetKey(g_m_unSystemIntegrity_LHCLogging_state, i)+"]: "+mappingGetValue(g_m_unSystemIntegrity_LHCLogging_state, i)[UN_LHCLOGGING_MESSAGEOUT];
          dynAppend(dsResult, sResult);
        }
        len=mappinglen(g_m_unSystemIntegrity_Archive_state);
        dynAppend(dsResult, "g_m_unSystemIntegrity_Archive_state: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+mappingGetKey(g_m_unSystemIntegrity_Archive_state, i)+"]: "+mappingGetValue(g_m_unSystemIntegrity_Archive_state, i)[UN_LHCLOGGING_STATEOUT];
          dynAppend(dsResult, sResult);
          sResult = "["+mappingGetKey(g_m_unSystemIntegrity_Archive_state, i)+"]: "+mappingGetValue(g_m_unSystemIntegrity_Archive_state, i)[UN_LHCLOGGING_MESSAGEOUT];
          dynAppend(dsResult, sResult);
          sResult = "["+mappingGetKey(g_m_unSystemIntegrity_Archive_state, i)+"]: "+mappingGetValue(g_m_unSystemIntegrity_Archive_state, i)[UN_LHCLOGGING_ARCHIVE_STATE];
          dynAppend(dsResult, sResult);
        }
        len=mappinglen(g_m_unSystemIntegrity_Archive_LastRefStart);
        dynAppend(dsResult, "g_m_unSystemIntegrity_Archive_LastRefStart: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+mappingGetKey(g_m_unSystemIntegrity_Archive_LastRefStart, i)+"]: "+mappingGetValue(g_m_unSystemIntegrity_Archive_LastRefStart, i);
          dynAppend(dsResult, sResult);
        }
        len=mappinglen(g_m_unSystemIntegrity_Archive_LHCLoggingManager);
        dynAppend(dsResult, "g_m_unSystemIntegrity_Archive_LHCLoggingManager: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+mappingGetKey(g_m_unSystemIntegrity_Archive_LHCLoggingManager, i)+"]: "+mappingGetValue(g_m_unSystemIntegrity_Archive_LHCLoggingManager, i);
          dynAppend(dsResult, sResult);
        }
        len=dynlen(g_diLogging_API);
        dynAppend(dsResult, "g_diLogging_API: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_diLogging_API[i];
          dynAppend(dsResult, sResult);
        }
        len=dynlen(g_diLogging_Thread);
        dynAppend(dsResult, "g_diLogging_Thread: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_diLogging_Thread[i];
          dynAppend(dsResult, sResult);
        }
        len=dynlen(g_di_unSystemIntegrity_LHCLogging_oldState);
        dynAppend(dsResult, "g_di_unSystemIntegrity_LHCLogging_oldState: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_di_unSystemIntegrity_LHCLogging_oldState[i];
          dynAppend(dsResult, sResult);
        }
        dynAppend(dsResult, "g_iLogging_ArchiveTimeout:" + g_iLogging_ArchiveTimeout);
        dynAppend(dsResult, "g_iLogging_ArchiveDelay:" + g_iLogging_ArchiveDelay);
        dynAppend(dsResult, "g_iLHCLogging_nbAllowedAlarm: "+ g_iLHCLogging_nbAllowedAlarm);
        dynAppend(dsResult, "g_iLHCLogging_nbTimeAlarmBeforeRestart: "+ g_iLHCLogging_nbTimeAlarmBeforeRestart);
        len=dynlen(g_diAlarmState);
        dynAppend(dsResult, "g_diAlarmState: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_diAlarmState[i];
          dynAppend(dsResult, sResult);
        }
        len=dynlen(g_diOKState);
        dynAppend(dsResult, "g_diOKState: "+len);
        for(i=1;i<=len;i++)
        {
          sResult = "["+i+"]: "+g_diOKState[i];
          dynAppend(dsResult, sResult);
        }
      }
      dpSet(UN_SYSTEM_INTEGRITY_LHCLogging+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
          UN_SYSTEM_INTEGRITY_LHCLogging+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", dsResult);
      DebugTN(dsResult);
      break;
    case UN_SYSTEMINTEGRITY_DEBUG:
      if(dynlen(parameters) >= 2)
      {
        g_b32DebugLevel = parameters[1];
        g_sDebugFilter = parameters[2];
      }
      else
      {
        g_b32DebugLevel = 0;
        g_sDebugFilter = "";
      }
      break;
    case UN_SYSTEMINTEGRITY_VERSION:
      sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("unLHCLogging"));
      dpSet(UN_SYSTEM_INTEGRITY_LHCLogging+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
            UN_SYSTEM_INTEGRITY_LHCLogging+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", makeDynString(getCurrentTime(),sMessage));
      break;
    default:
      break;
  }

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLogging_HandleCommand", command, parameters);
  if(dynlen(exceptionInfo) > 0) {
    if(isFunctionDefined("unMessageText_sendException")) {
      unMessageText_sendException("*", "*", "unSystemIntegrity_LHCLogging_HandleCommand", "user", "*", exceptionInfo);
    }
// handle any error uin case the send message failed
    if(dynlen(exceptionInfo) > 0) {
      DebugN(getCurrentTime(), exceptionInfo);
    }
  }
  if(!g_b_Connection_done)
  {
    g_b_Connection_done = true;
    //Register Function
    dpConnect("unSystemIntegrity_LHCLoggingConfig_CB","LHCLogging_Manager_Info.defaultSettings.Timeout", "LHCLogging_Manager_Info.defaultSettings.Delay");
    dpConnect("unSystemIntegrity_Connections_CB","_Connections.Api.ManNums");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLogging_checking
/** This function register/de-register the callback funtion of LHCLogging manager and creates the _unSystemAlarm_LHCLogging dp 
with the alarm config but it does not delete it.

  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sDp  input, data point name
@param bCreate  input, true create the dp and the alarm config if it does not exist, enable it
@param bRegister  input, true do a dpConnect, false do a dpDisconnect
@param exceptionInfoTemp output, exception are returned here
*/
unSystemIntegrity_LHCLogging_checking(string sDp_Num, bool bCreate, bool bRegister, dyn_string &exceptionInfoTemp)
{
  string sSystem, dp_Alarm, dpManager, sNum;
  int iRes, i, len;
  dyn_string dsArchives;

  sSystem=getSystemName();

  //1. Get Manager Name
  dp_Alarm = c_unSystemAlarm_dpPattern + sDp_Num;
  g_bPanel=false;

  //2. Get Archive List
  sDp_Num=substr(sDp_Num,strpos(sDp_Num,"_")+1);
  if (strlen(sDp_Num)<2)
    sNum="0" + sDp_Num;
  else
    sNum=sDp_Num;

  dpManager=LHCLogging_pattern + "Manager_" + sNum;

  if (dpExists(dpManager + ".Settings.man_ArchList"))
  {  
//DebugN(g_unSystemIntegrity_LHCLoggingList, dp_Alarm, iRes);
    if (bRegister)
    {
// check if the dp_alarm is in the global list if not then dpConnect, it is the first time, 
// if yes then it is a second time do not twice dpConnect
      //3. LHCLogging Manager
      unSystemIntegrity_Logging_SetManagerCheck(dpManager, bRegister, bCreate, dp_Alarm, sDp_Num, exceptionInfoTemp);
    }
    else {
      //4. LHCLogging Manager
      unSystemIntegrity_Logging_SetManagerCheck(dpManager, bRegister, bCreate, dp_Alarm, sDp_Num, exceptionInfoTemp);
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLoggingConfig_CB
/** Callback function on the LHCLogging manager config.

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sDpTimeout  input, Timeout dpe
@param Timeout  input, Timeout
@param sDpDelay  input, Delay dpe
@param Delay  input, Delay
*/
unSystemIntegrity_LHCLoggingConfig_CB(string sDpTimeout, int Timeout, string sDpDelay, int Delay)
{
  g_iLogging_ArchiveTimeout=Timeout;
  g_iLogging_ArchiveDelay=Delay;
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLoggingConfig_CB", Timeout, Delay);
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_Connections_CB
/** Callback function on the LHCLogging manager config.

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sDPManNums  input, Timeout dpe
@param diManNums  input, diManNums
*/
unSystemIntegrity_Connections_CB(string sDPManNums, dyn_int diManNums)
{
  string sManager;
  int i, len, iAlarmState, iWarningState;
  
  synchronized(g_unSystemIntegrity_LHCLoggingList)
  {
    g_diLogging_API=diManNums;
    len = dynlen(g_unSystemIntegrity_LHCLoggingManagerList);
    for(i=1;i<=len;i++)
    {
      sManager = g_unSystemIntegrity_LHCLoggingManagerList[i];
      unSystemIntegrity_LHCLogging_getAlarmState(sManager, iAlarmState, iWarningState);
      if(iAlarmState > 0)
      {
        if(g_di_unSystemIntegrity_LHCLogging_oldState[i] != iAlarmState)
        {
          dpSet(g_unSystemIntegrity_LHCLoggingList[i]+".alarm", iAlarmState);
          g_di_unSystemIntegrity_LHCLogging_oldState[i] = iAlarmState;
          unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_Connections_CB set", "sManager:"+sManager, "old:"+g_di_unSystemIntegrity_LHCLogging_oldState[i], "al:"+iAlarmState, "W:"+iWarningState);
        }
      }
    }
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_Connections_CB", diManNums, g_di_unSystemIntegrity_LHCLogging_oldState);
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_Logging_SetManagerCheck
/** initialise all the LHCLogging systemIntegrity CB, thread, and global variables
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param sManager: input, Manager's name
@param bRegister: input, Connect=true / Dicsonnect=false
@param bCreation: input, need to create sDp and its alarm config
@param dp_Alarm: input, Dpe for Alarm Integrity
@param sLHCLogging_Num: Number for the Manager
@param exceptionInfoTemp: output, error returned here
*/
unSystemIntegrity_Logging_SetManagerCheck(string sManager, bool bRegister, bool bCreation, string dp_Alarm, string sLHCLogging_Num, dyn_string &exceptionInfoTemp)
{
  string sDescription, dpeToCheck_1, dpeToCheck_2, dpeToCheck_3, dpeToCheck_4, sTemp;
  int i, iThread, len, iRes, iNum;
  int pos;
  dyn_string dsArchives;

  //1. Get data...
  dpeToCheck_1=sManager + ".State.st_StateOut";
  dpeToCheck_2=sManager + ".State.ev_MessageOut";
  
  //Create the sDp and its alarm config if it is not existing
  if(bCreation)
  {
    if(!dpExists(dp_Alarm))
    {
      sDescription = getCatStr("unSystemIntegrity", "LHCLOGGING_DESCRIPTION")+sLHCLogging_Num;
      unSystemIntegrity_createSystemAlarm(sLHCLogging_Num, LHCLogging_pattern, sDescription, exceptionInfoTemp);
    }
  }
  synchronized(g_unSystemIntegrity_LHCLoggingList)
  {
    if(bRegister)
    {
      pos = _unSystemIntegrity_isInList(g_unSystemIntegrity_LHCLoggingList, dp_Alarm);
      //Connect
      if(pos <= 0)
      {
        //Register Function
        _unSystemIntegrity_registerFunction(bRegister, "unSystemIntegrity_LHCLoggingManagerCB", makeDynString(dpeToCheck_1,dpeToCheck_2), 
            g_unSystemIntegrity_LHCLoggingList, dp_Alarm, exceptionInfoTemp);
        dpGet(sManager + ".Settings.man_ArchList", dsArchives);
        // initialise the global variables
        dynAppend(g_unSystemIntegrity_LHCLoggingManagerList, sManager);
        dynAppend(g_di_unSystemIntegrity_LHCLogging_oldState, -1);
        iNum = sLHCLogging_Num;
        dynAppend(g_unSystemIntegrity_LHCLoggingManagerNumber, iNum);
        g_m_unSystemIntegrity_LHCLogging_state[sManager] = makeDynInt(-1, -1);
        g_m_unSystemIntegrity_LHCLogging_archive[sManager] = dsArchives;
        len = dynlen(dsArchives);
        for(i=1;i<=len;i++)
        {
          g_m_unSystemIntegrity_Archive_state[dsArchives[i]] = makeDynInt(-1, -1, -1);
          g_m_unSystemIntegrity_Archive_LastRefStart[dsArchives[i]] = 0;
          g_m_unSystemIntegrity_Archive_LHCLoggingManager[dsArchives[i]] = sManager;
          sTemp=substr(dsArchives[i],strpos(dsArchives[i],"_ValueArchive_"));
          dpConnect("unSystemIntegrity_LHCLoggingArchiveCB", dsArchives[i]+".State.st_StateOut", dsArchives[i]+".State.ev_MessageOut", 
                    dsArchives[i]+".Settings.arch_LastRefStart", sTemp + ".state");
        }
        dynAppend(g_diLogging_Thread, startThread("unSystemIntegrity_LHCLogging_IntegrityCheck", sManager, dp_Alarm));
        unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_Logging_SetManagerCheck register", sManager, dp_Alarm, dsArchives, 
                                     g_unSystemIntegrity_LHCLoggingList);
      }
    }
    else
    {
      pos = _unSystemIntegrity_isInList(g_unSystemIntegrity_LHCLoggingList, dp_Alarm);
      if(pos > 0) {
        //Disconnect
        _unSystemIntegrity_registerFunction(bRegister, "unSystemIntegrity_LHCLoggingManagerCB", makeDynString(dpeToCheck_1,dpeToCheck_2), 
              g_unSystemIntegrity_LHCLoggingList, dp_Alarm, exceptionInfoTemp);
        dsArchives = g_m_unSystemIntegrity_LHCLogging_archive[sManager];
        len = dynlen(dsArchives);
        for(i=1;i<=len;i++)
        {
          mappingRemove(g_m_unSystemIntegrity_Archive_state, dsArchives[i]);
          mappingRemove(g_m_unSystemIntegrity_Archive_LastRefStart, dsArchives[i]);
          mappingRemove(g_m_unSystemIntegrity_Archive_LHCLoggingManager, dsArchives[i]);
          sTemp=substr(dsArchives[i],strpos(dsArchives[i],"_ValueArchive_"));
          dpDisconnect("unSystemIntegrity_LHCLoggingArchiveCB", dsArchives[i]+".State.st_StateOut", dsArchives[i]+".State.ev_MessageOut", 
                    dsArchives[i]+".Settings.arch_LastRefStart", sTemp + ".state");
        }
        dynRemove(g_unSystemIntegrity_LHCLoggingManagerList, pos);
        dynRemove(g_unSystemIntegrity_LHCLoggingManagerNumber, pos);
        dynRemove(g_di_unSystemIntegrity_LHCLogging_oldState, pos);
        mappingRemove(g_m_unSystemIntegrity_LHCLogging_state, sManager);
        mappingRemove(g_m_unSystemIntegrity_LHCLogging_archive, sManager);
        stopThread(g_diLogging_Thread[pos]);
        dynRemove(g_diLogging_Thread, pos);
        unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_Logging_SetManagerCheck deregister", sManager, dp_Alarm, dsArchives, 
                                     g_unSystemIntegrity_LHCLoggingList);
      }
    }
  }
}
//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLoggingManagerCB
/** Callback function on the LHCLogging manager state DPE.

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sDpst_StateOut  input, st_StateOut dpe
@param st_StateOut  input, st_StateOut
@param sDpev_MessageOut  input, ev_MessageOut dpe
@param ev_MessageOut  input, ev_MessageOut
*/
unSystemIntegrity_LHCLoggingManagerCB(string sDpst_StateOut, int st_StateOut, string sDpev_MessageOut, int ev_MessageOut)
{
  string sManager = dpSubStr(sDpst_StateOut, DPSUB_DP);
  dyn_int diData;
  int iAlarmState, iWarningState, iPos;
  
  synchronized(g_unSystemIntegrity_LHCLoggingList)
  {
    diData[UN_LHCLOGGING_STATEOUT] = st_StateOut;
    diData[UN_LHCLOGGING_MESSAGEOUT] = ev_MessageOut;
    g_m_unSystemIntegrity_LHCLogging_state[sManager] = diData;
    iPos = dynContains(g_unSystemIntegrity_LHCLoggingManagerList, sManager);
    unSystemIntegrity_LHCLogging_getAlarmState(sManager, iAlarmState, iWarningState);
    if(iAlarmState > 0)
    {
      if(g_di_unSystemIntegrity_LHCLogging_oldState[iPos] != iAlarmState)
      {
        dpSet(g_unSystemIntegrity_LHCLoggingList[iPos]+".alarm", iAlarmState);
        g_di_unSystemIntegrity_LHCLogging_oldState[iPos] = iAlarmState;
        unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLoggingManagerCB set", "sManager:"+sManager, iPos, "al:"+iAlarmState, "W:"+iWarningState);
      }
    }
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLoggingManagerCB", "sManager:"+sManager, "stSta:"+st_StateOut, "evMe:"+ev_MessageOut, mappingHasKey(g_m_unSystemIntegrity_LHCLogging_state, sManager));
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLoggingArchiveCB
/** Callback function on the LHCLogging Archive state DPE and _ValueArchive DPE

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sDpst_StateOut  input, st_StateOut dpe
@param st_StateOut  input, st_StateOut
@param sDpev_MessageOut  input, ev_MessageOut dpe
@param ev_MessageOut  input, ev_MessageOut
@param sDparch_LastRefStart  input, arch_LastRefStart dpe
@param arch_LastRefStart  input, arch_LastRefStart
@param sValarchState  input, _ValueArchive state dpe
@param state  input, _ValueArchive state
*/
unSystemIntegrity_LHCLoggingArchiveCB(string sDpst_StateOut, int st_StateOut, string sDpev_MessageOut, int ev_MessageOut,
                                      string sDparch_LastRefStart, int arch_LastRefStart, string sValarchState, int state)
{
  string sArchive = dpSubStr(sDpst_StateOut, DPSUB_DP);
  dyn_int diData;
  int iAlarmState, iWarningState, iPos;
  string sManager;
  
  synchronized(g_unSystemIntegrity_LHCLoggingList)
  {
    diData[UN_LHCLOGGING_STATEOUT] = st_StateOut;
    diData[UN_LHCLOGGING_MESSAGEOUT] = ev_MessageOut;
    diData[UN_LHCLOGGING_ARCHIVE_STATE] = 20000+state; // in order to use the same search mechanism
    g_m_unSystemIntegrity_Archive_state[sArchive] = diData;
    g_m_unSystemIntegrity_Archive_LastRefStart[sArchive] = arch_LastRefStart;
    sManager = g_m_unSystemIntegrity_Archive_LHCLoggingManager[sArchive];
    iPos = dynContains(g_unSystemIntegrity_LHCLoggingManagerList, sManager);
    unSystemIntegrity_LHCLogging_getAlarmState(sManager, iAlarmState, iWarningState);
    if(iAlarmState > 0)
    {
      if(g_di_unSystemIntegrity_LHCLogging_oldState[iPos] != iAlarmState)
      {
        dpSet(g_unSystemIntegrity_LHCLoggingList[iPos]+".alarm", iAlarmState);
        g_di_unSystemIntegrity_LHCLogging_oldState[iPos] = iAlarmState;
        unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLoggingArchiveCB set", "sManager:"+sManager, iPos, "al:"+iAlarmState, "W:"+iWarningState);
      }
    }
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLoggingArchiveCB", "sArchive:"+sArchive, "stSta:"+st_StateOut, "evMe:"+ev_MessageOut, "lastRe:"+arch_LastRefStart, "VARst:"+state, mappingHasKey(g_m_unSystemIntegrity_Archive_state, sArchive));
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLogging_IntegrityCheck
/** This function sets the systemIntegriy alarm of the LHCLogging manager

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sManager  input, LHCLogging Manager data point name
@param dp_alarm  input, systemIntegrity alarm dp name
*/
unSystemIntegrity_LHCLogging_IntegrityCheck(string sManager, string dp_alarm)
{
  int waitingTime, oldTime = -1;
  int alarmValue, iTime;
  int iAlarmState, iWarningState, iPos;
  int nbWarning, nbAlarm;
  int iNum, iManagerState, iIndex;
  dyn_string exInfo;
  bool bOnlineBackup;
  
  dpGet(sManager+".Settings.man_Num:_online.._stime", oldTime);
  while(true) {
    waitingTime = g_iLogging_ArchiveDelay;
    if(waitingTime <= 0)
      waitingTime = c_systemIntegrityLHCLogging_defaultCheckingDelay;
  // wait
    delay(waitingTime);
    synchronized(g_unSystemIntegrity_LHCLoggingList)
    {
      iPos = dynContains(g_unSystemIntegrity_LHCLoggingManagerList, sManager);
      iNum = g_unSystemIntegrity_LHCLoggingManagerNumber[iPos];
      unSystemIntegrity_LHCLogging_getAlarmState(sManager, iAlarmState, iWarningState);
      alarmValue = c_unSystemIntegrity_no_alarm_value;
      if(iAlarmState > 0) {
        alarmValue = iAlarmState;
        nbWarning = 0;
        nbAlarm++;
      }
      else
      {
        if(iWarningState <= 0)
        {
          dpGet(sManager+".Settings.man_Num:_online.._stime", iTime);
          if(iTime <= oldTime)
          {
            iWarningState = c_unSystemIntegrity_alarm_value_level1;
          }
          oldTime = iTime;
        }
        if(iWarningState > 0)
        {
          nbWarning++;
          if(nbWarning >= g_iLHCLogging_nbAllowedAlarm)
          {
            alarmValue = iWarningState;
            nbAlarm++;
          }
        }
        else {
          nbWarning = 0;
          nbAlarm = 0;
        }
      }
      if(g_di_unSystemIntegrity_LHCLogging_oldState[iPos] != alarmValue)
      {
        dpSet(g_unSystemIntegrity_LHCLoggingList[iPos]+".alarm", alarmValue);
        unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLogging_IntegrityCheck check set alarm", "sManager:"+sManager, iPos, "alarmValue:"+alarmValue, "old:"+g_di_unSystemIntegrity_LHCLogging_oldState[iPos], "alSt:"+iAlarmState, "W:"+iWarningState, "nbWarning:"+nbWarning);
        g_di_unSystemIntegrity_LHCLogging_oldState[iPos] = alarmValue;
      }
      if(nbAlarm >= g_iLHCLogging_nbTimeAlarmBeforeRestart)
      {
        // get the state
        unSystemIntegrity_getState("PVSS00LHCLogging", "-num "+iNum, iManagerState);
        switch(iManagerState) {
          case SYSTEMINTEGRITY_MANAGER_RUNNING:
          case SYSTEMINTEGRITY_MANAGER_BLOCKED:
          case SYSTEMINTEGRITY_MANAGER_INIT:
            // kill the archive manager
//            unSystemIntergity_manageManager("PVSS00LHCLogging", "-num "+iNum, SYSTEMINTEGRITY_KILL_COMMAND, iIndex, exInfo);
//            nbAlarm = 0;
//            nbWarning = 0;
            break;
          default:
            bOnlineBackup=unGenericDpFunctions_isOnlineBackupRunning(getSystemName());
            // restart the archive manager if the online backup is not running
            if(!bOnlineBackup)
              unSystemIntergity_manageManager("PVSS00LHCLogging", "-num "+iNum, SYSTEMINTEGRITY_START_COMMAND, iIndex, exInfo);
//            nbAlarm = 0;
//            nbWarning = 0;
            break;
        }
      }
    }
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLogging_IntegrityCheck check", "sManager:"+sManager, dp_alarm, 
                                 waitingTime, "nbAl:"+nbAlarm, "nbWar:"+nbWarning, "al:"+iAlarmState, "W:"+iWarningState, bOnlineBackup);
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_LHCLogging_getAlarmState
/** Calculate the alarm state and warning state

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sManager  input, LHCLogging Manager data point name
@param iAlarmState output, alarm state
@param iWarningState output, alarm state
*/
unSystemIntegrity_LHCLogging_getAlarmState(string sManager, int &iAlarmState, int &iWarningState)
{
  int iPos, i, len, iManNum;
  dyn_int diData;
  dyn_string dsArchive;
  
  iAlarmState = c_unSystemIntegrity_no_alarm_value;
  iWarningState = c_unSystemIntegrity_no_alarm_value;
  /*
    iAlarmState:
      - manager or one archive evMessageOut: W_CANCEL, W_ARCHNOTEXISTS, W_DPEWRONG, WS_FILENOTEX, A_CONFIGFILE, A_DPEERROR, A_DPCONNERROR, 
      A_DPEWRONG, A_TRLEGACY, A_FILEERR, A_THREADERR
    iWarningState: 
      - any manager evState and messageOut not in the OK list
      - any valarch stopped
      - any manager Archive query too late
      - manager not started
    */
  iPos = dynContains(g_unSystemIntegrity_LHCLoggingManagerList, sManager);
  
  if((iPos > 0) && mappingHasKey(g_m_unSystemIntegrity_LHCLogging_archive, sManager))
  {
    iManNum = g_unSystemIntegrity_LHCLoggingManagerNumber[iPos];
    dsArchive = g_m_unSystemIntegrity_LHCLogging_archive[sManager];
    // evaluate first the iAlarmState
    diData = g_m_unSystemIntegrity_LHCLogging_state[sManager];
    iAlarmState=_unSystemIntegrity_getAlarmState(iAlarmState, g_diAlarmState, diData[UN_LHCLOGGING_STATEOUT]);
    iWarningState=_unSystemIntegrity_getWarningState(iWarningState, g_diOKState, diData[UN_LHCLOGGING_STATEOUT]);
    iAlarmState=_unSystemIntegrity_getAlarmState(iAlarmState, g_diAlarmState, diData[UN_LHCLOGGING_MESSAGEOUT]);
    iWarningState=_unSystemIntegrity_getWarningState(iWarningState, g_diOKState, diData[UN_LHCLOGGING_MESSAGEOUT]);
    iWarningState=_unSystemIntegrity_checkAPINumber(iWarningState, g_diLogging_API, iManNum);
    len = dynlen(dsArchive);
    for(i=1;i<=len;i++)
    {
      diData = g_m_unSystemIntegrity_Archive_state[dsArchive[i]];
      iAlarmState=_unSystemIntegrity_getAlarmState(iAlarmState, g_diAlarmState, diData[UN_LHCLOGGING_STATEOUT]);
      iWarningState=_unSystemIntegrity_getWarningState(iWarningState, g_diOKState, diData[UN_LHCLOGGING_STATEOUT]);
      iAlarmState=_unSystemIntegrity_getAlarmState(iAlarmState, g_diAlarmState, diData[UN_LHCLOGGING_MESSAGEOUT]);
      iWarningState=_unSystemIntegrity_getWarningState(iWarningState, g_diOKState, diData[UN_LHCLOGGING_MESSAGEOUT]);
      iWarningState=_unSystemIntegrity_getWarningState(iWarningState, g_diOKState, diData[UN_LHCLOGGING_ARCHIVE_STATE]);
      iWarningState=_unSystemIntegrity_isTooLate(iWarningState, g_m_unSystemIntegrity_Archive_LastRefStart[dsArchive[i]], g_iLogging_ArchiveTimeout);
    }
  }
    
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "LHCLogging systemIntegrity", "unSystemIntegrity_LHCLogging_getAlarmState", "sManager:"+sManager, "iAlarmState:"+iAlarmState, "iWarningState:"+iWarningState, iPos, mappingHasKey(g_m_unSystemIntegrity_LHCLogging_archive, sManager));
}

//------------------------------------------------------------------------------------------------------------------------
// _unSystemIntegrity_getAlarmState
/** get the alarm state: if iValue is in diValue, return iValue

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param iCurrentValue  input, the current value
@param diValue input, the list of values to look in
@param iValue input, the new value
@return the new value
*/
int _unSystemIntegrity_getAlarmState(int iCurrentValue, dyn_int diValue, int iValue)
{
  int iPos;
  int iReturnValue = iCurrentValue;
  if(iCurrentValue == c_unSystemIntegrity_no_alarm_value)
  {
    iPos = dynContains(diValue, iValue);
    if(iPos > 0)
      iReturnValue = iValue;
  }
//  DebugN("Al", "cur:"+iCurrentValue, "newCu:"+iReturnValue, "val:"+iValue, iPos);
  return iReturnValue;
}

//------------------------------------------------------------------------------------------------------------------------
// _unSystemIntegrity_getWarningState
/** get the warning state: if iValue is not in diValue, return iValue

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param iCurrentValue  input, the current value
@param diValue input, the list of values to look in
@param iValue input, the new value
@return the new value
*/
int _unSystemIntegrity_getWarningState(int iCurrentValue, dyn_int diValue, int iValue)
{
  int iPos;
  int iReturnValue = iCurrentValue;
  
  if(iCurrentValue == c_unSystemIntegrity_no_alarm_value)
  {
    iPos = dynContains(diValue, iValue);
    if(iPos <= 0)
      iReturnValue = iValue;
  }
//  DebugN("War", "cur:"+iCurrentValue, "newCu:"+iReturnValue, "val:"+iValue, iPos);
  return iReturnValue;
}

//------------------------------------------------------------------------------------------------------------------------
// _unSystemIntegrity_isTooLate
/** check if the given time is not too far in the past. if (iTime - iLastTime) > (maxTimeMinute*60) return 3*c_unSystemIntegrity_alarm_value_level1

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param iWarningState  input, the current value
@param iLastTime input, the last time
@param maxTimeMinute input, the new value
@return the result
*/
int _unSystemIntegrity_isTooLate(int iWarningState, int iLastTime, int maxTimeMinute)
{
  int iReturnValue = iWarningState;
  int iMaxTimeSec = maxTimeMinute*60;
  int iTime = getCurrentTime();
  
  if(iWarningState == c_unSystemIntegrity_no_alarm_value)
  {
    if((iTime - iLastTime) > iMaxTimeSec)
      iReturnValue = 3*c_unSystemIntegrity_alarm_value_level1;
  }
//  DebugN("Too late", "cur:"+iWarningState, "newCu:"+iReturnValue, "time:"+iTime, "last:"+iLastTime, "iTime - iLastTime:"+(iTime - iLastTime),"maxsec:"+iMaxTimeSec);
  return iReturnValue;
}

//------------------------------------------------------------------------------------------------------------------------
// _unSystemIntegrity_checkAPINumber
/** check if the manger number is is the list of managers. if iValue is not in diValue return 2*c_unSystemIntegrity_alarm_value_level1

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param iCurrentValue  input, the current value
@param diValue input, the list of values to look in
@param iValue input, the new value
@return the new value
*/
int _unSystemIntegrity_checkAPINumber(int iCurrentValue, dyn_int diValue, int iValue)
{
  int iPos;
  int iReturnValue = iCurrentValue;
  
  if(iCurrentValue == c_unSystemIntegrity_no_alarm_value)
  {
    iPos = dynContains(diValue, iValue);
    if(iPos <= 0)
      iReturnValue = 2*c_unSystemIntegrity_alarm_value_level1;
  }
//  DebugN("API", "cur:"+iCurrentValue, "newCu:"+iReturnValue, "val:"+iValue, iPos);
  return iReturnValue;
}

//------------------------------------------------------------------------------------------------------------------------

//@}


