/**@name LIBRARY: unSystemIntegrity_Laser.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 12/07/2002

Modification History: 

  010/06/2012: Vasileios
  - IS-740 _unSystemIntegrity_LASERCallback function added T_Mode Function.
           const c_unSystemIntegrity_no_alarm_value_T_mode added

  03/10/2011: Herve
  - IS-600  unLHCServices  system name and component version added in the MessageText log and in the diagnostic 
 
  16/06/2011: Herve
  - IS-556: get the list of systemAlarm pattern
  
	29/11/2004: Herve
		move the LASER_pattern into unicosObjects_declarations.ctl

	27/05/2004: Herve
		bug: add 2 manager and remove the last one added->error message of dpDisconnect unSystemIntegrity__Laser_CB. one } missing in the for loop
		and remove one } at the end.

version 1.0

External Functions: 
	
Internal Functions: 
	
Purpose: 
	Library of the Laser component used by the systemIntegrity.
	 	
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. global variable: the following variables are global to the script
	. data point type needed: _UnSystemIntegrity
	. data point: Laser_systemIntegrity
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//------------------
const string UN_SYSTEM_INTEGRITY_Laser = "Laser";
const string c_unSystemIntegrity_LASER_callback = "_unSystemIntegrity_LASERCallback";
const int c_unSystemIntegrity_no_alarm_value_T_mode = 6;  //T mode alarm Value

// 
//------------------

// global declaration
global dyn_string g_unSystemIntegrity_LASERList; // list of the LASER dp checked
//LASER
global dyn_string g_dsLaser_Name;				//Manager List
global dyn_int g_diLaser_Message;				//Message Value for Managers
global dyn_int g_diLaser_PVSS;					//PVSS State in Console for Managers
global dyn_int g_diLaser_Command;				//Command Value for Managers
global dyn_int g_diLaser_State;					//Status Value for Managers
global dyn_int g_diLaser_Period;				//Period to check man_Num						
global dyn_int g_diLaser_Api;					//Api Connection
global dyn_int g_diLaser_T_mode;					//Api Connection
dyn_string g_dsLaser_Thread;					//Thread started
dyn_int g_diOld_alarm_Laser;					//Old Alarm State for Manager

//@{

//------------------------------------------------------------------------------------------------------------------------
// Laser_systemIntegrityInfo
/** Return the list of systemAlarm pattern. 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@return list of systemAlarm pattern
*/
dyn_string Laser_systemIntegrityInfo()
{
  return makeDynString(LASER_pattern);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_Laser_Initialize
/**
Purpose:
Get the list of Laser check by the systemIntegrity. The ones that are enabled.

@param dsResult: dyn_string, output, the list of enabled Laser that are checked

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_Laser_Initialize(dyn_string &dsResult)
{
	dyn_string dsList;
	int len, i;
	string dpToCheck;
	bool enabled;
  string sMessage;
  dyn_string exceptionInfo;
  
  sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("unLHCLASER"));
	
	unCoreLHCServices_initialize(c_Laser_Pattern);
	
// get the list of Laser manager dp to checked
	dsList = dpNames(c_unSystemAlarm_dpPattern+LASER_pattern+"*", c_unSystemAlarm_dpType);
	len = dynlen(dsList);
	for(i = 1; i<=len; i++) {
		dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
		dpToCheck = substr(dpToCheck, strpos(dpToCheck,LASER_pattern));
		dpGet(dsList[i]+".enabled", enabled);
		if(enabled)
			dynAppend(dsResult, dpToCheck);
	}
  unMessageText_send("*", "*", g_SystemIntegrity_sSystemName+"systemIntegrity", "user", "*", "INFO", "systemIntegrity "+sMessage+" loaded", exceptionInfo);
//DebugN(dsResult);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_Laser_HandleCommand
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param command: int, input, the systemIntegrity command
@param sDpe2: string, input, dpe
@param parameters: dyn_string, input, the list of parameters of the systemIntegrity command

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_Laser_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
	dyn_string exceptionInfo;
	int i, len =dynlen(parameters);
  string sMessage;
	
	switch(command) {
		case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_Laser_checking(parameters[i], true, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_Laser_checking(parameters[i], false, false, exceptionInfo);
				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					dpDelete(c_unSystemAlarm_dpPattern+parameters[i]);
			}
			break;
		case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					unSystemIntegrity_Laser_checking(parameters[i], false, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					unSystemIntegrity_Laser_checking(parameters[i], false, false, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
			dpSet(UN_SYSTEM_INTEGRITY_Laser+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
					UN_SYSTEM_INTEGRITY_Laser+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_unSystemIntegrity_LASERList);
			break;
    case UN_SYSTEMINTEGRITY_VERSION:
      sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("unLHCLASER"));
      dpSet(UN_SYSTEM_INTEGRITY_Laser+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
            UN_SYSTEM_INTEGRITY_Laser+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", makeDynString(getCurrentTime(),sMessage));
      break;
		default:
			break;
	}

	if(dynlen(exceptionInfo) > 0) {
		if(isFunctionDefined("unMessageText_sendException")) {
			unMessageText_sendException("*", "*", "unSystemIntegrity_Laser_HandleCommand", "user", "*", exceptionInfo);
		}
// handle any error uin case the send message failed
		if(dynlen(exceptionInfo) > 0) {
			DebugN(getCurrentTime(), exceptionInfo);
		}
	}
//	DebugN(sDpe1, command, parameters, g_unSystemIntegrity_LASERList);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_Laser_checking
/**
Purpose:
This function register/de-register the callback funtion of Laser manager. This function can also create the _unSystemAlarm_Laser dp 
with the alarm config but it cannot delete it.

	@param sDp_Num: string, input, data point name
	@param bCreate: bool, input, true create the dp and the alarm config if it does not exist, enable it
	@param bRegister: bool, input, true do a dpConnect, false do a dpDisconnect
	@param exceptionInfoTemp: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

unSystemIntegrity_Laser_checking(string sDp_Num, bool bCreate, bool bRegister, dyn_string &exceptionInfoTemp)
{
string sSystem, dp_Alarm, description, sManager, dpeToCheck_1, dpeToCheck_11, dpeToCheck_10, dpeToCheck_2, sNum;
int iIndex, iRes, iLaser_Num, iThread, len, j;
bool found = false;

//DebugN("LHCLaser", "_unSystemIntegrity_set_LASER_checking");
	sSystem=getSystemName();
	g_bPanel=false;
	
	//1. Get Alarm Name
	dp_Alarm = c_unSystemAlarm_dpPattern + sDp_Num;
	
	//2. Get Manager Name
	sDp_Num=substr(sDp_Num,strpos(sDp_Num,"_")+1);
	if (strlen(sDp_Num)<2)
		sNum="0" + sDp_Num;
	else
		sNum=sDp_Num;
	
	sManager=LASER_pattern + "Manager_" + sNum;
	
	//3. Get data...
	dpeToCheck_1 = sSystem + sManager + ".State.ev_MessageOut";
 	dpeToCheck_2 = sSystem + sManager + ".State.T_mode";
	dpeToCheck_10 = sSystem + "_Connections.Api.ManNums";
	dpeToCheck_11 = sSystem + sManager + ".Settings.man_NumPeriod";

	if (bRegister)
		{
// check if the dp_alarm is in the global list if not then dpConnect, it is the first time, 
// if yes then it is a second time do not twice dpConnect
// this global variable is set in the _unSystemIntegrity_registerFunction
		iRes = _unSystemIntegrity_isInList(g_unSystemIntegrity_LASERList, dp_Alarm);
		if(iRes <= 0 && dpExists(sSystem + sManager)) 
			{
			if (dynlen(g_unSystemIntegrity_LASERList)==0)			//ONE connect for all Managers
				iRes=dpConnect("unSystemIntegrity__Laser_CB_Api", dpeToCheck_10);
			
			if (iRes>=0)
				{
				//Initialize Global
				if (dynContains(g_dsLaser_Name, sManager)==0)
					{
					dynAppend(g_dsLaser_Name, sManager);
					dynAppend(g_diLaser_Message,0);
					dynAppend(g_diLaser_PVSS,0);
					dynAppend(g_diLaser_Command,0);	
					dynAppend(g_diLaser_State,0);
        dynAppend(g_diLaser_T_mode,0);  
					dynAppend(g_diOld_alarm_Laser,-1);
					dynAppend(g_diLaser_Period,0);
					}
				
				//Connect to Period settings
				iRes=dpConnect("unSystemIntegrity__Laser_CB", dpeToCheck_11);		
				
				//Create the dp_Alarm if it is not existing
				if(!dpExists(dp_Alarm))
					{
					description = getCatStr("unSystemIntegrity", "LASER_DESCRIPTION") + sDp_Num;
					unSystemIntegrity_createSystemAlarm(sDp_Num, LASER_pattern, description, exceptionInfoTemp);
					}
				
				//Connect
				if(dynlen(exceptionInfoTemp)<=0)
					{
					//Start Thread to check Period
					iThread=startThread("unLaser_Integrity", sManager);
					if (iThread>=0)
						dynAppend(g_dsLaser_Thread, sManager + "." + iThread);		
					
					_unSystemIntegrity_registerFunction(bRegister, c_unSystemIntegrity_LASER_callback, makeDynString(dpeToCheck_1, dpeToCheck_2), 
							g_unSystemIntegrity_LASERList, dp_Alarm, exceptionInfoTemp);
					}
				}
			}
		}
	else
		{
// check if the dp_alarm is in the global list if not then do nothing, 
// if yes then d a dpDisconnect
// this global variable is set in the _unSystemIntegrity_Logging_SetArchiveCheck
		iRes = _unSystemIntegrity_isInList(g_unSystemIntegrity_LASERList, dp_Alarm);
		if(iRes > 0) 
			{

			//Stop Thread
			len=dynlen(g_dsLaser_Thread);
			for (j=1;(j<=len) && (found == false);j++)		
				{
				if (patternMatch(sManager + "*", g_dsLaser_Thread[j]))
					{			
					iThread=substr(g_dsLaser_Thread[j],strpos(g_dsLaser_Thread[j],".")+1,strlen(g_dsLaser_Thread[j]));
					if (iThread>0)
						{
						found = true;
						iIndex = j;
//					j=len+1;	//exit loop
						}
					}
				}
			
			if (found)
				{
				iRes=stopThread(iThread);
			
				if (iRes==0)
					dynRemove(g_dsLaser_Thread,iIndex);
				}

			//Disconnect
			iRes=dpDisconnect("unSystemIntegrity__Laser_CB", dpeToCheck_11);
			
			_unSystemIntegrity_registerFunction(bRegister, c_unSystemIntegrity_LASER_callback, makeDynString(dpeToCheck_1), 
						g_unSystemIntegrity_LASERList, dp_Alarm, exceptionInfoTemp);
			
			//Remove Global
			if(dynlen(exceptionInfoTemp)<=0)
				{		
				iIndex=dynContains(g_dsLaser_Name, sManager);
				if (iIndex>0)
					{
					dynRemove(g_dsLaser_Name,iIndex);
					dynRemove(g_diLaser_Message,iIndex);
					dynRemove(g_diLaser_PVSS,iIndex);
					dynRemove(g_diLaser_Command,iIndex);	
					dynRemove(g_diLaser_State,iIndex);
        dynRemove(g_diLaser_T_mode,iIndex);
					dynRemove(g_diOld_alarm_Laser,iIndex);
					dynRemove(g_diLaser_Period,iIndex);
					}
				}
			
			if (dynlen(g_unSystemIntegrity_LASERList)==0)
				iRes=dpDisconnect("unSystemIntegrity__Laser_CB_Api", dpeToCheck_10);
			}
		}
	
}

//------------------------------------------------------------------------------------------------------------------------

// _unSystemIntegrity_LASERCallback
/*
Purpose:
Callback function for the command interface. This function is called when the system alarm is enabled for this device.

	@param sDp1: string, input, data point name
	@param iMessageOut: int, input, Message from Laser

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 2.12.1 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

_unSystemIntegrity_LASERCallback(string sDp1, int iMessageOut, string sDp2, bool Tmode)
{
int iStatus, iPosition, value, iRes, iNum;
string sManager, sNum;
dyn_string exceptionInfo;

//DebugN("LHCLaser", "_unSystemIntegrity_LASERCallback");
	
	sManager = substr(sDp1, strpos(sDp1, c_Laser_Pattern), strlen(c_Laser_Pattern) + 2);
	
	//1. Save State
	unLaser_getPosition(sManager, iPosition);
	
	//2. Gathering Status
	g_diLaser_Message[iPosition]=iMessageOut;
	
	//3. Get State
	unLaser_getStatus(iPosition, iStatus);
  
	//4. Get T mode State
 g_diLaser_T_mode[iPosition] = Tmode;
	
	if (iStatus!=0)	//Error or Warning
		{
		switch(g_diLaser_PVSS[iPosition])
			{
			case 0:									//Manager stopped
				value = c_Stopped;						
				break;
		
			case 1:									//Manager Started
				value = iMessageOut;
				break;
				
			case 2:									//Error with Manager (ie: Blocked)
				value = c_Blocked;		
				break;
			
			default:	
				value = 1300;		
			}
		}	
	else  // Manager OK
  {
    if(g_diLaser_T_mode[iPosition]) //If T-Mode Enabled 
      value = c_unSystemIntegrity_no_alarm_value_T_mode;	 // add 1  
    else
      value = c_unSystemIntegrity_no_alarm_value;	 // add 0
  }

 
  
  if (g_diOld_alarm_Laser[iPosition]!=value)
		{
		g_diOld_alarm_Laser[iPosition]=value;
		
		sNum=substr(sManager, strpos(sManager, c_Laser_Pattern) + strlen(c_Laser_Pattern));
		sscanf(sNum,"%d",iNum);
    
  if(g_diLaser_T_mode[iPosition]) //If T-Mode Enabled 
    value = value*2;  // multiply the value * 2
  						
		if (dpExists(c_unSystemAlarm_dpPattern + LASER_pattern + iNum + ".alarm"))
			iRes=dpSet(c_unSystemAlarm_dpPattern + LASER_pattern + iNum + ".alarm", value);	
		else
			{
			fwException_raise(exceptionInfo, "ERROR", 
						"_unSystemIntegrity_LHCLoggingCallback(): " + getCatStr("unSystemIntegrity", "WRONG_ALARM_DPE") + LASER_pattern + iNum + " value = " + value,"");
						
			if(isFunctionDefined("unMessageText_sendException")) 
				unMessageText_sendException("*", "*", "SystemIntegrity", "user", "*", exceptionInfo);
			}
		}			
}

//@}
